package cn.haliaeetus.sendmessage.bean.delivery;

import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import org.springframework.stereotype.Component;

import java.util.Calendar;

import static cn.haliaeetus.sendmessage.utils._.isNotEmpty;

@Component
class DeliveryMsgShelfChangeFilter implements DeliveryMsgFilter {
    private static final Logger LOG = Logger.getInstance(DeliveryMsgShelfChangeFilter.class);

    /**
     * 派件货架转移短信，若当前时间晚于门店最晚营业时间则不需要发送
     * @param deliveryInfo 派件信息
     * @return true：需要发送，false：不需要发送
     */
    @Override
    public boolean doFilter(DeliveryInfo deliveryInfo) {
        if (!DeliveryInfoUtil.isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        if (!DeliveryInfoUtil.isDeliveryShelfChange(deliveryInfo)) {
            return true;
        }

        // 门店开始营业时间
        String startTime = isNotEmpty(deliveryInfo.getStore().getStartTime()) ? deliveryInfo.getStore().getStartTime() : Constants.STORE_DEFAULT_START_BUSINESS_TIME;
        String[] array = startTime.split(":");
        int startHour = Integer.parseInt(array[0]);
        int startMinute = Integer.parseInt(array[1]);

        // 门店结束营业时间
        String endTime = isNotEmpty(deliveryInfo.getStore().getEndTime()) ? deliveryInfo.getStore().getEndTime() : Constants.STORE_DEFAULT_END_BUSINESS_TIME;
        array = endTime.split(":");
        int endHour = Integer.parseInt(array[0]);
        int endMinute = Integer.parseInt(array[1]);

        // 当前时间
        Calendar now = Calendar.getInstance();
        int nowHour = now.get(Calendar.HOUR_OF_DAY);
        int nowMinute = now.get(Calendar.MINUTE);

        // 早于营业时间
        boolean isEarlier = (nowHour < startHour) || (nowHour == startHour && nowMinute < startMinute);
        // 晚于营业时间
        boolean isLater = (nowHour > endHour) || (nowHour == endHour && nowMinute > endMinute);

        if (isEarlier || isLater) {
            LOG.record(String.format("Msg send filtered by DeliveryMsgShelfChangeFilter, %s.", deliveryInfo));
            return false;
        } else {
            return true;
        }
    }
}