package cn.haliaeetus.sendmessage.bean.delivery;

import cn.haliaeetus.sendmessage.bean.basedata.ExpressType;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.controller.util.APIConfig;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import cn.haliaeetus.sendmessage.framework.util.StringUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.utils._;
import cn.haliaeetus.sendmessage.utils.weixin.WxPushHelper;
import cn.haliaeetus.sendmessage.utils.weixin.WxTool;
import com.google.gson.JsonObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;

import static cn.haliaeetus.sendmessage.utils._.isNotEmpty;

@Component
public class DeliveryWeixinSender implements DeliveryMsgSender {
    private static final Logger LOG = Logger.getInstance(DeliveryWeixinSender.class);

    @Autowired
	private BasicInfoService basicInfoService;

    @Override
    public int send(Delivery delivery, String bussCode, int channel, final String msg, final BigDecimal smsPrice,User user,String string) {
        if (StringUtil.isEmpty(msg)) {
            throw new IllegalArgumentException("Send sms fail, msg is empty.");
        }

        if (isNotEmpty(msg)) {
        	try{
    			String openId = WxTool.getOpenId(delivery);
                if(StringUtil.isNotEmpty(openId)){
                	ExpressType expType = basicInfoService.getExpressById(delivery.getExpId());
    	    		
    	    		JsonObject jo = new JsonObject();
                    jo.addProperty("openid", openId);
                    String textMsg = msg;
                    String url = APIConfig.getWxDeliveryServiceUrl();
					if(_.isNotEmpty(url)) {
						textMsg += "\n<a href=\"" + handleUrl(url, openId) + "\">点击查看物流详情！</a> ";
					}
                    jo.addProperty("content", textMsg);
                    jo.addProperty("templateMsg", WxTool.generateDeliveryInfo(handleUrl(url, openId), msg, null, openId, 
    	    				expType.getExpName(), delivery.getExpcode()));
                    WxPushHelper.pushCustWx(jo.toString());
    	    		LOG.record(String.format("Push weixin msg finish, msg:[%s], url:[%s].", msg, handleUrl(url, openId)));
                }
    		} catch(Exception e) {
    			LOG.record(_.f("Push weixin msg error, storeId:[%s] expId:[%s] expcode:[%s].", delivery.getStoreId(), delivery.getExpId(), delivery.getExpcode()), e);
    		}
        }
        return 0;
    }

	@Override
	public int send(User user, Delivery delivery, String bussCode, int channel, final String msg, final BigDecimal smsPrice) {
		if (StringUtil.isEmpty(msg)) {
			throw new IllegalArgumentException("Send sms fail, msg is empty.");
		}

		if (isNotEmpty(msg)) {
			try{
				String openId = WxTool.getOpenId(delivery);
				if(StringUtil.isNotEmpty(openId)){
					ExpressType expType = basicInfoService.getExpressById(delivery.getExpId());

					JsonObject jo = new JsonObject();
					jo.addProperty("openid", openId);
					String textMsg = msg;
					String url = APIConfig.getWxDeliveryServiceUrl();
					if(_.isNotEmpty(url)) {
						textMsg += "\n<a href=\"" + handleUrl(url, openId) + "\">点击查看物流详情！</a> ";
					}
					jo.addProperty("content", textMsg);
					jo.addProperty("templateMsg", WxTool.generateDeliveryInfo(handleUrl(url, openId), msg, null, openId,
							expType.getExpName(), delivery.getExpcode()));
					WxPushHelper.pushCustWx(jo.toString());
					LOG.record(String.format("Push weixin msg finish, msg:[%s], url:[%s].", msg, handleUrl(url, openId)));
				}
			} catch(Exception e) {
				LOG.record(_.f("Push weixin msg error, storeId:[%s] expId:[%s] expcode:[%s].", delivery.getStoreId(), delivery.getExpId(), delivery.getExpcode()), e);
			}
		}
		return 0;
	}


	/**
	 * 组装快递服务入口链接参数
	 * 
	 * @param url 揽件服务入口链接
	 * @param openId openId
	 * @return
	 */
	private String handleUrl(String url, String openId) {
		return url + "?openid=" + openId;
	}
}