package cn.haliaeetus.sendmessage.bean.dto;

import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.basedata.ExpressType;
import cn.haliaeetus.sendmessage.bean.basedata.StoreShelf;
import cn.haliaeetus.sendmessage.bean.basedata.TypeLibrary;
import cn.haliaeetus.sendmessage.bean.delivery.DeliveryStatus;
import cn.haliaeetus.sendmessage.enums.SMSErrorCodeEnum;
import cn.haliaeetus.sendmessage.framework.toolkit.Jsoned;
import cn.haliaeetus.sendmessage.validate.groups.SendSms;
import com.google.gson.JsonObject;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 派件信息，包括：
 * 基础信息，货架信息，状态信息，快递信息，短信状态，备注信息，滞留天数
 * @author liutianyuan
 *
 */
public class DeliveryDto implements Jsoned {

	@NotNull(message = "不能为空", groups = SendSms.class)
	@Valid
	private Delivery delivery;
	private List<DeliveryStatus> status;
	private StoreShelf shelf;
	private ExpressType express;
	private int strandedDay;
	private int strandedSmsAgain;//滞留件是否当天是否重发短信
	private TypeLibrary typeLibrary;
	private int smsStatus = -1;
	private String bussCode;
	private String resultInfo;//短信失败原因

	private Integer sendSuccessType;//短信通道类型(1.短信发送 2.微信发送 3.短信,微信都发送')

	//排序用 时间
	private Date sortTime;

	public Integer getSendSuccessType() {
		return sendSuccessType;
	}

	public void setSendSuccessType(Integer sendSuccessType) {
		this.sendSuccessType = sendSuccessType;
	}

	public Date getSortTime() {
		return sortTime;
	}

	public void setSortTime(Date sortTime) {
		this.sortTime = sortTime;
	}

	public String getResultInfo() {
		return resultInfo;
	}
	public void setResultInfo(String resultInfo) {
		this.resultInfo = resultInfo;
	}
	public Delivery getDelivery() {
		return delivery;
	}
	public void setDelivery(Delivery delivery) {
		this.delivery = delivery;
	}
	public StoreShelf getShelf() {
		return shelf;
	}
	public void setShelf(StoreShelf shelf) {
		this.shelf = shelf;
	}
	public ExpressType getExpress() {
		return express;
	}
	public void setExpress(ExpressType express) {
		this.express = express;
	}
	public List<DeliveryStatus> getStatus() {
		return status;
	}
	public void setStatus(List<DeliveryStatus> status) {
		this.status = status;
	}
	public int getStrandedDay() {
		return strandedDay;
	}
	public void setStrandedDay(int strandedDay) {
		this.strandedDay = strandedDay;
	}
	public TypeLibrary getTypeLibrary() {
		return typeLibrary;
	}
	public void setTypeLibrary(TypeLibrary typeLibrary) {
		this.typeLibrary = typeLibrary;
	}
	public int getSmsStatus() {
		return smsStatus;
	}
	public void setSmsStatus(int smsStatus) {
		this.smsStatus = smsStatus;
	}
	public String getBussCode() {
		return bussCode;
	}
	public void setBussCode(String bussCode) {
		this.bussCode = bussCode;
	}
	public int getStrandedSmsAgain() {
		return strandedSmsAgain;
	}
	public void setStrandedSmsAgain(int strandedSmsAgain) {
		this.strandedSmsAgain = strandedSmsAgain;
	}

	public String getFailReason() {
    	return SMSErrorCodeEnum.of(this.resultInfo);
    }

    @Override
    public JsonObject toJson() {
        JsonObject jsonObject = new JsonObject();
        jsonObject.add("delivery", delivery.toJson());
        jsonObject.add("shelf", shelf.toJson());
        jsonObject.add("express", express.toJson());
        return jsonObject;
    }
}
