package cn.haliaeetus.sendmessage.constants;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 * 支付宝配置工具类
 *
 */
public class AlipayConfig {
	
	private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle.getBundle("alipay", Locale.CHINA);
	
	// --- 支付宝新老版本通用配置 ---
	/**
	 * 支付宝商户私钥
	 */
	public static String getAlipayPrivateKey() {
		return RESOURCE_BUNDLE.getString("alipay.privateKey");
	}

	/**
	 * 支付宝公钥
	 */
	public static String getAlipayPublicKey() {
		return RESOURCE_BUNDLE.getString("alipay.publicKey");
	}

	/**
	 * 请求使用的编码格式
	 */
	public static String getAlipayCharset() {
		return RESOURCE_BUNDLE.getString("alipay.charset");
	}

	/**
	 * 商户生成签名字符串所使用的签名算法类型
	 */
	public static String getAlipaySignType() {
		return RESOURCE_BUNDLE.getString("alipay.sign.type");
	}

	/**
	 * 支付宝服务器主动通知商户服务器里指定的页面http/https路径
	 */
	public static String getAlipayNotifyurl() {
		return RESOURCE_BUNDLE.getString("alipay.notifyurl");
	}

	/**
	 * 该笔订单允许的最晚付款时间，逾期将关闭交易
	 */
	public static String getAlipayTimeoutExpress() {
		return RESOURCE_BUNDLE.getString("alipay.timeout.express");
	}

	// --- 支付宝老版本配置 ---
	/**
	 * 接口名称
	 */
	public static String getAlipayPayService() {
		return RESOURCE_BUNDLE.getString("alipay.pay.service");
	}

	/**
	 * 签约的支付宝账号对应的支付宝唯一用户号。以2088开头的16位纯数字组成。
	 */
	public static String getAlipayPartner() {
		return RESOURCE_BUNDLE.getString("alipay.partner");
	}
	
	/**
	 * 支付类型
	 */
	public static String getAlipayPaymentType() {
		return RESOURCE_BUNDLE.getString("alipay.payment.type");
	}

	/**
	 * 卖家支付宝账号
	 */
	public static String getAlipaySellerId() {
		return RESOURCE_BUNDLE.getString("alipay.sellerId");
	}

	// --- 支付宝新版本配置 ---
	/**
	 * 支付宝分配给开发者的应用ID
	 */
	public static String getAlipayAppID() {
		return RESOURCE_BUNDLE.getString("alipay.appID");
	}
	
	/**
	 * 商家和支付宝签约的产品码
	 */
	public static String getAlipayProductCode() {
		return RESOURCE_BUNDLE.getString("alipay.product.code");
	}
	
	/**
	 * 接口名称2.0
	 */
	public static String getAlipayPayMethod() {
		return RESOURCE_BUNDLE.getString("alipay.pay.method");
	}
	
	/**
	 * 调用的接口版本，固定为：1.0
	 */
	public static String getAlipayVersion() {
		return RESOURCE_BUNDLE.getString("alipay.version");
	}

	/**
	 *
	 * @return 支付主题
	 */
	public static String getAlipaySubject() {
		return RESOURCE_BUNDLE.getString("alipay.subject");
	}

}