package cn.haliaeetus.sendmessage.controller;

import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveBean;
import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveRequest;
import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveResponse;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.utils.MD5Util;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * @PackageName: cn.haliaeetus.sendmessage.controller
 * @ClassName: ExpSignBackController
 * @Description: 快递签收回传
 * @Author: liujs
 * @Date: 2020/02/26 下午 16:48
 * @Version: 1.0
 */
@Controller
@RequestMapping(value = "/signBack")
@Slf4j
public class ExpSignBackController {

    @Autowired
    private DeliveryService deliveryService;

    @Autowired
    private PlatformConfigService platformConfigService;

    @RequestMapping(value = "/cainiao/receive")
    @ResponseBody
    public CaiNiaoReceiveResponse caiNiaoReceive(@RequestBody CaiNiaoReceiveRequest request) throws Exception {
        log.info("【菜鸟签收回传回执入参：】{}", request);
        PlatformConfig platformConfig = platformConfigService.getByPlatform("CaiNiao");
        // 签名
        String md5 = MD5Util.caiNiaoDoSign(request.getLogisticsInterface(),"UTF-8",platformConfig.getAppSecret());
        if (!md5.equals(request.getDataGigest())) {
            return CaiNiaoReceiveResponse.builder()
                    .success(false)
                    .errorCode("100")
                    .errorMsg("签名错误")
                    .build();
        }

        try {
            CaiNiaoReceiveBean caiNiaoReceiveBean = new Gson().fromJson(request.getLogisticsInterface(),CaiNiaoReceiveBean.class);
            if(null != caiNiaoReceiveBean){
                if (!("E000").equals(caiNiaoReceiveBean.getErrorCode())){
                    String reason = caiNiaoReceiveBean.getErrorCode().length() > 50 ? caiNiaoReceiveBean.getErrorCode().substring(0,50) : caiNiaoReceiveBean.getErrorCode();
                    deliveryService.updateExpSignBackStatus(caiNiaoReceiveBean.getMailNo(),caiNiaoReceiveBean.getSiteId(),reason);
                }
                return CaiNiaoReceiveResponse.builder()
                        .success(true)
                        .errorCode("0")
                        .errorMsg("接收成功")
                        .build();
            }
        }catch (Exception e){
            e.printStackTrace();
            return CaiNiaoReceiveResponse.builder()
                    .success(false)
                    .errorCode("100")
                    .errorMsg("系统错误")
                    .build();
        }
        return CaiNiaoReceiveResponse.builder()
                .success(false)
                .errorCode("100")
                .errorMsg("接收失败")
                .build();
    }
}
