package cn.haliaeetus.sendmessage.controller;

import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveBean;
import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveRequest;
import cn.haliaeetus.sendmessage.bean.cainiao.CaiNiaoReceiveResponse;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.sms.ResendSmsRequest;
import cn.haliaeetus.sendmessage.bean.sms.UpSmsInfo;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.MD5Util;
import cn.haliaeetus.sendmessage.utils.Zhongtong.ZtoReceiveRequest;
import cn.haliaeetus.sendmessage.utils.Zhongtong.ZtoResponse;
import cn.haliaeetus.sendmessage.utils.pdd.PddSmsReceiptRequest;
import cn.haliaeetus.sendmessage.utils.pdd.PddSmsReceiptResponse;
import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.pdd.pop.sdk.common.constant.Constants;
import com.pdd.pop.sdk.common.util.SignUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName SmsController
 * @Description 短信
 * @Author kael
 * @Date 2019/9/29 0029 下午 16:21
 * @Version 1.0
 **/
@Controller
@RequestMapping(value = "/sms")
@Slf4j
public class SmsController {
    @Autowired
    private SmsRecordService smsRecordService;

    @Autowired
    private PlatformConfigService platformConfigService;

    @Autowired
    private DeliveryService deliveryService;

    @Autowired
    private BasicInfoService basicInfoService;

    @RequestMapping(value = "/updateSmsRecord")
    @ResponseBody
    public PddSmsReceiptResponse updateSmsRecord(@RequestBody PddSmsReceiptRequest request) throws Exception {
        log.info("【PDD短信回执入参：】{}", request);
        PlatformConfig platformConfig = platformConfigService.getByPlatform("PDD");
        Map<String, String> returnMap = BeanUtils.describe(request);
        returnMap.remove("class");
        returnMap.remove("sign");
        returnMap = returnMap.entrySet().stream().filter(entry -> !Strings.isNullOrEmpty(entry.getValue())).collect(Collectors.toMap(
                (e) -> (String) e.getKey(),
                (e) -> e.getValue()
        ));

        // 签名原串
        String signSrc = SignUtils.buildSignSource(returnMap, platformConfig.getAppSecret());
        // 签名
        String md5 = SignUtils.sign(signSrc, Constants.MD5);
        if (!md5.equals(request.getSign())) {
            return PddSmsReceiptResponse.builder()
                    .code(1)
                    .msg("签名错误")
                    .sub_code("sign-check-failure")
                    .build();
        }

        return updateSms(request);

    }

    @RequestMapping(value = "/resendSms")
    @ResponseBody
    public boolean resendSms(@RequestBody ResendSmsRequest request) {
        boolean fale = true;
        try {
            if(Strings.isNullOrEmpty(request.getPlatform())){
                request.setPlatform(basicInfoService.getConfigurationByCode("spareSmsPlatform").getOValue1());
            }
            deliveryService.resendSms(request.getStoreId(),request.getIds(),request.getPlatform());
        } catch (Exception e){
            fale = false;
            e.printStackTrace();
        }
        return fale;
    }
    @RequestMapping(value = "/cainiao/receive")
    @ResponseBody
    public CaiNiaoReceiveResponse caiNiaoReceive(@RequestBody CaiNiaoReceiveRequest request) throws Exception {
        log.info("【菜鸟短信回执入参：】{}", request);
        PlatformConfig platformConfig = platformConfigService.getByPlatform("CaiNiao");

        // 签名
        String md5 = MD5Util.caiNiaoDoSign(request.getLogisticsInterface(),"UTF-8",platformConfig.getAppSecret());
        if (!md5.equals(request.getDataGigest())) {
            return CaiNiaoReceiveResponse.builder()
                    .success(false)
                    .errorCode("100")
                    .errorMsg("签名错误")
                    .build();
        }
        try {
            CaiNiaoReceiveBean caiNiaoReceiveBean = new Gson().fromJson(request.getLogisticsInterface(),CaiNiaoReceiveBean.class);
            smsRecordService.upSmsMsg(UpSmsInfo.builder()
                    .bizId(caiNiaoReceiveBean.getBizId())
                    .errorCode(caiNiaoReceiveBean.getErrCode())
                    .errorMsg(caiNiaoReceiveBean.getErrorMsg())
                    .expCode(caiNiaoReceiveBean.getReceiverNumber())
                    .mobile(caiNiaoReceiveBean.getPrivateNumber())
                    .platform("CaiNiao")
                    .receiptTime(caiNiaoReceiveBean.getReptTime())
                    .sendTime(caiNiaoReceiveBean.getSendTime())
                    .smsSize(1)
                    .success("DELIVRD".equals(caiNiaoReceiveBean.getErrCode()))
                    .build());
            return CaiNiaoReceiveResponse.builder()
                    .success(true)
                    .errorCode("0")
                    .errorMsg("接收成功")
                    .build();
        }catch (Exception e){
            e.printStackTrace();
            return CaiNiaoReceiveResponse.builder()
                    .success(false)
                    .errorCode("100")
                    .errorMsg("系统错误")
                    .build();
        }
    }
    private PddSmsReceiptResponse updateSms(PddSmsReceiptRequest request) {
        try {
            smsRecordService.upSmsMsg(UpSmsInfo.builder()
                    .bizId(request.getBiz_id())
                    .errorCode(request.getError_code())
                    .errorMsg(request.getError_msg())
                    .expCode(request.getWaybill_code())
                    .mobile(request.getMobile())
                    .platform("PDD")
                    .receiptTime(request.getReceipt_time())
                    .sendTime(request.getSend_time())
                    .smsSize(request.getSms_size())
                    .success(request.getSuccess())
                    .build());
            return PddSmsReceiptResponse.builder()
                    .code(200)
                    .msg("接收成功")
                    .build();
        }catch (Exception e){
            e.printStackTrace();
            return PddSmsReceiptResponse.builder()
                    .code(1)
                    .msg("系统错误")
                    .sub_code("system-failure")
                    .build();
        }

    }

    @RequestMapping(value = "/zto/receive")
    @ResponseBody
    public ZtoResponse ztoReceive(@RequestBody ZtoReceiveRequest request) throws Exception {
        log.info("【中通短信回执入参：】{}", request);
//        PlatformConfig platformConfig = platformConfigService.getByPlatform("ZTO");
        try {
            String nowDate = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now());
            smsRecordService.upSmsMsg(UpSmsInfo.builder()
                    .bizId(request.getBillCode())
                    .errorCode(request.getStatus().toString())
                    .errorMsg(request.getMessage())
                    .expCode(request.getBillCode())
//                    .mobile()
                    .platform("ZTO")
                    .receiptTime(nowDate)
                    .sendTime(nowDate)
                    .smsSize(1)
                    .success(request.getStatus().compareTo(1)==0)
                    .build());
            ZtoResponse response = new ZtoResponse();
            response.setMessage("请求成功");
            response.setStatusCode("200");
            response.setStatus(true);
            return response;
        }catch (Exception e){
            e.printStackTrace();
            ZtoResponse response = new ZtoResponse();
            response.setMessage("请求失败");
            response.setStatusCode("-1");
            response.setStatus(false);
            return response;
        }
    }


}
