package cn.haliaeetus.sendmessage.dao;

import cn.haliaeetus.sendmessage.bean.CompletePic;
import cn.haliaeetus.sendmessage.bean.delivery.DeliveryStatus;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.DeliveryStatusQueryCondition;
import cn.haliaeetus.sendmessage.framework.mybatis.persistence.BaseMybatisDAO;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 
 * @author liutianyuan
 *
 */

@Repository
public class DeliveryStatusDao extends BaseMybatisDAO {
	
	private static final String MAPPER_NAMESPACE = "cn.haliaeetus.sendmessage.dao.delivery.mapper.DeliveryStatusMapper";
	

	/**
	 * 插入派件状态信息，返回插入对象主键
	 * @param entity
	 * @return
	 */
	public int insert(DeliveryStatus entity){
		return save(MAPPER_NAMESPACE + ".insert", entity);
	} 
	

	/**
	 * 查询派件状态信息
	 * qc中的各个字段生成where条件，包含order by  和  limit。
	 * @param qc
	 * @return
	 */
	public List<DeliveryStatus> select(DeliveryStatusQueryCondition qc){
		return findAll(MAPPER_NAMESPACE + ".select", qc);
	}

	/**
	 * 查询某派件最早入库的状态
	 * @param deliveryId 派件ID
	 */
	public DeliveryStatus getEarliestStatusOfDelivery(int storeId, int deliveryId, int status) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("deliveryId", deliveryId);
		params.put("status", status);
		return findOne(MAPPER_NAMESPACE + ".selectEarliestStatusOfDelivery", params);
	}


	public List<DeliveryStatus> getStatusListByDeliveryId(int storeId,
			int deliveryId) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("deliveryId", deliveryId);
		return findAll(MAPPER_NAMESPACE + ".selectListByDeliveryId", params);
	}


	public int saveCompletePic(CompletePic pic) {
		// TODO Auto-generated method stub
		return save(MAPPER_NAMESPACE + ".saveCompletePic", pic);
	}
	
	public CompletePic getCompletePic(CompletePic pic) {
		// TODO Auto-generated method stub
		return findOne(MAPPER_NAMESPACE + ".getCompletePic", pic);
	}

	public void deleteByDeliveryId(int deliveryId,int storeId){
		Map<String, Object> params = new HashMap<>();
		params.put("deliveryId", deliveryId);
		params.put("storeId", storeId);
		delete(MAPPER_NAMESPACE + ".deleteByDeliveryId", params);
	}
}