package cn.haliaeetus.sendmessage.dao;

import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.mengwang.MengWangSmsStatusBean;
import cn.haliaeetus.sendmessage.bean.sms.SmsPurchaseRecord;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.framework.mybatis.persistence.BaseMybatisDAO;
import com.google.common.base.Strings;
import org.springframework.stereotype.Repository;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 短信发送记录DAO
 */
@Repository
public class SmsRecordDao extends BaseMybatisDAO {
	private static final String MAPPER_NAMESPACE = "cn.haliaeetus.sendmessage.dao.mapper.SmsRecordMapper";

	/**
	 * 插入
	 * @param entity
	 * @return
	 */

	public int insert(final SmsRecord entity){
		return save(MAPPER_NAMESPACE+".insert", entity);
	}
	
	public int update(final SmsRecord entity){
		return update(MAPPER_NAMESPACE+".updateByPrimaryKeySelective", entity);
	}
	
	public SmsRecord findByIsmgid(final String ismgid){
		return findOne(MAPPER_NAMESPACE+".selectByIsmgid",ismgid);
	}
	
	public SmsRecord findById(final int id){
		return findOne(MAPPER_NAMESPACE+".selectByPrimaryKey",id);
	}

	public List<SmsRecord> getFailSmsRecordsOfStoreToday(int storeId, int failStatus, int unknownStatus, String beginTime, String endTime) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("failStatus", failStatus);
		params.put("unknownStatus", unknownStatus);
		params.put("beginTime", beginTime);
		params.put("endTime", endTime);

		return findAll(MAPPER_NAMESPACE + ".selectFailSmsRecordsOfStoreToday", params);
	}

	public long countByStoreIdAndStatusAndTime(int storeId, Integer status, String beginTime, String endTime) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		if (status != null) {
			params.put("status", status);
		}
		params.put("beginTime", beginTime);
		params.put("endTime", endTime);
		
		return count(MAPPER_NAMESPACE + ".countByStoreIdAndStatusAndTime", params);
	}
	
	/**
	 * 查询某条订单当天滞留件是否发送短信
	 * @param storeId
	 * @param today
	 * @param deliveryId
	 * @param type
	 * @author zhang-shuai
	 * @return
	 */
	public long getTodayStandedSmsCount(int storeId, String today, int deliveryId, String type) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("today", today);
		params.put("type", type);
		params.put("deliveryid", deliveryId);
		return count(MAPPER_NAMESPACE + ".countTodayStrandedSms", params);
	}
	
	public List<SmsRecord> getTodayStandedSms(int storeId, String today, String deliveryIds, String type) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("today", today);
		params.put("type", type);
		params.put("deliveryIds", deliveryIds);
		return findAll(MAPPER_NAMESPACE + ".selectTodayStrandedSms", params);
	}
	
	/**
	 * 查询门店短信记录   liutianyuan 
	 */
	public List<SmsRecord> selectSmsRecordsOfStore(int storeId,String deliveryIds) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("deliveryIds", deliveryIds);

		return findAll(MAPPER_NAMESPACE + ".selectSmsRecordsOfStore", params);
	}

	public SmsRecord getSmsRecordByMsgid(String msgid) {
		Map<String, Object> params = new HashMap<>();
		params.put("ismgid", msgid);
		return findOne(MAPPER_NAMESPACE + ".selectSmsRecordByMsgid", params);
	}

	public SmsRecord getSmsRecord(SmsRecord smsRecord) {
		// TODO Auto-generated method stub
		return findOne(MAPPER_NAMESPACE + ".getSmsRecord", smsRecord);
	}
	
	public List<SmsRecord> getSendSmsList(Map<String, Object> params) {
		return findAll(MAPPER_NAMESPACE + ".getSendSmsList", params);
	}

	public int savePurchaserecord(SmsPurchaseRecord entity){
		return save(MAPPER_NAMESPACE+".savePurchaserecord", entity);
	}

	public int updatePurchaseRecord(final SmsPurchaseRecord entity){
		return update(MAPPER_NAMESPACE+".updatePurchaseRecord", entity);
	}

	public SmsPurchaseRecord getRecordByOutTradeNo(SmsPurchaseRecord entity) {
		// TODO Auto-generated method stub
		return findOne(MAPPER_NAMESPACE + ".getRecordByOutTradeNo", entity);
	}

	public List<SmsRecord> getSmsCenterList(Map<String, Object> params) {
		return findAll(MAPPER_NAMESPACE + ".getSmsCenterList", params);
	}

	public SmsRecord getOneSmsRecord(int storeId,int deliveryId) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("deliveryId", deliveryId);
		return findOne(MAPPER_NAMESPACE + ".selectOneSmsRecord", params);
	}
	public int updateSmsStatusById(final Delivery entity){
		return update(MAPPER_NAMESPACE+".updateSmsStatusById", entity);
	}

	public Integer updateSmsRecordByBizId(String resultCode,String msgId,int status,String donetime,Integer smsCount) throws Exception{
		SmsRecord entity=new SmsRecord();
		entity.setIsmgid(msgId);
		entity.setResultInfo(resultCode);
		entity.setStatus(status);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date date1 = null;
		if(!Strings.isNullOrEmpty(donetime)){
			date1=sdf.parse(donetime);
		}else{
			date1 = new Date();
		}
		entity.setFinishtime(date1);
		return update(MAPPER_NAMESPACE+".updateSmsRecordByBizId", entity);
	}

	public Integer updateSmsRecordByBizId(String resultCode,String msgId,int status,String donetime) throws Exception{
		return updateSmsRecordByBizId(resultCode,msgId,status,donetime,null);
	}

	public int updateSmsRecord(String phoneNumber,String resultCode,String msgId,String status,String donetime)throws Exception{
		SmsRecord entity=new SmsRecord();
		entity.setIsmgid(msgId);
		entity.setResultInfo(resultCode);
		entity.setTelphone(phoneNumber);
		entity.setStatus(Integer.valueOf(status));
		SimpleDateFormat sdf1=new SimpleDateFormat("yyyyMMddhhmmss");
		Date date1 = null;
		if(!Strings.isNullOrEmpty(donetime)){
			date1=sdf1.parse(donetime);
		}else{
			date1 = new Date();
		}
		entity.setFinishtime(date1);
		return update(MAPPER_NAMESPACE+".updateSmsRecord", entity);
	}

	public void addMengWangSmsStatusInfo(List<MengWangSmsStatusBean> mengWangSmsStatusBeanList) {
		saveBatch(MAPPER_NAMESPACE + ".addMengWangSmsStatusInfo", mengWangSmsStatusBeanList);
	}
}