package cn.haliaeetus.sendmessage.dao;

import cn.haliaeetus.sendmessage.bean.basedata.StoreShelf;
import cn.haliaeetus.sendmessage.bean.dto.StoreShelfDto;
import cn.haliaeetus.sendmessage.framework.mybatis.persistence.BaseMybatisDAO;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 货架管理
 * @author zhang-shuai
 *
 */

@Repository
public class StoreShelfDao extends BaseMybatisDAO {
	
	private static final String MAPPER_NAMESPACE = "cn.haliaeetus.sendmessage.dao.mapper.StoreShelfMapper.";

/* ****************************插入*************************** 	*/	
	/**
	 * 选择性添加，不需要更新字段设空null
	 * @param entity
	 * @return
	 */
	public int insert(final StoreShelf entity){
		return save(MAPPER_NAMESPACE+"insertSelective", entity);
	}
	
/* ****************************更新*************************** 	*/	
	/**
	 * 根据主键，更新所有数据
	 * @param entity
	 * @return
	 */
	public int update(final StoreShelf entity){
		return update(MAPPER_NAMESPACE+"updateByPrimaryKey", entity);
	}
	
	/**
	 * 根据主键，可选数据更新,不需要更新字段设空null
	 * @param entity
	 * @return
	 */
	public int updateSelective(final StoreShelf entity){
		return update(MAPPER_NAMESPACE+"updateByPrimaryKeySelective", entity);
	}
	
	
/* ****************************查询*************************** 	*/		
	/**
	 * 根据主键查询
	 * @param id
	 * @return
	 */
	public StoreShelf getById(final int id){
		return findOne(MAPPER_NAMESPACE+"selectByPrimaryKey", id);
	}
	/**
	 * 查询所有数据
	 * @return
	 */
	public List<StoreShelf> getAll(){
		return findAll(MAPPER_NAMESPACE+"selectAll");
	}


    public StoreShelf getMaxBarCode(final int storeId){
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("storeId", storeId);
        return findOne(MAPPER_NAMESPACE+"getMaxBarCode",map);
    }

    public StoreShelf getMinBarCodeIsdel(final int storeId){
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("storeId", storeId);
        return findOne(MAPPER_NAMESPACE + "getMinBarCodeIsdel",map);
    }


	/**
	 * 查询storeId关联所有数据2019
	 * @return list
	 */
	public List<StoreShelf> getListByStoreId(final int storeId,final int p,final int pageSize){
		Map<String, Object> map = new HashMap<String, Object>();
		int begin = (p - 1) * pageSize;
		map.put("begin", begin);
		map.put("pageSize", pageSize);
		map.put("storeId", storeId);
		return findAll(MAPPER_NAMESPACE+"selectByStoreId",map);
	}
	
	/**
	 * 查询storeId关联所有数据,带包裹数；按时间降序；pageSize>0时，分页
	 * @return list
	 */
	public List<StoreShelfDto> getDtoTimeListByStoreId(final int storeId, final int p, final int pageSize, List<Integer> statusList){
		Map<String, Object> map = new HashMap<String, Object>();
		String statusStr = "";
		if(statusList != null && statusList.size() > 0){
			for(Integer status : statusList){
				statusStr += "," + status;
			}
			statusStr = statusStr.substring(1);
		}else{
			statusStr = null;
		}
		map.put("statusStr", statusStr);
		int begin = (p - 1) * pageSize;
		map.put("begin", begin);
		map.put("pageSize", pageSize);
		map.put("storeId", storeId);
		return findAll(MAPPER_NAMESPACE+"selectDtoTimeByStoreId",map);
	}
	
	/**
	 * 查询storeId关联所有数据,带包裹数；按包裹数降序;pageSize>0时，分页
	 * @return list
	 */
	public List<StoreShelfDto> getDtoNumListByStoreId(final int storeId,final int p,final int pageSize,List<Integer> statusList){
		Map<String, Object> map = new HashMap<String, Object>();
		String statusStr = "";
		if(statusList != null && statusList.size() > 0){
			for(Integer status : statusList){
				statusStr += "," + status;
			}
			statusStr = statusStr.substring(1);
		}else{
			statusStr = null;
		}
		map.put("statusStr", statusStr);
		int begin = (p - 1) * pageSize;
		map.put("begin", begin);
		map.put("pageSize", pageSize);
		map.put("storeId", storeId);
		
		return findAll(MAPPER_NAMESPACE+"selectDtoNumByStoreId",map);
	}
	
	/**
	 * 查询storeId、shelfName关联所有数据
	 * @return list
	 */
	public List<StoreShelf> getListByStoreIdAndShelfName(final int storeId,final String shelfName){
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("storeId", storeId);
		map.put("shelfName", shelfName);
		return findAll(MAPPER_NAMESPACE+"selectByShelfName",map);
	}
	
	/**
	 * 查询storeId、shelfName关联所有数据,且不等于id
	 * @return list
	 */
	public List<StoreShelf> getListByStoreIdAndShelfName(final int id,final int storeId,final String shelfName){
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("id", id);
		map.put("storeId", storeId);
		map.put("shelfName", shelfName);
		return findAll(MAPPER_NAMESPACE+"selectByShelfNameNotId",map);
	}
	
	/**
	 * 查询storeId关联所有数据,且是已删除货架
	 * @return list
	 */
	public List<StoreShelf> getListDelByStoreId(final int storeId){
		return findAll(MAPPER_NAMESPACE+"selectDelByStoreId",storeId);
	}
	
	/**
	 * 查询storeId关联所有数据
	 * @return map
	 */
	public Map<Integer, StoreShelf> getByMapStoreId(final int storeId){
		Map<String, Object> mapParams = new HashMap<String, Object>();
		mapParams.put("begin", 0);
		mapParams.put("pageSize", 0);
		mapParams.put("storeId", storeId);
		List<StoreShelf> shelfs = findAll(MAPPER_NAMESPACE+"selectByStoreId",mapParams);
		Map<Integer, StoreShelf> map = new HashMap<Integer, StoreShelf>();
		for(StoreShelf shelf : shelfs){
			map.put(shelf.getId(), shelf);
		}
		return map;
	}
	
/* ****************************删除*************************** 	*/	
	/**
	 * 根据主键删除，软删除
	 * @param id
	 * @return
	 */
	public int delete(final int id){
        return delete(MAPPER_NAMESPACE+"delete", id);
	}

	/**
	 * 根据主键删除，删除
	 * @param id
	 * @return
	 */
	public int deleteReal(final int id){
		return delete(MAPPER_NAMESPACE+"deleteReal", id);
	}

/* ****************************统计*************************** 	*/	
	
	public long count(final int storeId){
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("storeId", storeId);
		return count(MAPPER_NAMESPACE+"count",map);
	}
	
	
/* ****************************废弃*************************** 	*/		
    public StoreShelf getMaxStoreShelf(int storeId) {
        return findOne(MAPPER_NAMESPACE + "getMaxStoreShelf", storeId);
    }

    public StoreShelf getMinFromDel(int storeId) {
        return findOne(MAPPER_NAMESPACE + "getMinFromDel", storeId);
    }

	public List<StoreShelf> getListByBarcode(int storeId, String barCode) {
		Map<String, Object> mapParams = new HashMap<String, Object>();
		mapParams.put("storeId", storeId);
		mapParams.put("barCode", barCode);
		return findAll(MAPPER_NAMESPACE+"getListByBarcode",mapParams);
	}

	//设为默认货架
	public void setIsDefault(StoreShelfDto storeShelfDto){
		 update(MAPPER_NAMESPACE+"setIsDefault", storeShelfDto);
	}

	public void setIsDefaultAll(StoreShelfDto storeShelfDto){
		update(MAPPER_NAMESPACE+"setIsDefaultAll", storeShelfDto);
	}

	//根据expId获取shelfName
	public String getByexpId(String epxId) {
    	return getExpId(MAPPER_NAMESPACE+"getByexpId", epxId);
	}

	


}

