package cn.haliaeetus.sendmessage.dao;

import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.UserQueryCondition;
import cn.haliaeetus.sendmessage.framework.mybatis.persistence.BaseMybatisDAO;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 用户管理
 * 
 * @author liutianyuan
 *
 */
@Repository
public class UserDao extends BaseMybatisDAO {
	private static final String MAPPER_NAMESPACE = "cn.haliaeetus.sendmessage.dao.mapper.UserMapper";

	/**
	 * 查询所有用户
	 * 
	 * @return
	 */
	public List<User> getAllUser() {
		return findAll(MAPPER_NAMESPACE + ".selectAll");
	}

	/**
	 * 根据ID和密码查询用户
	 * 
	 * @param userId
	 *            必填
	 * @param password
	 *            不需要时传null
	 * @return
	 */
	public User getUserByIdAndPwd(String userId, String password) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		params.put("password", password);
		return findOne(MAPPER_NAMESPACE + ".getUserByIdAndPwd", params);
	}

	/**
	 * 根据ID和密码查询用户
	 * 
	 * @param userId
	 *            必填
	 * @param storepwd
	 * @return
	 */
	public User getUserByIdAndStorePwd(String userId, String storepwd) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		params.put("storepwd", storepwd);
		return findOne(MAPPER_NAMESPACE + ".getUserByIdAndStorePwd", params);
	}

	/**
	 * 根据ID和密码查询用户
	 * 
	 * @param userId
	 *            必填
	 * @param password
	 *            不需要时传null
	 * @return
	 */
	public int update(UserQueryCondition qc) {
		return update(MAPPER_NAMESPACE + ".update", qc);
	}

	public User getUserById(String userId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		return findOne(MAPPER_NAMESPACE + ".getUserByuserId", params);
	}

	public void save(User user) {
		save(MAPPER_NAMESPACE + ".insertSelective", user);
	}

	/**
	 * delivery_status建表
	 * @return
	 */
	public void createTableDeliveryStatus(String userId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".createTableDeliveryStatus",params);

	}
	/**
	 * deliver建表
	 * @return
	 */
	public void createTableDelivery(String userId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".createTableDelivery",params);
	}
	
	/**
	 * 创建update触发器
	 * @param userId
	 */
	public void createDeliveryUpdateTrigger(String userId){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".createDeliveryUpdateTrigger",params);
	}
	/**
	 * 创建insert触发器
	 * @param userId
	 */
	public void createDeliveryInsertTrigger(String userId){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".createDeliveryInsertTrigger",params);
	}
	
	
	public long findCountByTableName(String userId){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		long count=count(MAPPER_NAMESPACE+".findCountByTableName", params);
		return count;
	}
	
	public void insertDelivery(String userId){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".insert_delivery",params);
	}
	
	public void insertDeliveryStatus(String userId){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		update(MAPPER_NAMESPACE+".insert_delivery_status",params);
	}
	
	
	public void deleteMsgTemplate(String ids){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("ids", ids);
		update(MAPPER_NAMESPACE+".deleteMsgTemplate",params);
	}

	public void insertMsgTemplate(int store_id,int client_type,String buss_code,String content){
		//store_id,client_type,type,buss_code,content
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("store_id", store_id);
		params.put("client_type", client_type);
		params.put("buss_code", buss_code);
		params.put("content", content);
		update(MAPPER_NAMESPACE+".insertMsgTemplate",params);
	}
	
	
	public void updateMsgTemplate(int store_id,String buss_code,String content){
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("store_id", store_id);
		params.put("buss_code", buss_code);
		params.put("content", content);
		update(MAPPER_NAMESPACE+".updateMsgTemplate",params);
	}
	
	public List<User> findMsgTemplate(){
			
		List<User> userList=findAll(MAPPER_NAMESPACE+".selectMsgTemplate");
		return userList;
	}
	
	/**
	 * 扣减余额
	 * @param store
	 * @return
	 */
	public int deductBalance(BigDecimal consume, String userId){
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("consume", consume);
		param.put("userId", userId);
		return update(MAPPER_NAMESPACE + ".deductBalance", param);
	}
	
	/**
	 * 充值余额
	 * @return
	 */
	public int addBalance(BigDecimal money, String userId){
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("money", money);
		param.put("userId", userId);
		return update(MAPPER_NAMESPACE + ".addBalance", param);
	} 
	
	/**
	 * 更新短信配置
	 * @return
	 */
	public int updateSmsPriceConf(String userId, int isSmsFree, String smsPrice){
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("userId", userId);
		param.put("isSmsFree", isSmsFree);
		param.put("smsPrice", smsPrice);
		return update(MAPPER_NAMESPACE + ".updateSmsPriceConf", param);
	}

	public User getUserByuserId(String userId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		return findOne(MAPPER_NAMESPACE + ".getUserByuserId", params);
	}

	public int updateOcrPriceConf(String userId, int isOrcFree, String ocrPrice){
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("userId", userId);
		param.put("isOrcFree", isOrcFree);
		param.put("ocrPrice", ocrPrice);
		return update(MAPPER_NAMESPACE + ".updateOcrPriceConf", param);
	}

	public int updateVigilance(String userId, String vigilance){
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("userId", userId);
		param.put("vigilance", vigilance);
		return update(MAPPER_NAMESPACE + ".updateVigilance", param);
	}

	public List<User> getUsers(UserQueryCondition qc) {

		List<User> userList=findAll(MAPPER_NAMESPACE+".getUsers",qc);
		return userList;
	}

	public Integer updateUserStatusEnable(Map map){
		return update(MAPPER_NAMESPACE+".updateUserStatusEnable",map);
	}

	public User selectByPrimaryKey(String userId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("userId", userId);
		return findOne(MAPPER_NAMESPACE + ".selectByPrimaryKey", params);
	}

	public Integer updateUserById(Map map){
		return update(MAPPER_NAMESPACE+".updateUserById",map);
	}

	/**
	 * 获取所有用户信息，通过用户状态
	 * @return
	 */
	public List<User> selectAllUserByStatus() {
		return findAll(MAPPER_NAMESPACE + ".selectAllUserByStatus");
	}

	/**
	 * 根据门店查询用户
	 * @return
	 */
	public List<User> getUserByStoreId(Map map) {
		return findAll(MAPPER_NAMESPACE + ".getUserByStoreId",map);
	}
}
