package cn.haliaeetus.sendmessage.dao;

import cn.haliaeetus.sendmessage.bean.WhiteList;
import cn.haliaeetus.sendmessage.framework.mybatis.persistence.BaseMybatisDAO;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 白名单设置
 * 
 * @author zhangyun
 * 
 */
@Repository
public class WhiteListDao extends BaseMybatisDAO {
	private static final String MAPPER_NAMESPACE = "cn.haliaeetus.sendmessage.dao.mapper.WhiteListMapper";

	/**
	 * 根据storeId查询白名单
	 * 
	 * @param storeId
	 * @return
	 */
	public List<WhiteList> getByStoreId(int storeId) {
		return findAll(MAPPER_NAMESPACE + ".getByStoreId", storeId);
	}

	/**
	 * 根据phone，storeId查询白名单
	 * 
	 * @param phone
	 *            ，storeId
	 * @return
	 */
	public List<WhiteList> getByPhoneStoreId(String phone, int storeId) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("storeId", storeId);
		params.put("userMobile", phone);
		return findAll(MAPPER_NAMESPACE + ".getByPhoneStoreId", params);
	}

	/**
	 * 根据主键，更新所有数据
	 * 
	 * @param entity
	 * @return
	 */
	public int updateWhiteList(WhiteList whiteList) {
		return update(MAPPER_NAMESPACE + ".updateByParams", whiteList);
	}

	/**
	 * 选择性添加，不需要更新字段设空null
	 * 
	 * @param entity
	 * @return
	 */
	public int saveWhiteList(WhiteList whiteList) {
		return save(MAPPER_NAMESPACE + ".insertSelective", whiteList);
	}

	/**
	 * 根据條件删除
	 
	 * @param id
	 * @return
	 */
	public int deleteWhiteList(int storeId, String mobilePhone) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("storeId", storeId);
		params.put("usermobile", mobilePhone);
		return delete(MAPPER_NAMESPACE + ".deleteByParams", params);
	}
}
