/**
 * @FileName: BaseContraller.java
 * @Package cn.haliaeetus.framework.web.controller
 * 
 * @author warship
 * @created 2013-11-22 上午11:03:39
 * 
 * Copyright 2013-2050 佰颂物联 版权所有
 */
package cn.haliaeetus.sendmessage.framework.controller;

import cn.haliaeetus.sendmessage.bean.basedata.FjUser;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.exception.NotLoginException;
import cn.haliaeetus.sendmessage.exception.StoreStatusErrorException;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.spring.DateConvertEditor;
import cn.haliaeetus.sendmessage.framework.toolkit.Jsoned;
import cn.haliaeetus.sendmessage.framework.util.StreamUtil;
import cn.haliaeetus.sendmessage.service.FjUserService;
import cn.haliaeetus.sendmessage.service.UserService;
import cn.haliaeetus.sendmessage.utils.PageData;
import cn.haliaeetus.sendmessage.utils.ServletUtils;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.TypeMismatchException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.ConversionFailedException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.haliaeetus.sendmessage.utils._.asMap;
import static cn.haliaeetus.sendmessage.utils._.isNotEmpty;

/**
 * <p>
 * Contraller基类
 * </p>
 * 
 * @author warship
 * @since 1.0
 * @version 1.0
 */
public abstract class BaseController {
    protected final Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private UserService userService;
	
    @Autowired
    private FjUserService fjUserService;

	protected static final String COOKIE_KEY = "_haliaeetus_web_";

	protected static final String COOKIE_USERID = "_haliaeetus_userId_";

	protected static final String FJ_COOKIE_KEY = "_fj_haliaeetus_web_";

	protected static final String FJ_COOKIE_USERID = "_fj_haliaeetus_userId_";
	
	/*private static final String fhmUrl = APIConfig.gefhmUrl();*/
	@InitBinder
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Date.class, new DateConvertEditor());
	}

	public HttpSession getHTTPSession(HttpServletRequest request) {
		if (request == null) {
			return null;
		}
		return request.getSession();
	}



	public ResponseEntity<String> plain(String msg){
		return new ResponseEntity<>(msg,HttpStatus.OK);
	}

	 /* ------------------------- json & util ------------------------- */

	public static final Gson gson = new Gson();
	public static Object buildErrJson(String msg) {
		return buildJson(500, msg);
	}
	
	public static Object buildJson(int code, String msg) {
		return asMap("code", code, "msg", msg);
	}
	
	public static Object buildJson(int code, String msg, Object data) {
		if(data == null) {
			return asMap("code", code, "msg", msg);
		}else {
			return asMap("code", code, "msg", msg, "data", data);
		}
	}
	
	public static boolean isAjax(HttpServletRequest req) {
		return ServletUtils.isAjax(req);
	}
	public static String getIp(HttpServletRequest req) {
		return ServletUtils.getIp(req);
	}

	public static String sendRedirect(HttpServletResponse resp, String url) throws IOException {
		resp.sendRedirect(url);
		return null;
	}

	public static JsonObject buildErrJson(int code, String msg){
		JsonObject object = new JsonObject();
		object.addProperty("code",code);
		object.addProperty("msg", msg);
		return object;
	}

	public static JsonObject buildSingleEleJson(int code, String msg, Object obj){
		JsonObject object = new JsonObject();
		object.addProperty("code",code);
		object.addProperty("msg", msg);
		object.addProperty("data", String.valueOf(obj));
		return object;
	}

	public User requireLogin(HttpServletRequest request ) {
		User user = getFromCookie(request);
		if( user != null ) {
			if(isNumeric(user.getUserId())){
				user = userService.getUserByuserId(user.getUserId());
			}else{
				user = userService.getUserByuserId(user.getUserName());
			}

//		Store store = basicInfoService.getStoreById(user.getStoreId());
//			if (store == null || store.getStatus() != Constants.STORE_STATUS_OPENED) {
//				log.error("门店状态异常，无法访问");
//	        	throw new StoreStatusErrorException("门店状态异常，无法访问");
//	        }
			return user;
		} 
		log.error("该页面需要登录才能访问~");
		throw new NotLoginException("该页面需要登录才能访问~");
	}

	public String getDataFromRequest(HttpServletRequest request) throws IOException {
		byte[] bytes = StreamUtil.consume(request.getInputStream());
		if( bytes != null && bytes.length > 0 )
			return new String(bytes,"UTF-8");
		return null;
	}
	//原方法
	/*public User getUserFromParam(HttpServletRequest request) {
		String userId = request.getParameter(COOKIE_USERID);
		String password = request.getParameter(COOKIE_KEY);
		if( isNotEmpty(userId) && isNotEmpty(password) ){
            return userService.getUserByIdAndPwd(userId,password);
//			if( user != null ){
//				return password.equalsIgnoreCase(Md5Util.md5AsLowerHex(user.getPassword())) ? user : null;
//			}
		}
		return null;
	}*/
	//修改原方法从缓存里取数据
	public User getUserFromParam(HttpServletRequest request) {
		String userId = request.getParameter(COOKIE_USERID);
		String password = request.getParameter(COOKIE_KEY);
		if( isNotEmpty(userId) && isNotEmpty(password) ){
			User user = userService.getUserById(userId);
			if(null == user){
				return null;
			}
			if(password.equals(user.getPassword())){
				return user;
			}
		}
		return null;
	}



//	public FJUser getFJUserFromCookie(HttpServletRequest request) throws SQLException {
//		Cookie[] cookies = request.getCookies();
//		String userId = null;
//		String password = null;
//		if( cookies != null ){
//			for( Cookie c : cookies ){
//				if( c.getName().equals(FJ_COOKIE_USERID) )
//					userId = c.getValue();
//				if( c.getName().equals(FJ_COOKIE_KEY) )
//					password = c.getValue();
//			}
//			if( isNotEmpty(userId) && isNotEmpty(password) ){
//				FJUser user = FJUser.getByUserId(userId);
//				if( user != null ){
//					return password.equalsIgnoreCase(Md5Util.md5AsLowerHex(user.password)) ? user : null;
//				}
//			}
//		}
//		return null;
//	}

	public User getFromCookie(HttpServletRequest request) {
		Cookie[] cookies = request.getCookies();
		String userId = null;
		String password = null;
		if( cookies != null ){
			for( Cookie c : cookies ){
				if( c.getName().equals(COOKIE_USERID) )
					userId = c.getValue();
				if( c.getName().equals(COOKIE_KEY) )
					password = c.getValue();
			}
			if( isNotEmpty(userId) && isNotEmpty(password) ){
//				User user = userService.getUserById(userId);
				System.out.println("===============Phoenix 登录============");
		        
		        
//		        String url = SMSConfig.getLpDeliverySmsConfigRptStatusUrl();
		        Map<String, String> params = new HashMap<String, String>();
		        String result = "";
		        params.put("userName",userId);
		        params.put("passWord",password);
				String md5Pwd = Md5Util.md5AsLowerHex(password);
				Map<String, String> result1=this.getLoginInfo(userId,md5Pwd);
		        JSONObject jsob = JSONObject.fromObject(result1);
		        String userStatus=jsob.getString("status");
		        System.out.println("===============:"+userStatus);
		        User user=new User();
		        if ("success".equals(userStatus)) {
		        	String fhmU = jsob.getString("user");
//		        	JSONObject fromObject = JSONObject.fromObject(fhmU.toString());
		        	JSONArray jsa = JSONArray.fromObject(fhmU);
					JSONObject jsonU = jsa.getJSONObject(0);
					user.setUserId(getJson(jsonU,"userId"));
					user.setUserName(getJson(jsonU,"userName"));
					user.setPassword(getJson(jsonU,"password"));
					user.setStoreId(getJson(jsonU,"storeId")!="" ? Integer.valueOf(getJson(jsonU,"storeId")) : 0);
					user.setShortAddress(getJson(jsonU,"shortAddress"));
					user.setTelPhone(getJson(jsonU,"telPhone"));
					user.setPush15(getJson(jsonU,"push15")!="" ? Integer.valueOf(getJson(jsonU,"push15")) : 0);
					user.setPush10(getJson(jsonU,"push10")!="" ? Integer.valueOf(getJson(jsonU,"push10")) : 0);
					user.setCodeType(getJson(jsonU,"codeType")!="" ? Integer.valueOf(getJson(jsonU,"codeType")) : 0);
					user.setSmsSendType(getJson(jsonU,"smsSendType")!="" ? Integer.valueOf(getJson(jsonU,"smsSendType")) : 0);
					user.setStorePwd(getJson(jsonU,"storePwd"));
					user.setWebVersion(getJson(jsonU,"webVersion"));
					user.setContainsShelf(getJson(jsonU,"containsShelf")!="" ? Integer.valueOf(getJson(jsonU,"containsShelf")) : 0);
					user.setStatus(getJson(jsonU,"status")!="" ? Integer.valueOf(getJson(jsonU,"status")) :0);
					user.setIsSmsFree(getJson(jsonU,"isSmsFree")!="" ? Integer.valueOf(getJson(jsonU,"isSmsFree")) :0);
					user.setSmsPrice(getJson(jsonU,"smsPrice")!="" ? new BigDecimal(getJson(jsonU,"smsPrice")) : new BigDecimal(0));
					user.setSmsBalance(getJson(jsonU,"smsBalance")!="" ? new BigDecimal(getJson(jsonU,"smsBalance")) : new BigDecimal(0));
					user.setIsOcrFree(getJson(jsonU,"isOcrFree")!="" ? Integer.valueOf(getJson(jsonU,"isOcrFree")) :0);
					user.setOcrPrice(getJson(jsonU,"ocrPrice")!="" ? new BigDecimal(getJson(jsonU,"ocrPrice")) : new BigDecimal(0));
					user.setVigilance(getJson(jsonU,"vigilance")!="" ? new Double(getJson(jsonU,"vigilance")) : new Double(0));
					user.setPhoneLogo(getJson(jsonU,"phoneLogo"));
					user.setComSign(getJson(jsonU,"comSign"));
					user.setSkin(getJson(jsonU,"skin"));
					user.setRights(getJson(jsonU,"rights"));
					user.setRole_id(getJson(jsonU,"role_id"));
					user.setDonateRate(getJson(jsonU,"donateRate")!="" ? Integer.valueOf(getJson(jsonU,"donateRate")) : 0);
					user.setSiduanma(getJson(jsonU,"siduanma"));
					//add 2018-01-22新增入库签收标示
					if(getJson(jsonU,"signFlag")!=null){
						user.setSignFlag( new Integer(getJson(jsonU,"signFlag")));
					}
					//新增快件提醒方式0短信，1微信
					user.setReminder(getJson(jsonU,"reminder")!="" ? Integer.valueOf(getJson(jsonU,"reminder")) :0);
		        	System.out.println("==========:");
		        	return user;
		        }else {
		        	return null;
		        }
		        
			/*	if( user != null ){
					return password.equalsIgnoreCase(user.getPassword()) ? user : null;
				}*/
			}
		}
		return null;
	}
	
    public FjUser getFJUserFromCookie(HttpServletRequest request) {
        Cookie[] cookies = request.getCookies();
        String userId = null;
        String password = null;
        if( cookies != null ){
            for( Cookie c : cookies ){
                if( c.getName().equals(FJ_COOKIE_USERID) )
                    userId = c.getValue();
                if( c.getName().equals(FJ_COOKIE_KEY) )
                    password = c.getValue();
            }
            if( isNotEmpty(userId) && isNotEmpty(password) ){
                FjUser fjUser = fjUserService.findFjUserById(userId);
                if( fjUser != null ){
                    return password.equalsIgnoreCase(Md5Util.md5AsLowerHex(fjUser.getPassword())) ? fjUser : null;
                }
            }
        }
        return null;
    }

	public String buildRetJson(int code,String msg,List<? extends Jsoned> jsoneds) {
		JsonObject jsonObject = new JsonObject();
		jsonObject.addProperty("code",code);
		jsonObject.addProperty("msg",msg);

		JsonArray array = new JsonArray();
		for( Jsoned ele : jsoneds ){
			if( ele != null )
				array.add(ele.toJson());
		}

		jsonObject.add("data",array);
		return jsonObject.toString();
	}

    public <T extends Jsoned> String buildRetJson(int code,String msg,T ele) {
        JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("code",code);
        jsonObject.addProperty("msg",msg);
        jsonObject.add("data",ele.toJson());
        return jsonObject.toString();
    }
//
//	public <T extends Jsoned> String buildRetJson(int code,String msg,T ele) throws SQLException {
//		JsonObject jsonObject = new JsonObject();
//		jsonObject.addProperty("code",code);
//		jsonObject.addProperty("msg",msg);
//		jsonObject.add("data",ele.toJson());
//		return jsonObject.toString();
//	}
	

//	public static JsonObject getPostJson(HttpServletRequest request) throws IOException {
//		StringBuffer jb = new StringBuffer();
//		String line = null;
//		
//		BufferedReader reader = request.getReader();
//		while ((line = reader.readLine()) != null) {
//			jb.append(line);
//		}
//		reader.close();
//		return new JsonParser().parse(jb.toString()).getAsJsonObject();
//	}

    /* ------------------------- err handler ------------------------- */

	@ExceptionHandler(NotLoginException.class)
	public String handleNotLoginException(Exception e,HttpServletRequest request){
		return "redirect:/login";
	}
	
	@ExceptionHandler(StoreStatusErrorException.class)
	public String handleStoreStatusErrException(Exception e,HttpServletRequest request){
		return "redirect:/login";
	}

	@ExceptionHandler(SQLException.class)
	@ResponseBody
	public String handleSQLException(Exception e, HttpServletRequest req, HttpServletResponse resp){
		e.printStackTrace();
		return buildErrJson(-1, "sql error").toString();
	}

	@ExceptionHandler({ MissingServletRequestParameterException.class, TypeMismatchException.class,
			ConversionFailedException.class, IllegalArgumentException.class})
	public String handleMissParamException(Exception e, HttpServletRequest req, HttpServletResponse resp){
		e.printStackTrace();
		req.setAttribute("msg",e.getLocalizedMessage());
		return "500";
	}

	@ExceptionHandler(Exception.class)
	@ResponseBody
	public String handleCommonException(Exception e, HttpServletRequest req, HttpServletResponse resp)
			throws IOException {
		// jetty插件时，浏览器取消请求会产生此Exception，别打log了
		if (e.getClass().getName().equals("org.eclipse.jetty.io.EofException")) {
			return "jetty error";
		}

		// demo期间，打开所有异常stackTrace
		System.err.println("------------------------------------------------------------");
		System.err.println("err on request: " + ServletUtils.getUrl(req));
		e.printStackTrace(System.err);
		System.err.println("------------------------------------------------------------");

		if (isAjax(req)) {
			Object json = buildJson(500, e.getMessage());
			resp.setStatus(500);
			resp.getWriter().print(gson.toJson(json));
			return null;
		} else {
			return buildErrJson(-1,e.getClass().getSimpleName()).toString();
		}
	}

	public static ResponseEntity<String> plainText(String body) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.TEXT_PLAIN);
		return new ResponseEntity<>(body, headers, HttpStatus.OK);
	}

	public static ResponseEntity<String> plainText(String body,Charset charset) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType( new MediaType("text","plain", charset) );
		return new ResponseEntity<>(body, headers, HttpStatus.OK);
	}

	public static ResponseEntity<String> plainJson(String body,Charset charset) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType( new MediaType("application","json", charset) );
		return new ResponseEntity<>(body, headers, HttpStatus.OK);
	}

	public static ResponseEntity<byte[]> binStream(byte[] datas){
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType( new MediaType("","") );
		return new ResponseEntity<>(datas,headers,HttpStatus.OK);
	}

	public static void setLogAttachment(HttpServletRequest req, String attachment){
		req.setAttribute("attachment",attachment);
	}
	
    public  String getJson(JSONObject j,String str){
    	try{
    		String  s= j.getString(str);
    		return s;
    	}catch(Exception e){
    		return "";
    	}

    }


	public Map<String, String> getLoginInfo(String userId,String passWord){
		PageData pd = new PageData();
		Map<String, String> params = new HashMap<String, String>();
		pd.put("userId", userId);
		pd.put("password", passWord);
		try {
			User user = userService.getPhenixUserInfo(pd);
			if(user!=null){
				params.put("status","success");
				params.put("user",JSONArray.fromObject(user).toString());
			}else{
				params.put("status","error");
			}
		}catch (Exception e){
			params.put("status","error");
			e.printStackTrace();
		}
		return params;
	}


	public static boolean isNumeric(String str) {
		String bigStr;
		try {
			bigStr = new BigDecimal(str).toString();
		} catch (Exception e) {
			return false;//异常 说明包含非数字。
		}
		return true;
	}
	//验证headers里的用户名和密码
	public  User getUserHeadrs(HttpServletRequest request){

		return null;
	}

	public User getAppFromCookie(HttpServletRequest request) {
		Cookie[] cookies = request.getCookies();
		String userId = null;
		String password = null;
		if( cookies != null ){
			for( Cookie c : cookies ){
				if( c.getName().equals(COOKIE_USERID) )
					userId = c.getValue();
				if( c.getName().equals(COOKIE_KEY) )
					password = c.getValue();
			}
			if( isNotEmpty(userId) && isNotEmpty(password) ){
//				User user = userService.getUserById(userId);
				System.out.println("===============Phoenix 登录============");
				User user=new User();
				user.setUserId(userId);
				user.setPassword(password);
				return user;
			}
		}
		return null;
	}

	public User requireAppLogin( HttpServletRequest request ) {
		User user = getAppFromCookie(request);
		if( user != null ) {
			if(isNumeric(user.getUserId())){
				user = userService.getUserByuserId(user.getUserId());
			}else{
				user = userService.getUserByuserId(user.getUserName());
			}

//		Store store = basicInfoService.getStoreById(user.getStoreId());
//			if (store == null || store.getStatus() != Constants.STORE_STATUS_OPENED) {
//				log.error("门店状态异常，无法访问");
//	        	throw new StoreStatusErrorException("门店状态异常，无法访问");
//	        }
			return user;
		}
		log.error("该页面需要登录才能访问~");
		throw new NotLoginException("该页面需要登录才能访问~");
	}

}