package cn.haliaeetus.sendmessage.framework.factory.express.realize;

import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.express.jitu.JiTuDeliveryCabinetDTO;
import cn.haliaeetus.sendmessage.bean.express.jitu.JiTuRequestBean;
import cn.haliaeetus.sendmessage.bean.express.jitu.JiTuResponseBean;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.DeliveryQueryCondition;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.factory.express.BaseExpress;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;

/**
 * @ClassName JiTuExpress
 * @Description 极兔快递公司接口
 * @Author Administrator
 * @Date 2019/8/28 0028 下午 16:52
 * @Version 1.0
 **/
@Slf4j
@Component("296")
public class JiTuExpress implements BaseExpress {

    @Autowired
    private DeliveryService deliveryService;

    @Autowired
    private PlatformConfigService platformConfigService;

    @Override
    public void expressSignBack(SignBackDto signBackDto) {
        int returnSignFlag = 0;
        if ("in".equals(signBackDto.getActionType())) {
            returnSignFlag = 3;
        } else {
            returnSignFlag = 1;
        }
        this.signBackJiTu(signBackDto.getExpCode(), signBackDto.getTelPhone(), signBackDto.getShelfCode(), signBackDto.getStore(), returnSignFlag);
    }

    private void signBackJiTu(String expCode, String telPhone, String shelfCode, Store store, int returnSignFlag) {
        ArrayList<JiTuDeliveryCabinetDTO> jituDeliveryCabinetDTOList = new ArrayList<>();
        PlatformConfig platformConfig = platformConfigService.getByPlatform("JTSD");
        // 生产地址
         String logisticsHost = platformConfig.getLogisticsUrl();
         String host = platformConfig.getUrl();

        // 测试环境地址
//        String logisticsHost = "http://callback.fengzhan100.com/logistics";
//        String host = "https://test-cabinet.jtexpress.com.cn";

        String logisticsUrl = logisticsHost + "/jitu/enteringCabinetList/uploadData";
        String url = host + "/ecs/standard/cabinet/upload";
        JiTuDeliveryCabinetDTO jiTuDeliveryCabinetDTO = JiTuDeliveryCabinetDTO.builder()
                .waybillId(expCode)
                .scanPda("FZYZ20200616001")
                .scanSourceCode("5")
                .scanSourceName("驿站")
                .stationContact(store.getBackupPhone())
                .partnerName("蜂站")
                .city(store.getCityName())
                .expressCabinet(store.getShortAddress() + "--" + store.getShortName())
                .scanTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")))
                .operationSource(23)
                .build();
        if (3 != returnSignFlag) {
            jiTuDeliveryCabinetDTO.setSigner(store.getShortName());
            url = host += "/ecs/standard/cabinet/sign";
            logisticsUrl = logisticsHost + "/jitu/cabinet/sign";
        }
        DeliveryQueryCondition qc = new DeliveryQueryCondition(store.getId());
        Delivery delivery = new Delivery();
        jituDeliveryCabinetDTOList.add(jiTuDeliveryCabinetDTO);
        try {
            String accessToken = getAccessToken();
            int i = 0;
            while (i < 3) {
                String res = "";
                String fullUrl = url + "?access_token=" + accessToken;
                res = OkHttpUtil.postJson(logisticsUrl, JiTuRequestBean.builder()
                        .jiTuDeliveryCabinetDTOList(jituDeliveryCabinetDTOList)
                        .url(fullUrl)
                        .build());
                log.info("调用接口返回结果:{}", res);
                JiTuResponseBean jiTuResponseBean = JSONObject.parseObject(res, JiTuResponseBean.class);
                boolean flag = jiTuResponseBean.getSucc();
                /*
                  15301009: access_token不能为空
                  15301010: 无效的access_token
                  重新获取tocken并重新请求
                 */
                if (!flag && ("15301009".equals(jiTuResponseBean.getCode()) || "15301010".equals(jiTuResponseBean.getCode()))) {
                    accessToken = getAccessToken(true);
                    i++;
                    continue;
                }
                delivery.setRemark(flag ? "success" : jiTuResponseBean.getCode() + ":" + jiTuResponseBean.getMsg());
                qc.setSignFlag(flag ? String.valueOf(returnSignFlag) : String.valueOf(returnSignFlag + 1));
                qc.setReturnSignStatus(Integer.parseInt(qc.getSignFlag()));
                qc.setUserPhone(telPhone);
                qc.setExp_code(expCode);
                qc.setEntity(delivery);
                deliveryService.update(qc);
                break;
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            ;
            delivery.setRemark(null == e.getMessage() ? "NULL" : e.getMessage().substring(0, 30));
            qc.setEntity(delivery);
            qc.setSignFlag(String.valueOf(returnSignFlag + 1));
            qc.setReturnSignStatus(returnSignFlag + 1);
            qc.setUserPhone(telPhone);
            qc.setExp_code(expCode);
            deliveryService.update(qc);
        }
    }

    /**
     * 快件取出扫描接口测试（接口已对接，但是系统没有该功能）
     */
    public void deliveryOut() throws Exception {
        ArrayList<JiTuDeliveryCabinetDTO> jituDeliveryCabinetDTOList = new ArrayList<>();
        PlatformConfig platformConfig = platformConfigService.getByPlatform("JTSD");
        String accessToken = getAccessToken();
        String url = platformConfig.getUrl() + "/ecs/standard/cabinet/sign?access_token=" + accessToken;

        JiTuDeliveryCabinetDTO jiTuDeliveryCabinetDTO = JiTuDeliveryCabinetDTO.builder()
                .waybillId("JT0000114566807")
                .scanPda("FZYZ20200616001")
                .scanSourceCode("5")
                .scanSourceName("驿站")
                .stationContact("17671645804")
                .partnerName("蜂站")
                .city("杭州市")
                .expressCabinet("昆仑天籁物业服务中心--昆仑天籁蜂站")
                .operationSource(23)
                .build();
        jiTuDeliveryCabinetDTO.setScanTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        jituDeliveryCabinetDTOList.add(jiTuDeliveryCabinetDTO);
        String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl(), JiTuRequestBean.builder()
                .jiTuDeliveryCabinetDTOList(jituDeliveryCabinetDTOList)
                .url(url)
                .build());
        log.info("调用接口返回结果:{}", res);
    }


    /**
     * 获取accessToken,默认在缓存中获取
     */
    private String getAccessToken(){
        return getAccessToken(false);
    }

    /**
     * refresh 是否刷新
     */
    private String getAccessToken(boolean refresh) {
        String accessToken = null;
        try {
            CacheTool cacheTool = CacheTool.initJiTuCacheTool();
            /*
             *  当refresh == true 时强制刷新,重新获取accessToken
             *  当缓存中的accessToken 为空时，重新获取accessToken
             */
            if (refresh || Strings.isNullOrEmpty((accessToken = cacheTool.getStringKey("access_token")))) {
                PlatformConfig platformConfig = platformConfigService.getByPlatform("JTSD");
                String corpid = platformConfig.getAppkey();
                String corpsecret = platformConfig.getAppSecret();
                String host = platformConfig.getUrl();
                // 测试环境
                // String host = "https://test-cabinet.jtexpress.com.cn";
                String res = OkHttpUtil.get(host + "/ecs/standard/cabinet/gettoken?corpid=" + corpid + "&corpsecret=" + corpsecret);
                JSONObject resObj = JSON.parseObject(res);
                JSONObject data = resObj.getJSONObject("data");
                accessToken = data.getString("access_token");
                cacheTool.setStringKey("access_token", accessToken, data.getInteger("expires_in"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println(accessToken);
        return accessToken;
    }

    public static void main(String[] args) {
        String.valueOf(1);
        String value = "" + new Integer(1).toString();
        System.out.println(String.format("姓名%s,%d", "sun", "lee"));
    }
}
