package cn.haliaeetus.sendmessage.framework.factory.express.realize;

import cn.haliaeetus.sendmessage.bean.basedata.SignBackSwitchBean;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.express.shentong.*;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.DeliveryQueryCondition;
import cn.haliaeetus.sendmessage.framework.factory.express.BaseExpress;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.ExpressHandleService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.delivery.DeliveryMsgTemplateConfigService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName ShenTongExpress
 * @Description TODO
 * @Author Administrator
 * @Date 2019/8/28 0028 下午 16:52
 * @Version 1.0
 **/
@Slf4j
@Component("272")
public class ShenTongExpress implements BaseExpress {

    @Resource
    private DeliveryMsgTemplateConfigService deliveryMsgTemplateConfigService;

    @Resource
    private BasicInfoService basicInfoService;

    @Resource
    private DeliveryService deliveryService;

    @Resource
    private PlatformConfigService platformConfigService;

    @Resource
    private ExpressHandleService expressHandleService;

    private final static String PLATFORM = "STO";

    @Override
    public void expressSignBack(SignBackDto signBackDto) {
        int returnSignFlag = 0;
        if("in".equals(signBackDto.getActionType())){
            returnSignFlag = 3;
        }else{
            returnSignFlag = 1;
        }
        this.signBackShentong(signBackDto.getExpCode(), signBackDto.getTelPhone(), signBackDto.getShelfCode(),signBackDto.getStore(), returnSignFlag);
    }

    private void signBackShentong(String expCode, String telPhone, String shelfCode, Store store, int returnSignFlag) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        SignBackSwitchBean signBackSwitchBean = expressHandleService.getSignBackSwitch(272 , store.getId());
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return ;
        }
        //正式地址
        String logisticsUrl = platformConfig.getLogisticsUrl() + "/shentong/scanAndUplodaMsgByStation";
        String url = platformConfig.getUrl();
        //测试地址
        /*String logisticsUrl = "http://localhost:8089/logistics/shentong/scanAndUplodaMsgByStation";
        String url = "http://cloudinter-linkgatewaytest.sto.cn/gateway/link.do";*/
        List<ShenTongScanRecordDtoBean> records = new ArrayList<>();
        if (!signBackSwitchBean.getIs_syStore() || !signBackSwitchBean.getIs_syUpStore()){
            //type = 0时在申通官网注册门店信息，type = 1时在申通官网修改门店信息
            int type = 0 ;
            if (signBackSwitchBean.getIs_syStore() && !signBackSwitchBean.getIs_syUpStore()){
                type = 1;
            }
            registerOrUpdateStoreInfo(store, signBackSwitchBean ,platformConfig, type);
        }
        ShenTongScanRecordDtoBean shenTongScanRecordDtoBean = ShenTongScanRecordDtoBean.builder()
                .waybillNo(expCode)
                .opCode("793")
                .desc(getDesc(store,returnSignFlag))
                .action(3 == returnSignFlag ? "STA_INBOUND" : "STA_SIGN")
                .city(store.getCityName())
                .stationName(store.getShortName())
                .stationContact(store.getTakeExpressPhone())
                .stationId(String.valueOf(store.getId()))
                .stationSource("1")
                .partnerName("蜂站")
                .time(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now()))
                .authCode(3 == returnSignFlag ? "" : shelfCode)
                .build();
        records.add(shenTongScanRecordDtoBean);
        String res = "";
        DeliveryQueryCondition qc = new DeliveryQueryCondition(store.getId());
        Delivery delivery=new Delivery();
        try{
            res = OkHttpUtil.postJson(logisticsUrl,ShenTongRequestBean.builder()
                    .api_name("GALAXY_RECEIVE_SCAN_MESSAGE_STATION")
                    .content(JSON.toJSONString(ShenTongScanUploadMsgBean.builder().records(records).msgType("CNTP_TRACE_NOTIFY").build()))
                    .from_appkey(platformConfig.getAppkey())
                    .from_code(platformConfig.getAppSecret())
                    .to_appkey("galaxy_receive")
                    .to_code("galaxy_receive")
                    .secretKey(platformConfig.getToken())
                    .url(url).build());
            log.info("调用申通驿站类扫描数据上传接口返回结果：{}",res);
            ShenTongResponseBean shenTongResponseBean = JSONObject.parseObject(res,ShenTongResponseBean.class);
            Boolean flag = shenTongResponseBean.getSuccess();
            delivery.setRemark(flag ? "success" : shenTongResponseBean.getErrorCode() + ":" + shenTongResponseBean.getErrorMsg());
            qc.setEntity(delivery);
            qc.setSignFlag(flag? String.valueOf(returnSignFlag) : String.valueOf(returnSignFlag + 1));
            qc.setReturnSignStatus(Integer.parseInt(qc.getSignFlag()));
            qc.setUserPhone(telPhone);
            qc.setExp_code(expCode);
            deliveryService.update(qc);
        }catch (Exception e){
            log.info("调用申通驿站类扫描数据上传接口返回结果：{}",res);
            log.error(e.getMessage());
            delivery.setRemark(e.getMessage() == null ? "NULL" : e.getMessage().substring(0,30));
            qc.setEntity(delivery);
            qc.setSignFlag(String.valueOf(returnSignFlag + 1));
            qc.setReturnSignStatus(Integer.parseInt(qc.getSignFlag()));
            qc.setUserPhone(telPhone);
            qc.setExp_code(expCode);
            deliveryService.update(qc);
        }
    }

    private String getDesc(Store store,int returnSignFlag) {
        String desc = "快件已被【蜂站】"+ store.getShortAddress() + store.getShortName() + "站点代收，请及时取件。有问题请联系" + store.getTakeExpressPhone();
        if ( 1 == returnSignFlag){
            desc = "已签收,签收人凭取货码签收";
        }
        return desc;
    }

    /**
     * 修改或者注册名店信息
     * @param type
     * @param store
     */
    private void registerOrUpdateStoreInfo(Store store, SignBackSwitchBean signBackSwitchBean, PlatformConfig platformConfig, int type){
        //生产地址
        String logisticsUrl =platformConfig.getLogisticsUrl() + "/shentong/site/updateStoreInfo";
        String url = platformConfig.getUrl();
        //测试地址
        /*String logisticsUrl ="http://localhost:8089/logistics/shentong/site/updateStoreInfo";
        String url = "http://cloudinter-linkgatewaytest.sto.cn/gateway/link.do";*/
        String api_name = "SITE_OPERATION_ENDPOINT_MODIFY";
        String res = "";
        ShenTongBean shentongBean = ShenTongBean.builder()
                .code(store.getId().toString())
                .name(store.getShortName())
                .provinceCode(String.valueOf(store.getProvinceId()))
                .cityCode(String.valueOf(store.getCityId()))
                .areaCode(String.valueOf(store.getAreaId()))
                .address(store.getShortAddress())
                .longitude(store.getLongitude())
                .latitude(store.getLatitude())
                .contactName(store.getOwner())
                .contactPhone(store.getBackupPhone())
                .officePhone(store.getTakeExpressPhone())
                .status(100)
                .brandCode("FENGZHAN")
                .build();
        if (0 == type){
            api_name = "SITE_OPERATION_ENDPOINT_REGISTER";
            //生产地址
            logisticsUrl = platformConfig.getLogisticsUrl() + "/shentong/site/registerStore";
            //测试地址
            /*logisticsUrl = "http://localhost:8089/logistics/shentong/site/registerStore";*/
            shentongBean.setType(100);
        }
        try {
            res = OkHttpUtil.postJson(logisticsUrl,ShenTongRequestBean.builder()
                    .api_name(api_name)
                    .content(JSON.toJSONString(shentongBean))
                    .from_appkey(platformConfig.getAppkey())
                    .from_code(platformConfig.getAppSecret())
                    .to_appkey("site-operation-endpoint")
                    .to_code("site-operation-endpoint")
                    .secretKey(platformConfig.getToken())
                    .url(url).build());
            log.info("调用申通注册或者修改门店信息接口返回结果：{}",res);
            ShenTongResponseBean shenTongResponseBean = JSONObject.parseObject(res, ShenTongResponseBean.class);
            signBackSwitchBean.setYuantongStoreCode(shenTongResponseBean.getErrorMsg());
            expressHandleService.updateSignBackSwitch(signBackSwitchBean);
        }catch (Exception e){
            log.info("调用申通注册或者修改门店信息接口返回结果：{}",res);
            log.error(e.getMessage());
        }
    }


    /**
     * 获取末端驿站列表
     */
    public void getEndpoibtList(){
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return ;
        }
        //正式地址
        String logisticsUrl = platformConfig.getLogisticsUrl() + "/shentong/site/getStationsList";
        String url = platformConfig.getUrl();
        //测试地址
        /*String logisticsUrl = "http://localhost:8089/logistics/shentong/site/getStationsList";
        String url = "http://cloudinter-linkgatewaytest.sto.cn/gateway/link.do";*/
        ShenTongOpreationEndpointListBean shenTongOpreationEndpointListBean = ShenTongOpreationEndpointListBean.builder()
                .pageNum(1)
                .pageSize(20).build();
        try{
            String res = OkHttpUtil.postJson(logisticsUrl,ShenTongRequestBean.builder()
                    .api_name("SITE_OPERATION_ENDPOINT_LIST")
                    .content(JSON.toJSONString(shenTongOpreationEndpointListBean))
                    .from_appkey(platformConfig.getAppkey())
                    .from_code(platformConfig.getAppSecret())
                    .to_appkey("site-operation-endpoint")
                    .to_code("site-operation-endpoint")
                    .secretKey(platformConfig.getToken())
                    .url(url).build());
            log.info("调用申通末端驿站列表接口返回结果:{}",res);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 末端品牌信息查询
     */
    public void getEndpoibtInfo(){
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return ;
        }
        //正式地址
        String logisticsUrl = platformConfig.getLogisticsUrl() + "/shentong/site/getStationInfo";
        String url = platformConfig.getUrl();
        //测试地址
        /*String logisticsUrl = "http://localhost:8089/logistics/shentong/site/getStationInfo";
        String url = "http://cloudinter-linkgatewaytest.sto.cn/gateway/link.do";*/
        ShenTongOpreationEndpointGetinfoBean shenTongOpreationEndpointGetinfoBean = ShenTongOpreationEndpointGetinfoBean.builder().code("10000").build();
        try{
            String res = OkHttpUtil.postJson(logisticsUrl,ShenTongRequestBean.builder()
                    .api_name("SITE_OPERATION_ENDPOINT_GET_INFO")
                    .content(JSON.toJSONString(shenTongOpreationEndpointGetinfoBean))
                    .from_appkey(platformConfig.getAppkey())
                    .from_code(platformConfig.getAppSecret())
                    .to_appkey("site-operation-endpoint")
                    .to_code("site-operation-endpoint")
                    .secretKey(platformConfig.getToken())
                    .url(url).build());
            log.info("调用申通末端品牌信息查询接口返回结果:{}",res);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 末端品牌状态修改
     */
    public void changeStatus(){
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return ;
        }
        //正式地址
        String logisticsUrl = platformConfig.getLogisticsUrl() + "/shentong/site/changeStatus";
        String url = platformConfig.getUrl();
        //测试地址
        /*String logisticsUrl = "http://localhost:8089/logistics/shentong/site/changeStatus";
        String url = "http://cloudinter-linkgatewaytest.sto.cn/gateway/link.do";*/
        List<String> codeList = new ArrayList<>();
        codeList.add("1000");
        ShenTongOperationEndpointChangeStatusBean shenTongOperationEndpointChangeStatusBean = ShenTongOperationEndpointChangeStatusBean.builder().status(100).codeList(codeList).build();
        try{
            String res = OkHttpUtil.postJson(logisticsUrl,ShenTongRequestBean.builder()
                    .api_name("SITE_OPERATION_ENDPOINT_CHANGE_STATUS")
                    .content(JSON.toJSONString(shenTongOperationEndpointChangeStatusBean))
                    .from_appkey(platformConfig.getAppkey())
                    .from_code(platformConfig.getAppSecret())
                    .to_appkey("site-operation-endpoint")
                    .to_code("site-operation-endpoint")
                    .secretKey(platformConfig.getToken())
                    .url(url).build());
            log.info("调用申通末端品牌状态修改接口返回结果:{}",res);
        }catch (Exception e){
            e.printStackTrace();
        }
    }
}
