package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.alibaba.AliBaBaSmsStatusReportBean;
import cn.haliaeetus.sendmessage.bean.alibaba.AliBaBaSmsStatusReportResultBean;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SendSmsRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.dao.IExpressTypeDao;
import cn.haliaeetus.sendmessage.dao.IWxTemplateDao;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.StoreShelfService;
import cn.haliaeetus.sendmessage.service.sms.AsyncSmsRecodeService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ALBB.ALBBSendSmsRequest;
import cn.haliaeetus.sendmessage.utils.ALBB.ALBBSendSmsResponse;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

/**
 * @ClassName AlbbPlatfrom
 * @Description 阿里短信服务
 * @Author liujs
 * @Date 2019/10/16 下午 16:59
 * @Modefy
 * @Version 1.0
 */
@Slf4j
@Component("ALBB")
@RestController
public class ALBBPlatform implements BasePlatform {

    @Autowired
    private PlatformConfigService platformConfigService;
    @Autowired
    private SmsRecordService smsRecordService;
    @Autowired
    private AsyncSmsRecodeService asyncSmsRecodeService;
    @Autowired
    private IWxTemplateDao iWxTemplateDao;
    @Autowired
    private IExpressTypeDao iExpressTypeDao;
    @Autowired
    private StoreShelfService shelfService;

    private final static String PLATFORM = "ALBB";

    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        String smsId = "";
        Integer smsCount = 0;
        SmsRecord smsRecord = smsData.getSmsRecord();
        Delivery delivery = smsData.getDelivery();
        Store store = smsData.getStore();
        //模板变量名：模板变量值 expressName-其他；expressCode-其他号码；storeAddress-其他；shelf-其他号码；takeCode-其他号码；latestBusinessTime-时间；storePhone-电话号码；
        Map<String, String> templateParam = new HashMap<String, String>(16);
        String expressName = iExpressTypeDao.getExpNameByExpId(delivery.getExpId());
        templateParam.put("expressName", expressName);
        templateParam.put("expressCode", getExpressCode(delivery.getExpcode()));
        templateParam.put("storeAddress", store.getShortAddress());
        templateParam.put("shelf", shelfService.getById(delivery.getShelfId()).getShelfName());
        templateParam.put("takeCode", delivery.getShelfCode());
        templateParam.put("latestBusinessTime", store.getEndTime());
        templateParam.put("storePhone", store.getBackupPhone());
        try {
            smsRecord.setId(smsData.getId());
            smsRecord.setSendSuccessType(1);
            smsRecordService.update(smsRecord);
            ALBBSendSmsRequest request = ALBBSendSmsRequest.builder()
                    .accessKeyId(platformConfig.getAppkey())
                    .url(platformConfig.getUrl())
                    .action("SendSms")
                    .version("2017-05-25")
                    .phoneNumbers(smsRecord.getTelphone())
                    .signName("蜂站")
                    .templateCode(getAlbbTemplateCode(delivery.getStoreId()))
                    .templateParam(templateParam)
                    .appSecret(platformConfig.getAppSecret())
                    .build();
            String aLiBaBaSendSmsParams = JSON.toJSONString(request);
            log.info("【调用ALBB发送短信接口，传入参数】：{}", aLiBaBaSendSmsParams);
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/baseSendSms", SendSmsRequest.builder().platform(PLATFORM).params(aLiBaBaSendSmsParams).build());
            // String res = OkHttpUtil.postJson("http://192.168.0.127:8089/logistics/baseSendSms", SendSmsRequest.builder().platform("AliBaBa").params(aLiBaBaSendSmsParams).build());
            log.info("【调用ALBB发送短信接口，返回参数】：{}，", res);
            if (!Strings.isNullOrEmpty(res)) {
                ALBBSendSmsResponse albbSendSmsResponse = JSON.parseObject(res, ALBBSendSmsResponse.class);
                if (albbSendSmsResponse.getCode().compareTo("OK") == 0) {
                    smsId = PLATFORM + albbSendSmsResponse.getBizId();
                    smsCount = smsData.getSmsCount();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }

    @RequestMapping("/smsStatusReportByALBB")
    public CompletableFuture<String> smsStatusReport(@RequestBody List<AliBaBaSmsStatusReportBean> request) throws Exception {
        log.info("参数为:{}", JSON.toJSONString(request));
        asyncSmsRecodeService.smsReceiptProcessALBB(request);
        return CompletableFuture.completedFuture(JSON.toJSONString(AliBaBaSmsStatusReportResultBean.builder().code(0).build()));
    }

    private String getExpressCode(String expcode) {
        String str2 = expcode.substring(0, 1);
        String str3 = expcode.substring(expcode.length() - 4);
        return str2 + "**" + str3;
    }

    private String getAlbbTemplateCode(Integer storeId) {
        return iWxTemplateDao.getALBBStoreTemplateCode(storeId);
    }

}
