package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.PlatformTemplate;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.basedata.SysParam;
import cn.haliaeetus.sendmessage.bean.cainiao.*;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.dao.IExpressTypeDao;
import cn.haliaeetus.sendmessage.dao.IWxTemplateDao;
import cn.haliaeetus.sendmessage.enums.PddExpressEnum;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.StoreShelfService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ALBB.ALBBSendSmsResponse;
import cn.haliaeetus.sendmessage.utils.ALBB.QuerySendDetailsRequest;
import cn.haliaeetus.sendmessage.utils.pdd.GetSystemParam;
import cn.haliaeetus.sendmessage.utils.pdd.InOutBoundRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 〈功能概述〉<br>
 *
 * @className: CainiaoPlatform
 * @package: cn.haliaeetus.sendmessage.framework.factory.platform.realize
 * @author: kael
 * @date: 2019/11/28 0028 下午 16:45
 */
@Component("CaiNiao")
@Slf4j
public class CainiaoPlatform  implements BasePlatform {

    @Autowired
    private PlatformConfigService platformConfigService;

    @Autowired
    private IWxTemplateDao iWxTemplateDao;

    @Autowired
    private IExpressTypeDao iExpressTypeDao;

    @Autowired
    private StoreShelfService shelfService;

    @Autowired
    private SmsRecordService smsRecordService;

    @Autowired
    private BasicInfoService basicInfoService;

    SimpleDateFormat sDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private static final String PLATFROM = "CaiNiao";

    /**
     * 签收回传快递信息
     *
     * @param signBackDto
     */
    @Override
    public void platformSign(SignBackDto signBackDto) {
        this.caiNiaoSignBack(signBackDto, 2);
    }

    /**
     * 入库回传快递信息
     *
     * @param signBackDto
     */
    @Override
    public void platformBack(SignBackDto signBackDto) {
        this.caiNiaoSignBack(signBackDto, 0);
    }

    private void caiNiaoSignBack(SignBackDto signBackDto, int type) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFROM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFROM);
            return;
        }
        CaiNiaoRequest caiNiaoRequest = new CaiNiaoRequest();
        List<Trace> traceList = new ArrayList<>();
        String desc = "[代收点]您的快件已由" + signBackDto.getStore().getShortName() + "代收，代收点地址：" + signBackDto.getStore().getShortAddress() + "，联系电话：" + signBackDto.getStore().getTakeExpressPhone();
        String desc1 = "[代收点]您的快件已从" + signBackDto.getStore().getShortName() + "取出";
        ExtendField[] extendFields ={new ExtendField("receiverTel",signBackDto.getTelPhone() , "取件人电话"),
                new ExtendField("siteId", signBackDto.getStore().getId().toString(), "三方代收站点ID"),
                new ExtendField("siteBrandCode", "FENGZHAN", "三方代收点品牌Code"),
                new ExtendField("dsLongitude", Strings.isNullOrEmpty(signBackDto.getStore().getLongitude()) ? "121.48" : signBackDto.getStore().getLongitude(), "三方代收点经度"),
                new ExtendField("dsLatitude", Strings.isNullOrEmpty(signBackDto.getStore().getLatitude()) ? "31.22" : signBackDto.getStore().getLatitude(), "三方代收点纬度"),
                new ExtendField("officeTime", signBackDto.getStore().getStartTime() + "-" + signBackDto.getStore().getEndTime(), "代收点营业时间"),
                new ExtendField("overDueCharge", "0", "超期不收费")};
        List<ExtendField> extendFieldsList = Arrays.asList(extendFields);
        List<ExtendField> extendFieldArrayList = new ArrayList<>(extendFieldsList);
        if (type == 0){
                    extendFieldArrayList.add(new ExtendField("authCode", signBackDto.getShelfCode(), "取件码"));
                    extendFieldArrayList.add(new ExtendField("smsSignature", "蜂站", "触达消费者的短信签名"));
                    extendFieldArrayList.add(new ExtendField("needSendNotify", "0", "是否需要菜鸟通知消费者到代收点取件"));
        }
        traceList.add(Trace.builder()
                .country("China")
                .province(signBackDto.getStore().getProvinceName())
                .city(signBackDto.getStore().getCityName())
                .facilityType("3")
                .facilityName(platformConfig.getSpare1())
                .facilityNo(signBackDto.getStore().getId().toString())
                .time(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))
                .desc(type == 0 ? desc : desc1)
                .action(type == 0 ? "SH_INBOUND" : "SH_SIGNED")
                .contactPhone(signBackDto.getStore().getTakeExpressPhone())
                .address(signBackDto.getStore().getShortAddress())
                .tz("8")
                .extendFields(extendFieldArrayList)
                .build());
        List<TraceElement> traceElements = new ArrayList<>();
        traceElements.add(TraceElement.builder().logisticProviderID(getExpressCompanyByExpCode(signBackDto.getExpCode()))
                .mailNos(signBackDto.getExpCode())
                .traces(traceList)
                .build());
        String logisticsInterface = JSON.toJSONString(
                CaiNiaoWaybillQuery.builder().tracesList(traceElements).build());
        caiNiaoRequest.setMsg_type("TRACEPUSH");
        caiNiaoRequest.setLogistic_provider_id(platformConfig.getToken());
        caiNiaoRequest.setData_digest(Md5Util.caiNiaoDoSign(logisticsInterface, "utf-8", platformConfig.getAppSecret()));
        caiNiaoRequest.setLogistics_interface(logisticsInterface);
        caiNiaoRequest.setTo_code("TRACEPUSH");
        caiNiaoRequest.setUrl(platformConfig.getUrl());
        String res = "";
        try {
            log.info("liujs----->cainiao快递回传信息接口" + (type == 0 ? "入库" : "签收") + "传入参数：" + JsonUtils.toJson(caiNiaoRequest));
            res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/cainiao/caiNiaoSignBack", caiNiaoRequest);
            log.info("liujs----->cainiao快递回传信息接口" + (type == 0 ? "入库" : "签收") + "返回参数：" + res);
            //无需关注是否调用接口成功
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFROM);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFROM);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        SmsRecord smsRecord = smsData.getSmsRecord();
        String smsId = "";
        Integer smsCount = 0;
        try {
            smsRecord.setId(smsData.getId());
            smsRecord.setSendSuccessType(1);
            log.info("请求url：{}", platformConfig.getLogisticsUrl() + "/cainiao/sendSms");
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/cainiao/smsSend", CaiNiaoRequest.builder()
                    .msg_type("CNSMS_VIRTUALNO_MAILNO_SEND")
                    .logistic_provider_id(platformConfig.getToken())
                    .data_digest(platformConfig.getAppSecret())
                    .logistics_interface(JSON.toJSONString(CaiNiaoSendSmsRequest.builder()
                            .mailNo(smsData.getDelivery().getExpcode())
                            .cpCode(getCpCode(smsData.getDelivery().getExpId()))
                            .smsCode(getPlatformTemplate(smsData.getStore().getId()).getTemplateCode())
                            .smsSign(platformConfig.getSpare1())
                            .smsParam(getContent(smsData.getDelivery(), smsData.getStore()))
                            .build()))
                    .url(platformConfig.getUrl())
                    .build());
            log.info("菜鸟传入参数：{}", res);
            if (!Strings.isNullOrEmpty(res)) {
                CaiNiaoResponse caiNiaoResponse = JSON.parseObject(res, CaiNiaoResponse.class);
                if (caiNiaoResponse.getSuccess()) {
                    smsId = PLATFROM + caiNiaoResponse.getResult();
                    smsCount = 1;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }
        smsRecordService.update(smsRecord);
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }

    private String getCpCode(Integer expId) {
        Map reqParam = new HashMap(3);
        reqParam.put("typeId", "1");
        reqParam.put("shortName", expId);
        SysParam sysParam = iExpressTypeDao.getSysParam(reqParam);
        Map respParam = new Gson().fromJson(sysParam.getContent(), Map.class);
        return respParam == null ? "" : respParam.get(PLATFROM).toString();
    }

    private String getContent(Delivery delivery, Store store) {
        // 您的${expressName}快递${expressCode}已到${storeAddress}，${shelf}取货码${takeCode}，请${latestBusinessTime}前领取。询${storePhone}
        Map<String, String> smsParam = new HashMap<String, String>(16);
        smsParam.put("expressName", iExpressTypeDao.getExpNameByExpId(delivery.getExpId()));
        smsParam.put("expressCode", getExpressCode(delivery.getExpcode()));
        smsParam.put("storeAddress", store.getShortAddress());
        smsParam.put("shelf", shelfService.getById(delivery.getShelfId()).getShelfName());
        smsParam.put("takeCode", delivery.getShelfCode());
        smsParam.put("latestBusinessTime", store.getEndTime());
        smsParam.put("storePhone", store.getBackupPhone());
        return JSON.toJSONString(smsParam);
    }

    private String getExpressCode(String expcode) {
        String str2 = expcode.substring(0, 1);
        String str3 = expcode.substring(expcode.length() - 4, expcode.length());
        return str2 + "**" + str3;
    }

    private PlatformTemplate getPlatformTemplate(Integer id) {
        Map param = new HashMap<>(3);
        param.put("storeId", id);
        param.put("platform", PLATFROM);

        PlatformTemplate template = iWxTemplateDao.getStoreTemplateCode(param);
        // 默认模板
        if(template == null){
            template = new PlatformTemplate();
            template.setTemplateCode("SMS_10100827");
        }

        return template;
    }

    /**
     * 根据快递单号去菜鸟查快递公司(猜面单接口)
     *
     * @param ExpCode
     * @return
     */
    public String getExpressCompanyByExpCode(String ExpCode) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform("CaiNiao");
        String logisticsInterface = JSON.toJSONString(CaiNiaoWaybillQuery.builder().appName("FENGZHAN").mailNo(ExpCode).build());
        CaiNiaoRequest caiNiaoRequest = new CaiNiaoRequest();
        caiNiaoRequest.setMsg_type("LD_GUESSCP");
        caiNiaoRequest.setLogistic_provider_id(platformConfig.getToken());
        caiNiaoRequest.setData_digest(Md5Util.caiNiaoDoSign(logisticsInterface, "utf-8", platformConfig.getAppSecret()));
        caiNiaoRequest.setLogistics_interface(logisticsInterface);
        caiNiaoRequest.setTo_code("LD-PACKPUSH");
        caiNiaoRequest.setUrl(platformConfig.getUrl());
        CaiNiaoResponse caiNiaoResponse = null;
        try {
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/cainiao/getExpressCompanyByExpCode", caiNiaoRequest);
            caiNiaoResponse = JSONObject.parseObject(res, CaiNiaoResponse.class);
        } catch (Exception e) {
            log.error(e.toString());
        }
        if (caiNiaoResponse != null && caiNiaoResponse.getCpCodeList() != null) {
            return caiNiaoResponse.getCpCodeList().get(0);
        }
        return null;
    }
}
