package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.haidiao.HaiDiaoSendSmsBean;
import cn.haliaeetus.sendmessage.bean.haidiao.HaiDiaoSendSmsRespond;
import cn.haliaeetus.sendmessage.bean.haidiao.HaiDiaoSmsStatusReportBean;
import cn.haliaeetus.sendmessage.bean.sms.SendSmsRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.framework.util.StringUtil;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.sms.AsyncSmsRecodeService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * @PackageName: cn.haliaeetus.sendmessage.framework.factory.platform.realize
 * @ClassName: HaiDiaoPlatform
 * @Author: liujs
 * @Description: 海雕短信平台
 * @Date: 13:55 2020/7/1
 * @Version: 1.0
 */
@Slf4j
@Component(Constants.PLATFORM_HAIDIAO)
@RestController
public class HaiDiaoPlatform implements BasePlatform {
    @Resource
    private PlatformConfigService platformConfigService;

    @Resource
    private AsyncSmsRecodeService asyncSmsRecodeService;

    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) throws Exception {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(Constants.PLATFORM_HAIDIAO);
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", Constants.PLATFORM_HAIDIAO);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        String smsId = "";
        Integer smsCount = 0;
        //海雕发送短信参数
        String content = smsData.getMsg();
        HaiDiaoSendSmsBean haiDiaoSendSmsBean = HaiDiaoSendSmsBean.builder()
                .userName(platformConfig.getAppkey())
                .passWord(Md5Util.md5AsLowerHex(platformConfig.getAppSecret()))
                .comSign(platformConfig.getSpare1())
                .phoneNumber(smsData.getDelivery().getUserPhone())
                .content(content)
                .md5Content(Md5Util.md5AsLowerHex(content + platformConfig.getToken()))
                .url(platformConfig.getUrl())
                .build();
        //调用发送短信接口
        String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/baseSendSms", SendSmsRequest.builder().platform(Constants.PLATFORM_HAIDIAO).params(JSON.toJSONString(haiDiaoSendSmsBean)).build());
        if (!StringUtil.isEmpty(res)) {
            HaiDiaoSendSmsRespond respond = JSONObject.parseObject(res, HaiDiaoSendSmsRespond.class);
            if (0 == respond.getCode()) {
                smsId = Constants.PLATFORM_HAIDIAO + respond.getMsgid();
                smsCount = smsData.getSmsCount();
            }
        }
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }

    @RequestMapping("/smsStatusReportByHaiDiao")
    public String smsStatusReport(@RequestBody HaiDiaoSmsStatusReportBean request) throws Exception {
        log.info("参数为:{}", JSON.toJSONString(request));
        asyncSmsRecodeService.smsReceiptProcessHaiDiao(request.getResult());
        return "0";
    }
}
