package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.mengwang.*;
import cn.haliaeetus.sendmessage.bean.sms.SendSmsRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.framework.util.StringUtil;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.sms.AsyncSmsRecodeService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * @PackageName: cn.haliaeetus.sendmessage.framework.factory.platform.realize
 * @ClassName: MengWangPlatform
 * @Description: TODO
 * @Author: liujs
 * @Date: 2020/05/20 下午 17:25
 * @Version: 1.0
 */
@Slf4j
@Component(Constants.PLATFORM_MENGWANG)
@RestController
public class MengWangPlatform implements BasePlatform {

    @Resource
    private PlatformConfigService platformConfigService;

    @Resource
    private SmsRecordService smsRecordService;

    @Autowired
    private AsyncSmsRecodeService asyncSmsRecodeService;

    private final static String PLATFORM = "MengWang";


    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) throws Exception {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        String smsId = "";
        Integer smsCount = 0;
        if (platformConfig == null) {
            log.info("配置表中没有{}的配置", PLATFORM);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        //梦网发送短信参数
        String timestamp = new SimpleDateFormat("MMddHHmmss").format(new Date());
        String userId = platformConfig.getAppkey();
        MengWangSendSmsBean mengWangSendSmsParam = MengWangSendSmsBean.builder()
                .userid(userId)
                .pwd(Md5Util.md5AsLowerHex(userId + "00000000" + platformConfig.getAppSecret() + timestamp))
                .timestamp(timestamp)
                .mobile(smsData.getSmsRecord().getTelphone())
                .content(URLEncoder.encode(smsData.getMsg(), "GBK")).build();
        String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/baseSendSms", SendSmsRequest.builder().platform("MengWang").params(JSON.toJSONString(mengWangSendSmsParam)).build());
        // String res = OkHttpUtil.postJson("http://192.168.0.127:8089/logistics/baseSendSms", SendSmsRequest.builder().platform("MengWang").params(JSON.toJSONString(mengWangSendSmsParam)).build());
        if (!StringUtil.isEmpty(res)) {
            MengWangSendSmsRespond respond = JSONObject.parseObject(res, MengWangSendSmsRespond.class);
            if (0 == respond.getResult()) {
                smsId = PLATFORM + respond.getMsgId();
                smsCount = smsData.getSmsCount();
            }
        }
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }

    @RequestMapping("/smsStatusReportByMengWang")
    public CompletableFuture<String> smsStatusReport(@RequestBody MengWangSmsStatusReportBean request) throws Exception {
        log.info("参数为:{}", request);
        String userId = "JJ2978";
        String pwd = Md5Util.md5AsLowerHex(request.getUserid() + "00000000853992" + request.getTimestamp());
        if (userId.equals(request.getUserid()) && pwd.equals(request.getPwd())) {
            List<MengWangSmsStatusBean> mengWangSmsStatusBeanList = JSONObject.parseArray(request.getRpts(), MengWangSmsStatusBean.class);
            asyncSmsRecodeService.smsReceiptProcessMengWang(mengWangSmsStatusBeanList);
            return CompletableFuture.completedFuture(JSON.toJSONString(MengWangSmsStatusReportResultBean.builder()
                    .result(0)
                    .cmd("RPT_RESP")
                    .seqid(request.getSeqid()).build()));
        } else {
            return CompletableFuture.completedFuture(JSON.toJSONString(MengWangSmsStatusReportResultBean.builder()
                    .result(-10009)
                    .cmd("RPT_RESP")
                    .seqid(request.getSeqid()).build()));
        }
    }


    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }
}
