package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.SignBackSwitchBean;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.enums.PddExpressEnum;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.ExpressHandleService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ALBB.QuerySendDetailsRequest;
import cn.haliaeetus.sendmessage.utils.pdd.*;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import com.pdd.pop.sdk.common.util.JsonUtil;
import com.pdd.pop.sdk.http.api.response.PddOpenMsgServiceSendExpressMsgResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @ClassName PddPlatform
 * @Description 拼多多
 * @Author kael
 * @Date 2019/8/28 0028 下午 15:45
 * @Version 1.0
 **/
@Component("PDD")
@Slf4j
public class PddPlatform implements BasePlatform {

    @Autowired
    private BasicInfoService basicInfoService;

    @Autowired
    private ExpressHandleService expressHandleService;

    @Autowired
    private PlatformConfigService platformConfigService;

    @Autowired
    private SmsRecordService smsRecordService;

    SimpleDateFormat sDateFormat=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private final static String PLATFROM = "PDD";

    @Override
    public void platformSign(SignBackDto signBackDto) {
        this.pddSignBack(signBackDto,2);
    }

    @Override
    public void platformBack(SignBackDto signBackDto) {
        this.pddSignBack(signBackDto,0);
    }

    @Override
    public void platformAuthStore(Store store,Integer type) {
        GetSystemParam getSystemParam = new GetSystemParam().invoke(basicInfoService);
        if (getSystemParam.is()) {
            return ;
        }
        CabinetInfoRequest request = CabinetInfoRequest.builder()
                .companyCode(getSystemParam.getPartnerId())
                .cabCode(store.getId().toString())
                .cabName(store.getName())
                .cabType("01")
                .province(store.getProvinceName())
                .city(store.getCityName())
                .district(store.getAreaName())
                .buildingName(store.getShortAddress())
                .address(store.getShortAddress())
                .deliverAddress(store.getShortAddress())
                .status(1)
                .type(type)
                .latitude(Strings.isNullOrEmpty(store.getLatitude())?"31.22":store.getLatitude())
                .longitude(Strings.isNullOrEmpty(store.getLongitude())?"121.48":store.getLongitude())
                .coordType("GPS")
                .build();
        String res = "";
        request = this.sysParamHandle(request,getSystemParam.getPddUrl(),getSystemParam.getPartnerId(),getSystemParam.getSecret(),"/express/cabinet/"+getSystemParam.getPartnerId());
        try{
            log.info("kael----->pdd基础信息同步接口"+(type==1?"新增":"修改")+"传入参数："+ JsonUtils.toJson(request));
            res= OkHttpUtil.postJson(getSystemParam.getUrl()+"/pddAuth",request);
            log.info("kael----->pdd基础信息同步接口"+(type==1?"新增":"修改")+"返回参数："+ res);
            PddResponse response = new Gson().fromJson(res,PddResponse.class);
            if(response!=null&&response.isSuccess()){
                SignBackSwitchBean signBackSwitchBean = expressHandleService.getSignBackSwitch(9998 , store.getId());
                expressHandleService.updateSignBackSwitch(signBackSwitchBean);
            }
        }catch (Exception e){
            log.error(e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData){
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFROM);
        if(platformConfig == null){
            log.info("配置表中没有{}的配置",PLATFROM);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        SmsRecord smsRecord = smsData.getSmsRecord();
        Delivery delivery = smsData.getDelivery();
        Store store = smsData.getStore();
        List<Map<String, String>> templateParamJson = new ArrayList<Map<String, String>>();
        //模板变量名：模板变量值
        Map<String, String> item = new HashMap<String, String>(16);
        item.put("${content}", smsData.getMsg());
        templateParamJson.add(item);
        List<String> waybillCodes = new ArrayList<String>();
        waybillCodes.add(delivery.getExpcode());
        PddOpenMsgServiceSendExpressMsgResponse response = null;
        String res = "";
        String smsId = "";
        Integer smsCount = 0;
        try {
            smsRecord.setId(smsData.getId());
            smsRecord.setSendSuccessType(1);
            smsRecordService.update(smsRecord);
            PddSendSmsData request = PddSendSmsData.builder()
                    .clientId(platformConfig.getAppkey())
                    .clientSecret(platformConfig.getAppSecret())
                    .templateParamJson(templateParamJson)
                    .waybillCodes(waybillCodes)
                    .wpCode(PddExpressEnum.of(delivery.getExpId()).getBrand())
                    .signName(basicInfoService.getConfigurationByCode("signBackName").getOValue1())
                    .templateCode(1)
                    .build();
            log.info("【调用PDD发送短信接口，传入参数】：{}，", JSON.toJSONString(request));
            res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl()+"/pddSendSms", request);
            log.info("【调用PDD发送短信接口，返回参数】：{}，",res);
            if(res != null){
                response = JsonUtil.transferToObj(res, PddOpenMsgServiceSendExpressMsgResponse.class);
                if(response != null && response.getCode().compareTo(0)==0){
                    smsId = "PDD"+response.getBizId();
                    smsCount = 1;
                }else{
                    errorSmsRecord(smsData, smsRecord,response.getMessage());
                }
            }else{
                errorSmsRecord(smsData, smsRecord,"拼多多短信接口调用失败");
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }

    private void errorSmsRecord(SmsDataRequest smsData, SmsRecord smsRecord,String msg) {
        smsRecord.setStatus(Constants.SMS_SEND_STATUS_FAIL);
        smsRecord.setResultInfo(msg);
        smsRecord.setId(smsData.getId());
        smsRecord.setCreatetime(null);
        smsRecordService.update(smsRecord);
    }

    private void pddSignBack(SignBackDto signBackDto, Integer type) {
        GetSystemParam getSystemParam = new GetSystemParam().invoke(basicInfoService);
        if (getSystemParam.is()) {
            return ;
        }
        InOutBoundRequest request = InOutBoundRequest.builder()
                .companyCode(getSystemParam.getPartnerId())
                .cabCode(signBackDto.getStore().getId().toString())
                .type(type)
                .mailNo(signBackDto.getExpCode())
                .expressCode(PddExpressEnum.of(signBackDto.getExpId()).getBrand())
                .deliverPhone(signBackDto.getStore().getBackupPhone())
                .receiverPhone(signBackDto.getTelPhone())
                .time(sDateFormat.format(new Date()))
                .deliverAddress(signBackDto.getStore().getShortAddress())
                .verifyCode(signBackDto.getShelfCode())
                .build();
        String res = "";
        request = this.sysParamHandle(request,getSystemParam.getPddUrl(),getSystemParam.getPartnerId(),getSystemParam.getSecret(),"/express/cabinet/"+getSystemParam.getPartnerId()+"/in-out-bound");
        try{
            log.info("kael----->pdd消息通知接口"+(type==0?"入库":"签收")+"传入参数："+ JsonUtils.toJson(request));
            res= OkHttpUtil.postJson(getSystemParam.getUrl()+"/pddSignBack",request);
            log.info("kael----->pdd消息通知接口"+(type==0?"入库":"签收")+"返回参数："+ res);
            //无需关注是否调用接口成功
        }catch (Exception e){
            log.error(e.getMessage());
            e.printStackTrace();
        }
    }

    private <T extends PddRequest> T  sysParamHandle(T request,String url,String partnerId,String secret,String afterUrl) {
        request.setApiName(url+afterUrl);
        request.setPartner_id(partnerId);
        request.setSecret(secret);
        return request;
    }
}
