package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.ExpressType;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.bean.wx.TemplateData;
import cn.haliaeetus.sendmessage.bean.wx.WxResBean;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplate;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplateEntity;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.service.wx.WxTemplateService;
import com.alibaba.fastjson.JSON;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName WeChatPlatform
 * @Description
 * @Author SunJJ
 * @Date 2020/5/25 17:46
 * @Version V1.0
 **/
@Component(Constants.PLATFORM_WECHAT)
@Slf4j
public class WeChatPlatform implements BasePlatform {

    @Autowired
    private WxTemplateService wxTemplateService;

    @Autowired
    private BasicInfoService basicInfoService;

    @Autowired
    private SmsRecordService smsRecordService;


    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) throws Exception {
        String smsId = "";
        Integer smsCount = 0;
        try {
            SmsRecord smsRecord = smsData.getSmsRecord();
            User user = smsData.getUser();
            Integer id = smsData.getId();
            Delivery delivery = smsData.getDelivery();
            String msg = smsData.getMsg();

            Store store = smsData.getStore();

            Map temMap = new HashMap();
            temMap.put("storeId", user.getStoreId());
            List<WxTemplateEntity> wxTemplateEntityList;
            wxTemplateEntityList = wxTemplateService.selectTemplateByStoreId(temMap);
            WxTemplate wxTemplate2 = getTemplate(user, delivery, wxTemplateEntityList, msg, store);
            smsRecord.setWxSendContent(JsonUtils.toJson(wxTemplate2));
            //发送微信消息
            WxResBean wxResBean = getWxResBean(id, smsRecord, wxTemplate2);
            if ("0".equals(wxResBean.getErrcode())) {
                smsCount = 0;
                smsRecord.setIsmgid(wxResBean.getMsgid());
                smsRecord.setSmsPrice(new BigDecimal(0));
                smsRecord.setSendSuccessType(2);
                smsRecord.setId(id);
                smsId = wxResBean.getErrcode();
                smsRecordService.update(smsRecord);
            }
        } catch (Exception e) {
            log.warn(e.getMessage());
        }
        return SmsDataResponse.builder()
                .smsCount(smsCount)
                .smsId(smsId)
                .build();
    }

    private WxTemplate getTemplate(User user, Delivery delivery, List<WxTemplateEntity> wxTemplateEntityList, String msg, Store store) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        WxTemplate wxTemplate = new WxTemplate();
        wxTemplate.setStore_id(String.valueOf(user.getStoreId()));
        wxTemplate.setMobile_number(Long.valueOf(delivery.getUserPhone()));
        wxTemplate.setTemplate_id(wxTemplateEntityList.get(0).getMsgtempid());
        wxTemplate.setTouser("");
        Map<String, TemplateData> data = new HashMap<String, TemplateData>();
        String[] msgs = msg.split("，");
        TemplateData templateData = new TemplateData();
        if (msgs.length > 3) {
            templateData.setValue(msgs[0] + "，" + msgs[2] + "，" + msgs[3]);
        } else {
            templateData.setValue(msgs[0] + "，" + msgs[2]);
        }
        data.put("first", templateData);
        String shelfCode = "";
        for (WxTemplateEntity wxTemplateEntity : wxTemplateEntityList) {
            try {
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (wxTemplateEntity.getValue().equals("tel")) {
                TemplateData template = new TemplateData();
                template.setValue(delivery.getUserPhone());
                data.put(wxTemplateEntity.getKey(), template);
            }
            if (wxTemplateEntity.getValue().equals("address")) {
                TemplateData template = new TemplateData();
                String[] bb2 = msg.split("已到")[1].split("，");
                template.setValue(bb2[0] + "，" + bb2[2]);
                data.put(wxTemplateEntity.getKey(), template);
                shelfCode = bb2[1].split("码")[1];
            }
            if (wxTemplateEntity.getValue().equals("time")) {
                TemplateData template = new TemplateData();
                template.setValue(sdf.format(delivery.getCreateTime()));
                data.put(wxTemplateEntity.getKey(), template);
            }
            //TODO
            if (wxTemplateEntity.getValue().equals("company")) {
                TemplateData template = new TemplateData();
                int expId = delivery.getExpId();
                List<ExpressType> allExps = basicInfoService.getExpressByStoreId(user.getStoreId()).get(1);
                String expName = "";
                for (ExpressType expressType : allExps) {
                    if (expId == expressType.getExpId()) {
                        expName = expressType.getExpName();
                        break;
                    }
                }
                template.setValue(expName);
                data.put(wxTemplateEntity.getKey(), template);
            }
            if (wxTemplateEntity.getValue().equals("expcode")) {
                TemplateData template = new TemplateData();
                template.setValue(delivery.getExpcode());
                data.put(wxTemplateEntity.getKey(), template);
            }
        }
        TemplateData templateData6 = new TemplateData();
        templateData6.setValue(store.getAdvertisingMsg());
        templateData6.setColor("#FF0000");
        data.put("remark", templateData6);

        wxTemplate.setData(data);
        wxTemplate.setUrl(getUrl(store.getAdvertisingUrl(), delivery, shelfCode));
        return wxTemplate;
    }

    private static String getUrl(String url, Delivery delivery, String shelfCode) {
        if (Strings.isBlank(url)) {
            return null;
        }
        Map<String, String> param = new HashMap<>(16);
        param.put("{shelfCode}", shelfCode);
        param.put("{deliveryId}", delivery.getId().toString());
        param.put("{storeId}", delivery.getStoreId().toString());
        param.put("{phone}", delivery.getUserPhone());
        for (Map.Entry<String, String> entry : param.entrySet()) {
            if (url.indexOf(entry.getKey()) >= 0) {
                url = url.replace(entry.getKey(), entry.getValue());
                System.out.println(url);
            }
        }
        return url;
    }

    private WxResBean getWxResBean(int id, SmsRecord smsRecord, WxTemplate wxTemplate2) throws Exception {
        String url = basicInfoService.getConfigurationByCode("sendWxSignMsgUrl").getOValue1();
        WxResBean wxResBean = null;
        try {
            Long beginTime = System.currentTimeMillis();
            String res = OkHttpUtil.postJson(url, wxTemplate2);
            Long endTime = System.currentTimeMillis();
            smsRecord.setWxSendUsedTime(endTime - beginTime);
            wxResBean = JSON.parseObject(res, WxResBean.class);
        } catch (Exception e) {
            smsRecord.setStatus(Constants.SMS_SEND_STATUS_FAIL);
            smsRecord.setWxResultInfo("微信消息接口异常");
            smsRecord.setId(id);
            smsRecord.setCreatetime(null);
            smsRecordService.update(smsRecord);
            throw new Exception(e);
        }
        return wxResBean;
    }
}
