package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.ExpressType;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.bean.wx.TemplateData;
import cn.haliaeetus.sendmessage.bean.wx.WxResBean;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplate;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplateEntity;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.constants.MapConstants;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.sms.SMSHelper;
import cn.haliaeetus.sendmessage.framework.sms.entity.HttpResponse;
import cn.haliaeetus.sendmessage.framework.util.JsonUtil;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.rsync.ConsumeRecordService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.service.wx.WxTemplateService;
import cn.haliaeetus.sendmessage.utils.ALBB.QuerySendDetailsRequest;
import cn.haliaeetus.sendmessage.utils._;
import com.alibaba.fastjson.JSON;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName WuXiPlatform
 * @Description 无锡短信
 * @Author kael
 * @Date 2019/9/27 0027 下午 15:09
 * @Version 1.0
 **/
@Component("WUXI")
@Slf4j
public class WuXiPlatform  implements BasePlatform {

    @Autowired
    private SmsRecordService smsRecordService;

    @Autowired
    private ConsumeRecordService consumeRecordService;

    @Autowired
    private WxTemplateService wxTemplateService;

    @Autowired
    private BasicInfoService basicInfoService;

    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) {
        String smsId = "";
        SmsRecord smsRecord = smsData.getSmsRecord();
        Integer reminder = smsData.getReminder();
        User user = smsData.getUser();
        Integer id = smsData.getId();
        Delivery delivery = smsData.getDelivery();
        String msg = smsData.getMsg();
        Integer smsCount = smsData.getSmsCount();
        Store store = smsData.getStore();

        smsRecord.setId(id);
        smsRecord.setSendSuccessType(1);
        smsRecordService.update(smsRecord);
        smsId = sendSMS_WX(user,delivery, msg, smsRecord, id);

        return SmsDataResponse.builder()
                .smsCount(smsCount)
                .smsId(smsId)
                .build();
    }

    /**
     * 无锡线上线下发送短信
     */
    private String sendSMS_WX(User user, Delivery delivery, String msg, SmsRecord smsRecord, int id) {
        String smsId = "";
        try {
            String msgid = "";
            Long beginTime = System.currentTimeMillis();
            String response = SMSHelper.sendSMS_WX(user.getComSign(),delivery.getUserPhone(), msg, msgid);
            Long endTime = System.currentTimeMillis();
            smsRecord.setSmsSendUsedTime(endTime-beginTime);
            log.info("SMSHelper.sendSMS_WX.response:" + response);

            HttpResponse sendResponse = JSON.parseObject(response, HttpResponse.class);
            if (sendResponse == null) {
                log.info(_.f("dahan sms send error, send response is empty, error info : smsRecordId:[%s], expcode:[%s], expId:[%s].",
                        smsRecord.getId(), delivery.getExpcode(), delivery.getExpId()));
                smsRecord.setStatus(Constants.SMS_SEND_STATUS_FAIL);
                smsRecord.setResultInfo("无锡平台短信提交失败");
                smsRecord.setId(id);
                smsRecord.setCreatetime(null);
                smsRecordService.update(smsRecord);
                return smsId;
            }

            if (sendResponse != null&&!sendResponse.getCode().equals("0")) {
                smsRecord.setStatus(Constants.SMS_SEND_STATUS_FAIL);
                smsRecord.setResultInfo(MapConstants.WXSMSErrorCodeMsg.get(sendResponse.getCode()));
                smsRecord.setId(id);
                smsRecord.setCreatetime(null);
                smsRecordService.update(smsRecord);
                return smsId;
            }
            // 短信Id
            return "WX" + sendResponse.getMsgid();

        } catch (Exception e) {
//            log.error(_.f("无锡平台发送短信失败，订单信息：%s，异常信息：%s.", delivery, e.getMessage()));
//            smsRecord.setStatus(Constants.SMS_SEND_STATUS_FAIL);
//            smsRecord.setResultInfo("无锡线上线下接口异常");
//            smsRecord.setId(id);
//            smsRecord.setCreatetime(null);
//            smsRecordService.update(smsRecord);
            return smsId;
        }
    }

    public static boolean isNumeric(String str) {
        String bigStr;
        try {
            bigStr = new BigDecimal(str).toString();
        } catch (Exception e) {
            return false;//异常 说明包含非数字。
        }
        return true;
    }

    private WxTemplate getTemplate(User user,Delivery delivery,List<WxTemplateEntity> wxTemplateEntityList,String msg,Store store){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        WxTemplate wxTemplate=new WxTemplate();
        wxTemplate.setStore_id(String.valueOf(user.getStoreId()));
        wxTemplate.setMobile_number(Long.valueOf(delivery.getUserPhone()));
        wxTemplate.setTemplate_id(wxTemplateEntityList.get(0).getMsgtempid());
        wxTemplate.setTouser("");
        Map<String, TemplateData> data = new HashMap<String,TemplateData>();
        String[] msgs = msg.split("，");
        TemplateData templateData = new TemplateData();
        if(msgs.length > 3){
            templateData.setValue(msgs[0]+"，"+msgs[2]+"，"+msgs[3]);
        }else{
            templateData.setValue(msgs[0]+"，"+msgs[2]);
        }
        data.put("first", templateData);
        String shelfCode = "";
        for(WxTemplateEntity wxTemplateEntity:wxTemplateEntityList){
            try {
                log.info("wxTemplateEntity:::::::::::::::::::::"+ JsonUtil.format(wxTemplateEntity));
            }catch (Exception e){
                e.printStackTrace();
            }
            if(wxTemplateEntity.getValue().equals("tel")){
                TemplateData template = new TemplateData();
                template.setValue(delivery.getUserPhone());
                data.put(wxTemplateEntity.getKey(), template);
            }
            if(wxTemplateEntity.getValue().equals("address")){
                TemplateData template = new TemplateData();
                String[] bb2=msg.split("已到")[1].split("，");
                template.setValue(bb2[0]+"，"+bb2[2]);
                data.put(wxTemplateEntity.getKey(), template);
                shelfCode = bb2[1].split("码")[1];
            }
            if(wxTemplateEntity.getValue().equals("time")){
                TemplateData template = new TemplateData();
                template.setValue(sdf.format(delivery.getCreateTime()));
                data.put(wxTemplateEntity.getKey(), template);
            }
            //TODO
            if(wxTemplateEntity.getValue().equals("company")){
                TemplateData template = new TemplateData();
                int expId=delivery.getExpId();
                List<ExpressType> allExps = basicInfoService.getExpressByStoreId(user.getStoreId()).get(1);
                String expName="";
                for(ExpressType expressType:allExps){
                    if(expId==expressType.getExpId()){
                        expName=expressType.getExpName();
                        break;
                    }
                }
                template.setValue(expName);
                data.put(wxTemplateEntity.getKey(), template);
            }
            if(wxTemplateEntity.getValue().equals("expcode")){
                TemplateData template = new TemplateData();
                template.setValue(delivery.getExpcode());
                data.put(wxTemplateEntity.getKey(), template);
            }
//            if(wxTemplateEntity.getValue().equals("pwd")){
//                TemplateData template = new TemplateData();
//                template.setValue(delivery.getShelfCode());
//                data.put(wxTemplateEntity.getKey(), template);
//            }

        }
        TemplateData templateData6 = new TemplateData();
        templateData6.setValue(store.getAdvertisingMsg());
        templateData6.setColor("#FF0000");
        data.put("remark", templateData6);

        wxTemplate.setData(data);
        wxTemplate.setUrl(getUrl(store.getAdvertisingUrl(),delivery,shelfCode));
        return wxTemplate;
    }

    private static final String[] STRINGS = {"{shelfCode}","{deliveryId}","{storeId}","{phone}"};

    private String getUrl(String url,Delivery delivery,String shelfCode){
        if(Strings.isBlank(url)){
            return null;
        }
        Map<String,String> param = new HashMap<>(16);
        param.put("{shelfCode}",shelfCode);
        param.put("{deliveryId}",delivery.getId().toString());
        param.put("{storeId}",delivery.getStoreId().toString());
        param.put("{phone}",delivery.getUserPhone());
        String[] sb = {new String()};
        param.entrySet().forEach(entry -> {
            if(url.indexOf(entry.getKey())>=0){
                sb[0] = url.replace(entry.getKey(),entry.getValue());
            }
        });
        return sb[0];
    }

}
