package cn.haliaeetus.sendmessage.framework.factory.platform.realize;

import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.framework.factory.platform.BasePlatform;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ALBB.QuerySendDetailsRequest;
import cn.haliaeetus.sendmessage.utils.Zhongtong.ZtoResponse;
import cn.haliaeetus.sendmessage.utils.Zhongtong.ZtoSendSmsRequest;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @ClassName ZTOPlatform
 * @Description 中通短信服务
 * @Author kael
 * @Date 2019/10/16 下午 16:59
 * @Modefy
 * @Version 1.0
 */
@Component("ZTO")
@Slf4j
public class ZTOPlatform implements BasePlatform {

    @Autowired
    private PlatformConfigService platformConfigService;
    @Autowired
    private SmsRecordService smsRecordService;

    private final static String PLATFORM = "ZTO";

    @Override
    public void platformSign(SignBackDto signBackDto) {

    }

    @Override
    public void platformBack(SignBackDto signBackDto) {

    }

    @Override
    public void platformAuthStore(Store store, Integer type) {

    }

    @Override
    public SmsDataResponse sendSms(SmsDataRequest smsData) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        if (platformConfig ==null){
            log.info("配置表中没有{}的配置",PLATFORM);
            return SmsDataResponse.builder()
                    .smsId("")
                    .smsCount(0)
                    .build();
        }
        String smsId = "";
        Integer smsCount = 0;
        SmsRecord smsRecord = smsData.getSmsRecord();
        Delivery delivery = smsData.getDelivery();
        Store store = smsData.getStore();

        try {
            smsRecord.setId(smsData.getId());
            smsRecord.setSendSuccessType(1);
            smsRecordService.update(smsRecord);
            ZtoSendSmsRequest request = ZtoSendSmsRequest.builder()
                    .billCode(delivery.getExpcode())
                    .companyCode(platformConfig.getToken())
                    .hallCode(store.getId().toString())
                    .mobile(delivery.getUserPhone())
                    .sendCode(delivery.getShelfCode())
                    .build();
            request.setApiName(platformConfig.getUrl()+"/sendMsg");
            request.setCompanyId(platformConfig.getAppSecret());
            request.setKey(platformConfig.getAppkey());
            log.info("【调用中通发送短信接口，传入参数】：{}，", JSON.toJSONString(request));
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl()+"/ztoSendSms", request);
            log.info("【调用中通发送短信接口，返回参数】：{}，",res);
            if(!Strings.isNullOrEmpty(res)){
                ZtoResponse response = JSON.parseObject(res,ZtoResponse.class);
                if (response.getStatus() && "200".equals(response.getStatusCode())){
                    smsId = PLATFORM+delivery.getExpcode();
                    smsCount = smsData.getSmsCount();
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return SmsDataResponse.builder()
                .smsId(smsId)
                .smsCount(smsCount)
                .build();
    }
}
