/**
 * @FileName: Logger.java
 * @Package cn.haliaeetus.framework.base.log
 * 
 * @author warship
 * @created 2013年11月10日 上午12:00:05
 * 
 * Copyright 2013-2050 佰颂物联 版权所有
 */
package cn.haliaeetus.sendmessage.framework.log;


/**
 * <p>日志</p>
 * 
 * @author warship
 * @since 1.0
 * @version 1.0
 */
public class Logger {
	
	private org.apache.log4j.Logger logger;
	
	private String FQCN = Logger.class.getName();
	
	/**
	 * 私有构造器，初始化时设置日志适配器为Log4j
	 */
	private Logger(Class<?> clazz) {
		logger = org.apache.log4j.Logger.getLogger(clazz.getName());
	}
	
	/**
	 * 
	 * 获取Logger实例
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:01:34
	 *
	 * @param clazz
	 * @return
	 */
	public static Logger getInstance(Class<?> clazz) {
		return new Logger(clazz);
	}
	
	private org.apache.log4j.Logger getLogger() {
		return logger;
	}
	
	/**
	 * 
	 * debug日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:06:14
	 *
	 * @param message	日志详细信息
	 * @param t	异常
	 */
	public void debug(String message, Throwable t) {
		getLogger().log(FQCN, org.apache.log4j.Level.DEBUG, message, t);
	}
	
	/**
	 * 
	 * info日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:17:36
	 *
	 * @param message	日志详细信息
	 */
	public void info(String message) {
		getLogger().log(FQCN, org.apache.log4j.Level.INFO, message, null);
	}
	
	/**
	 * 
	 * info日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:18:33
	 *
	 * @param message	日志详细信息
	 * @param t	异常
	 */
	public void info(String message, Throwable t) {
		getLogger().log(FQCN, org.apache.log4j.Level.INFO, message, t);
	}
	
	/**
	 * 
	 * warn日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:18:54
	 *
	 * @param message	日志详细信息
	 * @param t	异常
	 */
	public void warn(String message, Throwable t) {
		getLogger().log(FQCN, org.apache.log4j.Level.WARN, message, t);
	}

	/**
	 * 
	 * error日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:19:15
	 *
	 * @param message	日志详细信息
	 * @param t	异常
	 */
	public void error(String message, Throwable t) {
		getLogger().log(FQCN, org.apache.log4j.Level.ERROR, message, t);
	}
	
	
	/**
	 * 
	 * 自定义类型日志，级别介于WARN和ERROR中间，主要用于记录业务日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:19:30
	 *
	 * @param message	日志详细信息
	 */
	public void record(String message) {
		getLogger().log(FQCN, org.apache.log4j.Level.INFO, message, null);
	}
	
	/**
	 * 
	 * 自定义类型日志，级别介于WARN和ERROR中间，主要用于记录业务日志
	 *
	 * @author warship
	 * @created 2013年11月10日 上午12:19:50
	 *
	 * @param message	日志详细信息
	 * @param t	异常
	 */
	public void record(String message, Throwable t) {
		getLogger().log(FQCN, org.apache.log4j.Level.ERROR, message, t);
	}

}
