/**
 * @FileName: BaseMybatisDAO.java
 * @Package main.java.cn.haliaeetus.framework.common.persistence
 * 
 * @author warship
 * @created 2013年11月9日 下午5:03:35
 * 
 * Copyright 2013-2050 佰颂物联 版权所有
 */
package cn.haliaeetus.sendmessage.framework.mybatis.persistence;

import cn.haliaeetus.sendmessage.bean.mengwang.MengWangSmsStatusBean;
import cn.haliaeetus.sendmessage.framework.entity.baseEntity.BaseEntity;
import cn.haliaeetus.sendmessage.framework.mybatis.page.PagingResult;
import cn.haliaeetus.sendmessage.framework.mybatis.search.Condition;
import cn.haliaeetus.sendmessage.framework.mybatis.search.Search;
import org.apache.ibatis.session.RowBounds;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>Mybatis基类</p>
 * 
 * @author warship
 * @since 1.0
 * @version 1.0
 */
public class BaseMybatisDAO {

	@Autowired
	protected SqlSessionTemplate template;
	
	
	/**
	 * liutianyuan  
	 * 
	 * 查询参数为queryCondition类型
	 * dao层可以不用写一堆凑参数的代码，交给接口层来统一封装queryCondition
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId, Object params) {
		return template.selectList(sqlId, params);
	}
	public long count(String sqlId, Object params) {
		return template.selectOne(sqlId, params);
	}
	public <T extends BaseEntity> int update(String sqlId, Object params) {
		return template.update(sqlId, params);
	}
	/**
	 * 
	 * 查询指定SQL语句的所有记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:17:41
	 *
	 * @param sqlId	SQL语句ID
	 * @return	查询到的结果集合
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId) {
		return template.selectList(sqlId);
	}
	
	/**
	 * 
	 * 查询指定SQL语句的所有记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:19:58
	 *
	 * @param sqlId	SQL语句ID
	 * @param params	条件参数
	 * @return	查询到的结果集合
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId, Map<String, Object> params) {
		return template.selectList(sqlId, params);
	}
	
	
	/**
	 * 
	 * 查询指定SQL语句的所有记录
	 *
	 * @author warship
	 * @created 2014年2月23日 上午1:07:16
	 *
	 * @param sqlId	SQL语句ID
	 * @param params	条件参数
	 * @return	查询到的结果集合
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId, List<? extends Object> params) {
		return template.selectList(sqlId, params);
	}
	
	
	/**
	 * 
	 * 查询指定SQL语句的所有记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午11:24:47
	 *
	 * @param sqlId	SQL语句ID
	 * @param param	条件参数
	 * @return	查询到的结果集合
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId, int param) {
		return template.selectList(sqlId, param);
	}
	
	/**
	 * 
	 * 查询指定SQL语句的所有记录
	 *
	 * @author warship
	 * @created 2014年2月9日 下午5:26:33
	 *
	 * @param sqlId	SQL语句ID
	 * @param param	条件参数
	 * @return	查询到的结果集合
	 */
	public <T extends BaseEntity> List<T> findAll(String sqlId, String param) {
		return template.selectList(sqlId, param);
	}
	
	/**
	 * 
	 * 分页查询指定SQL语句的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午11:24:03
	 *
	 * @param countSqlId 总数查询SQL语句ID
	 * @param sqlId	SQL语句ID
	 * @param search 查询
	 * @return 分页查询结果PagingResult
	 */
	public <T extends BaseEntity> PagingResult<T> findForPage(String countSqlId, String sqlId, Search search) {
		RowBounds rowBounds = new RowBounds(search.getFirstRowNum(), search.getRows());
		List<T> list = template.selectList(sqlId, getConditionMap(search), rowBounds);
		return new PagingResult<T>(count(countSqlId, search), list, search.getPage(), search.getRows());
	}
	
	
	/**
	 * 
	 * 分页查询指定SQL语句的数据，不查询总的数据量
	 *
	 * @author warship
	 * @created 2013年12月1日 下午1:33:03
	 *
	 * @param sqlId	SQL语句ID
	 * @param search 查询
	 * @return 分页查询结果PagingResult
	 */
	public <T extends BaseEntity> PagingResult<T> findForPageWithoutTotal(String sqlId, Search search) {
		RowBounds rowBounds = new RowBounds(search.getFirstRowNum(), search.getRows());
		List<T> list = template.selectList(sqlId, getConditionMap(search), rowBounds);
		return new PagingResult<T>(0, list, search.getPage(), search.getRows());
	}
	
	/**
	 * 
	 * 查询指定SQL语句的一条记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:30:44
	 *
	 * @param sqlId	SQL语句ID
	 * @return	查询到的实体
	 */
	public <T extends BaseEntity> T findOne(String sqlId) {
		return template.selectOne(sqlId);
	}

    /**
     *
     * 查询指定SQL语句的一条记录
     *
     * @author warship
     * @created 2013年11月9日 下午7:30:44
     *
     * @param sqlId	SQL语句ID
     * @return	查询到的实体
     */
    public <T extends BaseEntity> T findOne(String sqlId, Object obj) {
        return template.selectOne(sqlId, obj);
    }
	
	/**
	 * 
	 * 根据条件查询指定SQL语句的一条记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:30:15
	 *
	 * @param sqlId	SQL语句ID
	 * @param clazz	返回值类型
	 * @param params	条件参数
	 * @return	查询到的结果
	 */
	public <T extends BaseEntity> T findOne(String sqlId, Map<String, Object> params) {
		return template.selectOne(sqlId, params);
	}
	
	/**
	 * 
	 * 根据条件查询指定SQL语句的一条记录，主要用于关联查询的情况
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:29:55
	 *
	 * @param sqlId	SQL语句ID
	 * @param param	条件参数
	 * @return	查询到的结果
	 */
	public <T extends BaseEntity> T findOne(String sqlId, int param) {
		return template.selectOne(sqlId, param);
	}
	
	/**
	 * 
	 * 根据条件查询指定SQL语句的一条记录，主要用于关联查询的情况
	 *
	 * @author warship
	 * @created 2014年2月9日 下午5:25:47
	 *
	 * @param sqlId	SQL语句ID
	 * @param param	条件参数
	 * @return	查询到的结果
	 */
	public <T extends BaseEntity> T findOne(String sqlId, String param) {
		return template.selectOne(sqlId, param);
	}

	public String getExpId(String sqlId, String param) {
		return template.selectOne(sqlId, param);
	}
	
	public Map<String, Object> findData(String sqlId, Map<String, Object> params) {
		return template.selectOne(sqlId, params);
	}

	/**
	 * 
	 * 查询指定SQL语句的一条记录
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:29:43
	 *
	 * @param sqlId	SQL语句ID
	 * @param param	SQL语句中占位符对应的值
	 * @return	查询到的实体
	 */
	public <T extends BaseEntity> T findOne(String sqlId, Search search) {
		return template.selectOne(sqlId, getConditionMap(search));
	}
	
	/**
	 * 
	 * 查询指定SQL语句所包含的记录数
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:29:12
	 *
	 * @param sqlId	SQL语句ID
	 * @return	记录数
	 */
	public long count(String sqlId) {
		return template.selectOne(sqlId);
	}
	
	/**
	 * 
	 * 查询指定SQL语句所包含的记录数
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:28:58
	 *
	 * @param sqlId	SQL语句ID
	 * @param params	参数
	 * @return	符合条件的记录数
	 */
	public long count(String sqlId, Map<String, Object> params) {
		return template.selectOne(sqlId, params);
	}

	
	 /**
	  * 
	  * 查询指定SQL语句所包含的记录数
	  *
	  * @author warship
	  * @created 2013年11月9日 下午7:28:03
	  *
	 * @param sqlId	SQL语句ID
	 * @param searchModel	查询模型
	 * @return	符合条件的记录数
	  */
	public long count(String sqlId, Search searchModel) {
		return template.selectOne(sqlId, getConditionMap(searchModel));
	}

	
	/**
	 * 
	 * 插入指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:27:40
	 *
	 * @param sqlId	SQL语句ID
	 * @return	插入对象的主键
	 */
	public Object save(String sqlId) {
		return template.insert(sqlId);
	}

	/**
	 * 
	 * 插入指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:27:27
	 *
	 * @param sqlId	SQL语句ID
	 * @param entity	要插入的实体
	 * @return	
	 */
	public <T extends BaseEntity> int save(String sqlId, T entity) {
		return template.insert(sqlId, entity);
	}
	
	/**
	 * 
	 * 插入指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:27:27
	 *
	 * @param sqlId    SQL语句ID
	 * @param entitys    要插入的实体集合
	 * @return
	 */
	public <T extends BaseEntity> int saveBatch(String sqlId, List<MengWangSmsStatusBean> entitys) {
		return template.insert(sqlId, entitys);
	}

	public  String getSelect(String sqlId,String parm){
		return template.selectOne(sqlId,parm);
	};

	/**
	 * 
	 * 更新指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:26:25
	 *
	 * @param sqlId	SQL语句ID
	 * @return	成功更新的记录数
	 */
	public int update(String sqlId) {
		return template.update(sqlId);
	}

	/**
	 * 
	 * 更新指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:26:28
	 *
	 * @param sqlId	SQL语句ID
	 * @param entity	要更新的对象
	 * @return	成功更新的记录数
	 */
	public <T extends BaseEntity> int update(String sqlId, T entity) {
		return template.update(sqlId, entity);
	}
	
	/**
	 * 
	 * 批量更新指定SQL的数据
	 *
	 * @author warship
	 * @created 2014年2月21日 上午10:43:34
	 *
	 * @param sqlId	SQL语句ID
	 * @param entitys	要更新的对象
	 * @return	成功更新的记录数
	 */
	public <T extends BaseEntity> int updateBatch(String sqlId, List<T> entitys) {
		return template.update(sqlId, entitys);
	}
	
	/**
	 * 
	 * 更新指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:25:34
	 *
	 * @param sqlId	SQL语句ID
	 * @param params	参数
	 * @return	成功更新的记录数
	 */
	public int update(String sqlId, Map<String, Object> params) {
		return template.update(sqlId, params);
	}
	
	/**
	 * 
	 * 删除指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:24:54
	 *
	 * @param sqlId	SQL语句ID
	 * @return	成功删除的记录数
	 */
	public int delete(String sqlId) {
		return template.delete(sqlId);
	}
	
	/**
	 * 
	 * 删除指定条件的SQL的数据
	 *
	 * @author warship
	 * @created 2014年2月21日 上午10:41:49
	 *
	 * @param sqlId
	 * @param key	
	 * @return
	 */
	public int delete(String sqlId, Integer key) {
		return template.delete(sqlId, key);
	}
	
	/**
	 * 
	 * 删除指定条件的SQL的数据
	 *
	 * @author warship
	 * @created 2014年2月21日 上午10:41:49
	 *
	 * @param sqlId
	 * @param key	
	 * @return
	 */
	public int delete(String sqlId, String key) {
		return template.delete(sqlId, key);
	}
	
	/**
	 * 
	 * 删除指定SQL的数据
	 *
	 * @author warship
	 * @created 2013年11月9日 下午7:24:23
	 *
	 * @param sqlId	SQL语句ID
	 * @param params	查询参数
	 * @return	成功删除记录数
	 */
	public int delete(String sqlId, Map<String, Object> params) {
		return template.delete(sqlId, params);
	}
	
	/**
	 * 
	 * 以Map形式得到所有查询条件，key为参数名称对应Condition的name，value为参数值对应Condition的value
	 *
	 * @author warship
	 * @created 2013年11月9日 下午11:11:45
	 *
	 * @param search
	 * @return
	 */
	private Map<String, Object> getConditionMap(Search search) {
		Map<String, Object> conditionMap = new HashMap<String, Object>();
		if (search.getConditionList() != null) {
			for (Condition condition : search.getConditionList()) {
				conditionMap.put(condition.getName(), condition.getValue());
			}
		}
		return conditionMap;
	}
	
//	private int getCount(String sqlId, Map<String, Object> params) {
//		MappedStatement ms = template.getConfiguration().getMappedStatement(sqlId);
//		BoundSql boundSql = ms.getBoundSql(params);
////		String databaseId = template.getConfiguration().getDatabaseId();//Microsoft SQL Server
//		StringBuffer countSql = new StringBuffer(boundSql.getSql().length() + 100);
//		
//		String tempSql = boundSql.getSql().toUpperCase();
////		if (databaseId.toUpperCase().equals("MICROSOFT SQL SERVER")) {
////			if (tempSql.contains("ORDER BY")) {
////				tempSql = tempSql.substring(0, tempSql.lastIndexOf("ORDER BY"));
////			}
//			countSql.append("SELECT count(*) FROM (").append(tempSql).append(") TOTAL ").toString();
////		}
//	    try {
//	    	DefaultSqlSession defaultSession = (DefaultSqlSession) SqlSessionUtils.getSqlSession(template.getSqlSessionFactory(), template.getExecutorType(), template.getPersistenceExceptionTranslator());
//			PreparedStatement ps = defaultSession.getConnection().prepareStatement(countSql.toString());
//			DefaultParameterHandler dph = new DefaultParameterHandler(ms, params, boundSql);
//			dph.setParameters(ps);
//			ResultSet rs = ps.executeQuery();
//			int tolalCount = 0;
//			if (rs.next()) {
//				tolalCount = rs.getInt(1);
//			}
//			if(rs!=null){
//				rs.close();
//				rs = null;
//			}
//			return tolalCount;
//		} catch (SQLException e) {
//			e.printStackTrace();
//			return 0;
//		}
//	}
}