/**
 * @FileName: Search.java
 * @Package cn.haliaeetus.framework.common.search
 * 
 * @author warship
 * @created 2013年11月9日 下午10:50:54
 * 
 * Copyright 2013-2050 佰颂物联 版权所有
 */
package cn.haliaeetus.sendmessage.framework.mybatis.search;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>查询类</p>
 * 
 * @author warship
 * @since 1.0
 * @version 1.0
 */
public class Search implements Serializable {

	private static final long serialVersionUID = -328270505586112262L;
	
	/**
	 * 当前页码
	 */
	private int page = 1;
	
	/**
	 * 每页记录数
	 */
	private int rows = 20;
	
	/**
	 * 查询条件组
	 */
	private List<Condition> conditionList = new ArrayList<>();
	
	/**
	 * 排序
	 */
	private List<Sort> sortList;

	/**
	 * @return the page
	 */
	public int getPage() {
		return page;
	}

	/**
	 * @param page the page to set
	 */
	public void setPage(int page) {
		this.page = page;
		
		if (page < 1) {
			this.page = 1;
		}
	}
	
	/**
	 * @return the rows
	 */
	public int getRows() {
		return rows;
	}

	/**
	 * @param pageSize the pageSize to set
	 */
	public void setRows(int rows) {
		this.rows = rows;
		
		if (rows < 1) {
			this.rows = 1;
		}
	}

	/**
	 * @return the conditionList
	 */
	public List<Condition> getConditionList() {
		return conditionList;
	}

	/**
	 * @param conditionList the conditionList to set
	 */
	public void setConditionList(List<Condition> conditionList) {
		this.conditionList = conditionList;
	}
	
	/**
	 * @return the sortList
	 */
	public List<Sort> getSortList() {
		return sortList;
	}

	/**
	 * @param sortList the sortList to set
	 */
	public void setSortList(List<Sort> sortList) {
		this.sortList = sortList;
	}

	/**
	 * 
	 * 得到当前页第一行数据的行号
	 *
	 * @author warship
	 * @created 2013年11月9日 下午11:06:43
	 *
	 * @return
	 */
	public int getFirstRowNum() {
		return ((page - 1) * rows);
	}
	
	public boolean containsRouterCall() {
		if (null == conditionList || null == getRouterName() || null == getMethods() 
				|| null == getConfirmMethods() || null == getGridId()) {
			return false;
		}
		return true;
	}
	
	public String getRouterName() {
		for (Condition cond : conditionList) {
			if ("routerName".equals(cond.getName())) {
				return (String)cond.getValue();
			}
		}
		return null;
	}
	
	public String[] getMethods() {
		for (Condition cond : conditionList) {
			if ("methods".equals(cond.getName())) {
				return ((String)cond.getValue()).split(",");
			}
		}
		return null;
	}
	
	public String[] getConfirmMethods() {
		for (Condition cond : conditionList) {
			if ("confirmMethods".equals(cond.getName())) {
				return ((String)cond.getValue()).split(",");
			}
		}
		return null;
	}
	
	public String getGridId() {
		for (Condition cond : conditionList) {
			if ("gridId".equals(cond.getName())) {
				return (String)cond.getValue();
			}
		}
		return null;
	}

}
