package cn.haliaeetus.sendmessage.framework.sms;

import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.constants.SMSConfig;
import cn.haliaeetus.sendmessage.framework.base.crawler.CrawlResponse;
import cn.haliaeetus.sendmessage.framework.base.crawler.Crawler;
import cn.haliaeetus.sendmessage.framework.base.digt.EscapeUtil;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.sms.entity.CmppResponse;
import cn.haliaeetus.sendmessage.framework.sms.entity.HttpResponse;
import cn.haliaeetus.sendmessage.framework.sms.entity.MsgDto;
import cn.haliaeetus.sendmessage.framework.sms.entity.qidian.Base64;
import cn.haliaeetus.sendmessage.framework.sms.entity.qidian.RSA;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.utils._;
import com.google.gson.JsonObject;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.Consts;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StreamUtils;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 短信服务工具类
 */
public class SMSHelper {
    private static final Logger LOG = LoggerFactory.getLogger(SMSHelper.class);

    private static final Crawler CRAWLER = Crawler.of("SMSHelper");

    private static String consume(HttpEntity entity) {
        String result = "";
        try {
            InputStream inputStream = entity.getContent();
            result = StreamUtils.copyToString(inputStream, Charset.forName("UTF-8"));
            result = URLDecoder.decode(result, "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }
    
    /**
     * 调用云信短信平台发送短信
     */
    public static String sendSMS_YX(String phones, String content) {
    	
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("userCode", SMSConfig.getYunXinSmsUserCode()));
    	params.add(new BasicNameValuePair("userPass", SMSConfig.getYunXinSmsUserPass()));
    	params.add(new BasicNameValuePair("DesNo", phones));
    	params.add(new BasicNameValuePair("Msg", content));
    	params.add(new BasicNameValuePair("Channel", ""));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getYunXinSmsSendUrl(), entity);
    	LOG.info("SMS send Yunxin: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getYunXinSmsSendUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用云信短信平台获取状态报告
     */
    public static String reportStatus_YX() {
    	
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("userCode", SMSConfig.getYunXinSmsUserCode()));
    	params.add(new BasicNameValuePair("userPass", SMSConfig.getYunXinSmsUserPass()));
    	params.add(new BasicNameValuePair("batchNumber", ""));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getYunXinSmsReportUrl(), entity);
    	LOG.info("SMS report status Yunxin: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getYunXinSmsReportUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用云信短信平台余额查询
     */
    public static String getBalance_YX() {
    	
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("userCode", SMSConfig.getYunXinSmsUserCode()));
    	params.add(new BasicNameValuePair("userPass", SMSConfig.getYunXinSmsUserPass()));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getYunXinSmsBalanceUrl(), entity);
    	LOG.info("SMS get balance Yunxin: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getYunXinSmsBalanceUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用快鸽短信平台发送短信
     */
    public static String sendSMS_KG(String phones, String content) {
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("action", "send"));
    	params.add(new BasicNameValuePair("userid", SMSConfig.getKuaigeSmsUserid()));
    	params.add(new BasicNameValuePair("account", SMSConfig.getKuaigeSmsAccount()));
    	params.add(new BasicNameValuePair("password", SMSConfig.getKuaigeSmsPassword()));
    	params.add(new BasicNameValuePair("mobile", phones));
    	params.add(new BasicNameValuePair("content", content));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getKuaigeSmsSendUrl(), entity);
    	LOG.info("SMS send Kuaige: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getKuaigeSmsSendUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用快鸽短信平台获取状态报告
     */
    public static String reportStatus_KG() {
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("action", "query"));
    	params.add(new BasicNameValuePair("userid", SMSConfig.getKuaigeSmsUserid()));
    	params.add(new BasicNameValuePair("account", SMSConfig.getKuaigeSmsAccount()));
    	params.add(new BasicNameValuePair("password", SMSConfig.getKuaigeSmsPassword()));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getKuaigeSmsReportUrl(), entity);
    	LOG.info("SMS report status Kuaige: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getKuaigeSmsReportUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用快鸽短信平台获取余额及已发送量查询
     */
    public static String overageQuery_KG() {
    	final List<NameValuePair> params = new ArrayList<>();
    	params.add(new BasicNameValuePair("action", "overage"));
    	params.add(new BasicNameValuePair("userid", SMSConfig.getKuaigeSmsUserid()));
    	params.add(new BasicNameValuePair("account", SMSConfig.getKuaigeSmsAccount()));
    	params.add(new BasicNameValuePair("password", SMSConfig.getKuaigeSmsPassword()));
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getKuaigeSmsSendUrl(), entity);
    	LOG.info("SMS overage query Kuaige: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getKuaigeSmsReportUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 根据物流单号调用快宝短信平台查看该单号是否可发送短信
     * 
     * @param wayBillNo 单号
     * @param brand 快递商
     */
    public static String checkWayBillNoMobile_KB(String wayBillNo, String brand) {
    	final List<NameValuePair> params = new ArrayList<>();
    	Date now = new Date();
    	params.add(new BasicNameValuePair("app_id", SMSConfig.getKuaibaoApiAppId()));
    	params.add(new BasicNameValuePair("ts", String.valueOf(now.getTime())));
    	params.add(new BasicNameValuePair("sign", sign(SMSConfig.getKuaibaoApiAppId(), SMSConfig.getKuaibaoApiNameCheck(), now.getTime(), SMSConfig.getKuaibaoApiAppKey())));
    	
    	// 业务参数json串
    	JsonObject obj = new JsonObject();
    	obj.addProperty("wayBillNo", wayBillNo);
    	obj.addProperty("brand", brand);
    	params.add(new BasicNameValuePair("data", obj.toString()));
    	
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getKuaibaoApiUrl() + SMSConfig.getKuaibaoApiNameCheck(), entity);
    	LOG.info("SMS check wayBillNo kuaibao: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getKuaibaoApiUrl() + SMSConfig.getKuaibaoApiNameCheck(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 根据物流单号调用快宝短信平台发送短信
     * 
     * @param wayBillNo 单号
     * @param brand 快递商
     * @param content 短信内容
     */
    public static String sendSmsByWayBillNo_KB(String wayBillNo, String brand, String content) {
    	final List<NameValuePair> params = new ArrayList<>();
    	Date now = new Date();
    	params.add(new BasicNameValuePair("app_id", SMSConfig.getKuaibaoApiAppId()));
    	params.add(new BasicNameValuePair("ts", String.valueOf(now.getTime())));
    	params.add(new BasicNameValuePair("sign", sign(SMSConfig.getKuaibaoApiAppId(), SMSConfig.getKuaibaoApiNameSend(), now.getTime(), SMSConfig.getKuaibaoApiAppKey())));
    	
    	// 业务参数json串
    	JsonObject obj = new JsonObject();
    	obj.addProperty("wayBillNo", wayBillNo);
    	obj.addProperty("brand", brand);
    	obj.addProperty("content", content);
    	params.add(new BasicNameValuePair("data", obj.toString()));
    	
    	UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getKuaibaoApiUrl() + SMSConfig.getKuaibaoApiNameSend(), entity);
    	LOG.info("SMS send sms by wayBillNo kuaibao: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getKuaibaoApiUrl() + SMSConfig.getKuaibaoApiNameSend(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 生成快宝短信平台签名
     */
    private static String sign(String appId, String apiName, long ts, String appKey) {
    	return Md5Util.md5AsLowerHex(appId + apiName + ts + appKey);
    }
    
    /**
     * 处理韵达单号，去除校验位，取前13位
     * 
     * @param expCode 单号
     * @param expId 快递Id
     * @return 处理后的单号
     */
    public static String subStrYunDaExpCode(String expCode, int expId) {
    	if (_.isEmpty(expCode)) {
    		return "";
    	}
    	
    	if (expId == Constants.EXPRESS_TYPE_YD && expCode.length() > 13) {
    		return StringUtils.substring(expCode, 0, 13);
    	}
    	
    	return expCode;
    }
    
    /**
     * 调用大汉三通短信平台发送短信
     */
    public static String sendSMS_DH(String phones, String content, String msgid) {
    	
    	final JsonObject obj = new JsonObject();
    	obj.addProperty("account", SMSConfig.getDahanSmsAccount());
    	obj.addProperty("password", Md5Util.md5AsLowerHex(SMSConfig.getDahanSmsPassword()));
    	obj.addProperty("msgid", msgid);
    	obj.addProperty("phones", phones);
    	obj.addProperty("content", content.replaceAll("【[\\S]+】$", "")); // 将短信内容中的签名去掉
    	obj.addProperty("sign", "");
    	obj.addProperty("subcode", "");
    	
    	StringEntity entity = new StringEntity(obj.toString(), Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getDahanSmsSendUrl(), entity);
    	LOG.info("SMS send dahan: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getDahanSmsSendUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用大汉三通短信平台获取状态报告
     */
    public static String reportStatus_DH() {
    	
    	final JsonObject obj = new JsonObject();
    	obj.addProperty("account", SMSConfig.getDahanSmsAccount());
    	obj.addProperty("password", Md5Util.md5AsLowerHex(SMSConfig.getDahanSmsPassword()));
    	
    	StringEntity entity = new StringEntity(obj.toString(), Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getDahanSmsReportUrl(), entity);
    	LOG.info("SMS report status dahan: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getDahanSmsReportUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用大汉三通短信平台查询余额
     */
    public static String getBalance_DH() {
    	
    	final JsonObject obj = new JsonObject();
    	obj.addProperty("account", SMSConfig.getDahanSmsAccount());
    	obj.addProperty("password", Md5Util.md5AsLowerHex(SMSConfig.getDahanSmsPassword()));
    	
    	StringEntity entity = new StringEntity(obj.toString(), Consts.UTF_8);
    	Pair<Integer, String> result = CRAWLER.post(SMSConfig.getDahanSmsBalanceUrl(), entity);
    	LOG.info("SMS get balance dahan: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMSConfig.getDahanSmsBalanceUrl(), consume(entity), result.getLeft(), result.getRight());
      return result.getRight();
    }
    
    /**
     * 调用流驿短信平台发送短信
     * @throws UnsupportedEncodingException 
     */
    public static String sendSMS_LY(String phones, String content, String msgid) throws UnsupportedEncodingException {

		// 计算签名，注意这里的消息内容转字节时，编码对应dc=15
		byte[] messageBytes = (SMSConfig.getLiuyiSmsSign() + content).getBytes("GB18030");
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		String timestamp = sdf.format(new Date());
		String sign = null;
		try {
			sign = calculateSign(SMSConfig.getLiuyiSmsUserName(), SMSConfig.getLiuyiSmsPassword(), timestamp, messageBytes);
		} catch (Exception e) {
			e.printStackTrace();
			LOG.error("LiuYi SMS Send Error :" + e.getMessage());
			return null;
		}

		// 装配GET所需的参数
		StringBuilder sb = new StringBuilder(2000);
		sb.append(SMSConfig.getLiuyiSmsSendUrl());
		sb.append("/mt?dc=15"); // 表明发送的是中文dc=15
		sb.append("&sm=").append(Hex.encodeHexString(messageBytes)); // HEX方式
		sb.append("&da=").append(phones);
		sb.append("&sa=").append(msgid);
		sb.append("&un=").append(SMSConfig.getLiuyiSmsUserName());
		sb.append("&pw=").append(URLEncoder.encode(sign, "utf8"));// 这里使用签名,不是密码
		sb.append("&ts=").append(timestamp); // 指示服务器使用签名(数字摘要)验证方式
		sb.append("&tf=0"); // 表示短信内容为 HEX
		sb.append("&rd=1"); // 需要状态报告
		sb.append("&rf=2"); // 控制返回格式，0：默认格式，即文本格式。1：xml  2：json 格式

		String request = sb.toString();
		System.out.println(request);

		// 以GET方式发起请求
		CrawlResponse result = CRAWLER.getResp(request);
		System.out.println("response: " + result.content);

		return result.content;
    }
    /**
     * 调用奇点云短信平台发送短信
     * @throws UnsupportedEncodingException
     */
    public static String sendSMS_QD(String phones, String content, String msgid) throws UnsupportedEncodingException {
		/**
		 * 组装params
		 */

		StringBuilder sb = new StringBuilder(2000);
		sb.append("&").append(phones);
		sb.append("&").append(content);
		sb.append("&").append(URLEncoder.encode(SMSConfig.getQiDianSmsSign(), "utf-8"));
		sb.append("&").append(SMSConfig.getQiDianTempCode());
		sb.append("&").append(URLEncoder.encode(SMSConfig.getQiDianSmsSign(), "utf-8"));
		sb.append("&").append(System.currentTimeMillis());

		String sign = null;
		byte[] data = sb.toString().getBytes();
		byte[] encodedData = null;
		try {
			encodedData = RSA.encryptByPrivateKey(data, SMSConfig.getQiDianSmsPrivateKey());
			sign = RSA.sign(encodedData, SMSConfig.getQiDianSmsPrivateKey()) + "&" + SMSConfig.getQiDianSmsPrivateKey();
			sign = Base64.encode(sign.getBytes("UTF-8"));

			Header[] headers = new Header[2];
			headers[0] = new BasicHeader("Accept", "application/json");
			headers[1] = new BasicHeader("Content-Type", "application/json");

			JsonObject entity = new JsonObject();
			entity.addProperty("sign", sign);
			entity.addProperty("params", sb.toString());

			CrawlResponse resp = CRAWLER.postResp(SMSConfig.getQiDianSmsSendUrl(), entity.toString(), headers,
					EscapeUtil.CHARSET_UTF8);
			if (resp.httpCode / 100 == 2) {
				return resp.content;
			}
		} catch (Exception e) {
			LOG.error("QiDian SMS Send Error :" + e.getMessage());
			e.printStackTrace();
		}

		return null;
    }
    
    /**
     * 调用流驿短信平台获取状态报告
     * @throws UnsupportedEncodingException 
     */
    public static String reportStatus_LY() throws UnsupportedEncodingException {
    	
    	int fetchSize = 100; // 每次返回的最大条数
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    	String timestamp = sdf.format(new Date() );
    	String sign = null;
    	try {
			sign = calculateSign(SMSConfig.getLiuyiSmsUserName(), SMSConfig.getLiuyiSmsPassword(), timestamp, null);
		} catch (Exception e) {
			e.printStackTrace();
			LOG.error("LiuYi SMS Send Error :" + e.getMessage());
			return null;
		}
    	
		StringBuilder sb = new StringBuilder();
		sb.append(SMSConfig.getLiuyiSmsSendUrl() + "/mo?")
			.append("un=").append(SMSConfig.getLiuyiSmsUserName())
			.append("&pw=").append(URLEncoder.encode(sign, "utf8"))
			.append("&fs=").append(fetchSize)
			.append("&ts=").append(timestamp)
			.append("&rf=").append(2); // 控制返回格式，0：默认格式，即文本格式。1：xml  2：json 格式

    	String req = sb.toString();
		System.out.println("request: " + req);
		CrawlResponse result = CRAWLER.getResp(req);
		System.out.println("response: " + result.content);
	
    	return result.content;
    }
    /**
     * 调用奇点云短信平台获取状态报告
     * @throws UnsupportedEncodingException 
     */
    public static String reportStatus_QD() throws UnsupportedEncodingException {
    	
    	int fetchSize = 100; // 每次返回的最大条数
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    	
    	String timestamp = sdf.format(new Date() );
    	String sign = null;
    	try {
			sign = calculateSign(SMSConfig.getLiuyiSmsUserName(), SMSConfig.getLiuyiSmsPassword(), timestamp, null);
		} catch (Exception e) {
			e.printStackTrace();
			LOG.error("LiuYi SMS Send Error :" + e.getMessage());
			return null;
		}
    	
		StringBuilder sb = new StringBuilder();
		sb.append(SMSConfig.getLiuyiSmsSendUrl() + "/mo?")
			.append("un=").append(SMSConfig.getLiuyiSmsUserName())
			.append("&pw=").append(URLEncoder.encode(sign, "utf8"))
			.append("&fs=").append(fetchSize)
			.append("&ts=").append(timestamp)
			.append("&rf=").append(2); // 控制返回格式，0：默认格式，即文本格式。1：xml  2：json 格式

    	String req = sb.toString();
		System.out.println("request: " + req);
		CrawlResponse result = CRAWLER.getResp(req);
		System.out.println("response: " + result.content);
	
    	return result.content;
    }
    
    
    public static void main(String[] args) throws UnsupportedEncodingException {
		
    	/*try {
			//System.out.println(sendSMS_LY("17765151930","您的验证码是：11122,十分钟内有效。", "sf"));
			String msg = reportStatus_LY();
			
			
			LYReportResponse reportStatusResponse = (LYReportResponse) JSON.parseObject(msg, LYReportResponse.class);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		sendSMS_QD("17765151930", "8888", null);
    	
	}
    
  
	/**
	 * 计算签名(摘要)
	 */
	public static String calculateSign(String username, String password, String timestamp, byte[] message)
			throws Exception {
		MessageDigest md = MessageDigest.getInstance("MD5");
		md.update(username.getBytes("UTF8"));
		md.update(password.getBytes("UTF8"));
		md.update(timestamp.getBytes("UTF8"));
		if(null != message){
			md.update(message);
		}
		
		byte[] md5result = md.digest();
		return org.apache.commons.codec.binary.Base64.encodeBase64String(md5result);
	}

	public static String sendSMS_WX(String comSign,String phones, String content, String msgid) throws UnsupportedEncodingException {
		String res="";
		try{
			String key=SMSConfig.getWuXiSmsSendKey();
			String uri=SMSConfig.getWuXiSmsSendUrl();
			MsgDto msgDto=new MsgDto();
			msgDto.setUserName(SMSConfig.getWuXiSmsUserName());
			msgDto.setPassWord(getMDtLower32(SMSConfig.getWuXiSmsPassword()));
			msgDto.setContent(content);
			msgDto.setComSign(comSign);
			msgDto.setPhoneNumber(phones);
			msgDto.setExtCode("");
			msgDto.setMd5Content(Md5Util.MD5Encode(content.concat(key),"UTF-8"));
			String resStr=OkHttpUtil.postJson(uri,msgDto);
			System.out.println("+++++++++++++++++++++++++++++++++++"+resStr);
			HttpResponse response=JsonUtils.parse(resStr,HttpResponse.class);
			if(response.getCode().equals("0")){
				//res= EntityUtils.toString(httpResponse.getEntity());
				res=resStr;
				System.out.println(resStr);
			}
		}catch (Exception ex){
			System.out.println("发送失败:"+ex.getMessage());
			res=ex.getMessage();
		}
		return  res;
	}


	public static String getMDtLower32(String str) throws Exception{
		String result;
		MessageDigest md5 = MessageDigest.getInstance("MD5");
		md5.update((str).getBytes("UTF-8"));
		byte b[] = md5.digest();

		int i;
		StringBuffer buf = new StringBuffer("");

		for(int offset=0; offset<b.length; offset++){
			i = b[offset];
			if(i<0){
				i+=256;
			}
			if(i<16){
				buf.append("0");
			}
			buf.append(Integer.toHexString(i));
		}

		result = buf.toString();
		return result;
	}


	public static String sendSMS_QuanDi(String comSign,String phones, String content, String msgid) throws UnsupportedEncodingException {
		String res="";
		try{
			String uri="http://message.mijihome.cn/MessageService/Message/sendMessage";
			HttpPost post=new HttpPost(uri);
			List<NameValuePair> nvps=new ArrayList<NameValuePair>();
			nvps.add(new BasicNameValuePair("appkey","xiaomai"));
			nvps.add(new BasicNameValuePair("name",comSign));
			nvps.add(new BasicNameValuePair("text",content));
			SimpleDateFormat sdf=new SimpleDateFormat("yyyyMMddHHmm");
			Date currentTime = new Date();
			String dateString = sdf.format(currentTime);
			nvps.add(new BasicNameValuePair("phone",phones));
			nvps.add(new BasicNameValuePair("sign", getMDtLower32("xiaomai"+content+"5ef46026a47f70f8")));
			nvps.add(new BasicNameValuePair("sendtime",dateString));
			nvps.add(new BasicNameValuePair("uid",""));
			UrlEncodedFormEntity entity=new UrlEncodedFormEntity(nvps,"utf-8");
			post.setEntity(entity);
			CloseableHttpResponse httpResponse= HttpClients.createDefault().execute(post);
			int statusCode=httpResponse.getStatusLine().getStatusCode();
			if(statusCode==200){
				res= EntityUtils.toString(httpResponse.getEntity());
				System.out.println(res);
			}
		}catch (Exception ex){
			System.out.println("发送失败:"+ex.getMessage());
			res=ex.getMessage();
		}
		return  res;
	}



	public static String sendSMS_CMPP(String comSign,String phones, String content, String msgid) throws UnsupportedEncodingException {
		String key="3ba2c284-ba6b-41a8-8674-9a600165c2dds";
		String res="";
		try{
			//TODO
			String uri=SMSConfig.getCmppSmsSendUrl();
			MsgDto msgDto=new MsgDto();
			msgDto.setUserName(SMSConfig.getCmppSmsUserName());//TODO
			msgDto.setPassWord(getMDtLower32(SMSConfig.getCmppSmsPassword()));//TODO
			msgDto.setContent(content);
			msgDto.setComSign(comSign);
			msgDto.setPhoneNumber(phones);
			msgDto.setExtCode("");
			msgDto.setMd5Content(Md5Util.MD5Encode(content.concat(key),"UTF-8"));
			String resStr=OkHttpUtil.postJson(uri,msgDto);
			System.out.println("+++++++++++++++++++++++++++++++++++"+resStr);
			CmppResponse response=JsonUtils.parse(resStr,CmppResponse.class);
			//CloseableHttpResponse httpResponse= HttpClients.createDefault().execute(post);

			//int statusCode=httpResponse.getStatusLine().getStatusCode();
			if(response.getCode().equals("200")){
				//res= EntityUtils.toString(httpResponse.getEntity());
				res=resStr;
				System.out.println(resStr);
			}
		}catch (Exception ex){
			System.out.println("发送失败:"+ex.getMessage());
			res=ex.getMessage();
		}
		return  res;
	}
}