package cn.haliaeetus.sendmessage.framework.toolkit;

import cn.haliaeetus.sendmessage.bean.basedata.*;
import cn.haliaeetus.sendmessage.bean.config.PlatformConfig;
import cn.haliaeetus.sendmessage.bean.dto.PhoneLibDto;
import cn.haliaeetus.sendmessage.bean.jspo.Configuration;
import cn.haliaeetus.sendmessage.bean.sms.SMSFlag;
import cn.haliaeetus.sendmessage.framework.toolkit.redis.JedisClient;
import cn.haliaeetus.sendmessage.utils.SpringContextUtil;
import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.WebApplicationContext;

import java.util.List;
import java.util.Set;


/**
 * 缓存数据的操作类
 * @author Administrator
 *
 */
@Service
public class CacheTool {

	@Autowired
	public JedisClient jedisClient;
	
	private String jedisKey;
	private Class className;
	
	private static final String CACHE_NAMESPACE = "haliaeetus:";
    private static final String CACHB_DX = "smsSender";
	private static final String SMSCOUNT = "SMSCOUNT";
	private static final String PDD = "pdd";
	private static final String JI_TU = "JITU:";

//	private static final SessionBusiness storeBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"S");
//	private static final SessionBusiness expressBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"E");
//	private static final SessionBusiness typeBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"T");
	
	public CacheTool(String Hashkey, Class className){
		this.jedisKey = Hashkey;
		this.className = className;
	}

    public CacheTool(String Hashkey){
        this.jedisKey = Hashkey;
    }

	private static CacheTool storeTool = new CacheTool(CACHE_NAMESPACE + "S",Store.class);
	private static CacheTool expressTool = new CacheTool(CACHE_NAMESPACE + "E",ExpressType.class); 
	private static CacheTool typeLibraryTool = new CacheTool(CACHE_NAMESPACE + "T", TypeLibrary.class); 
	private static CacheTool userTool = new CacheTool(CACHE_NAMESPACE + "U", User.class);
	private static CacheTool storeShelfTool = new CacheTool(CACHE_NAMESPACE + "SS", StoreShelf.class);
	private static CacheTool shelfTool = new CacheTool(CACHE_NAMESPACE + "ST", StoreShelf.class);
	private static CacheTool provinceTool = new CacheTool(CACHE_NAMESPACE + "P", Province.class);
	private static CacheTool cityTool = new CacheTool(CACHE_NAMESPACE + "C", City.class);
	private static CacheTool areaTool = new CacheTool(CACHE_NAMESPACE + "A", Area.class);
	private static CacheTool menuTool = new CacheTool(CACHE_NAMESPACE+"M", Menu.class);
	private static CacheTool phoneTool = new CacheTool(CACHE_NAMESPACE+"PHONE", PhoneLibDto.class);
	private static CacheTool configurationTool = new CacheTool(CACHE_NAMESPACE + "O",Configuration.class);
	private static CacheTool limitTool = new CacheTool(CACHE_NAMESPACE + "LI",StoreReqLimit.class);
    private static CacheTool smsFlagTool = new CacheTool(CACHB_DX );
	private static CacheTool smsCount = new CacheTool(SMSCOUNT + "SMSCOUNT",Long.class);
	private static CacheTool pddDeli = new CacheTool(PDD);
	private static CacheTool platformConfigTool = new CacheTool(CACHE_NAMESPACE + "pfc",PlatformConfig.class);
	private static CacheTool jiTuCacheTool = new CacheTool(JI_TU);

	public CacheTool(){
		
	}
	
	public static CacheTool initStore() {   
        return storeTool;    
    }

	public static CacheTool initConfiguration() {
		return configurationTool;
	}

	public static CacheTool initLimit() {
		return limitTool;
	}

	public static CacheTool initExpress(){
		 return expressTool;    
	}
	public static CacheTool initTypeLibrary(){
		 return typeLibraryTool;    
	}
	public static CacheTool initUser(){
		 return userTool;    
	}
	public static CacheTool initStoreShelf(){
		 return storeShelfTool;    
	}
	public static CacheTool initShelf(){
		 return shelfTool;    
	}
	public static CacheTool initProvince(){
		 return provinceTool;    
	}
	public static CacheTool initCity(){
		 return cityTool;    
	}
	public static CacheTool initArea(){
		 return areaTool;    
	}
	public static CacheTool initMenu(){
		return menuTool;
	}
	public static CacheTool initPhone(){
		return phoneTool;
	}
	public static CacheTool initSmsCount(){
		return smsCount;
	}
	public static CacheTool initPddDeli(){
		return pddDeli;
	}
	public static CacheTool initPlatformConfig(){
		return platformConfigTool;
	}
	public static CacheTool initJiTuCacheTool(){
		return jiTuCacheTool;
	}

	public static CacheTool initSmsFlagTool(){
		return smsFlagTool;
	}
	/**
	 * 根据key获取对象
	 */
	public Object getBean(String key ){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		String json = jedisClient.hget(this.jedisKey, key);
		if("0".equals(key)){
			//map or list
			return json;
		}else{
			return JSON.parseObject(json, this.className);
		}
	}
	
	/**
	 * 根据key获取对象
	 * @param <T>
	 */
	public <T> List<T> getBean(String key, Class<T> clazz){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		String json = jedisClient.hget(this.jedisKey, key);
		return JSON.parseArray(json, clazz);
		
	}
	
	
	
	/**
	 * 保存被序列化后的对象
	 */
	public boolean saveBean(String key, Object value){
		long ret = -1;
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		
		if(null == value){
			//删除操作
			ret = jedisClient.hdel(this.jedisKey, key);
		}
		else{
			String json = JSON.toJSONString(value);
			ret = jedisClient.hset(this.jedisKey, key, json);
		}
		
		return ret == 0 ? true: false;
	}

	/**
	 * 根据key获取String
	 */
	public String getString(String key ){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.hget(this.jedisKey, key);
	}

	/**
	 * 自增
	 * @param key
	 * @return
	 */
	public Long incrBy(String key,Integer num){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.incrBy(this.jedisKey+":"+key,num);
	}

    /**
     * 根据key获取对象String
     */
    public String getStringKey(String key){
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        return jedisClient.get(this.jedisKey+":"+key);
    }

    /**
     * set对象String
     */
    public void setStringKey(String key,String value,Integer dataTime){
        long ret = -1;
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        jedisClient.set(this.jedisKey+":"+key, value, dataTime);
    }

    /**
     * set对象Sting
     */
    public void setStringKey(String key,String value){
        long ret = -1;
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        jedisClient.set(this.jedisKey+":"+key, value);
    }

	/**
	 * 获取所有key
	 * @return
	 */
    public Set<byte[]> getKeys(){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.keys(this.jedisKey+"*");
	}

	/**
	 * 根据key删除数据
	 * @return
	 */
	public void del(String key){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		jedisClient.del(this.jedisKey+":"+key);
	}

	/**
	 * set对象String并获取过期时间
	 */
	public void setStringandDatakey(String key,String value,Integer dataTime){
		long ret = -1;
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		jedisClient.set(this.jedisKey+":"+key, value, dataTime);
	}

}
