package cn.haliaeetus.sendmessage.framework.util;

import cn.haliaeetus.sendmessage.framework.log.Logger;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.*;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.util.StringUtils;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * Http工具类
 * </p>
 *
 * @author sqj
 * @version 1.0
 * @since 1.0
 */
public class HttpUtil {
    private static Logger LOG = Logger.getInstance(HttpUtil.class);
    private final static int CONNECT_TIMEOUT = 5000; // in milliseconds
    private final static String DEFAULT_ENCODING = "UTF-8";
    /**
     * 发送GET请求
     *
     * @param url 请求URL
     * @return
     * @author sqj
     * @created 2013年11月27日 上午8:46:51
     */
    public static String get(String url) {
        LOG.record(String.format("Http Get start, URL[%s].", url));
        if (!RegexUtil.isValidURL(url)) {
            return null;
        }

        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            HttpResponse response = httpClient.execute(new HttpGet(url));
            int status = response.getStatusLine().getStatusCode();
            String content = "";
            if (status == HttpStatus.SC_OK) {
                HttpEntity entity = response.getEntity();
                BufferedReader reader = new BufferedReader(new InputStreamReader(entity.getContent(), "UTF-8"));
                String tmp;
                while ((tmp = reader.readLine()) != null) {
                    content += tmp + "\r\n";
                }
            }
            LOG.record(String.format("Http Get finish, URL[%s], Response[status: %s, content: %s].", url, status, content));
            return content;
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 发送GET请求
     *
     * @param url    请求URL
     * @param params 请求参数
     * @return
     * @author sqj
     * @created 2013年11月27日 上午8:46:51
     */
    public static String get(String url, Map<String, String> params) {
        if (!RegexUtil.isValidURL(url) || params == null || params.size() == 0) {
            return null;
        }

        url += "?";
        Set<String> keys = params.keySet();
        for (String key : keys) {
            url += key + "=" + params.get(key) + "&";
        }
        url = url.substring(0, url.length() - 1);

        return get(url);
    }

    /**
     * 发送POST请求
     *
     * @param url    请求URL
     * @param params 请求参数
     * @return
     * @throws IOException
     * @throws ClientProtocolException
     * @author sqj
     * @created 2014-4-9 下午1:10:01
     */
    public static String post(String url, Map<String, String> params) {
        LOG.record(String.format("Http Post start, URL[%s], Params[%s].", url, params));
        try {
            if (!RegexUtil.isValidURL(url) || params == null || params.size() == 0) {
                return null;
            }

            Set<String> keys = params.keySet();
            List<NameValuePair> paramList = new ArrayList<NameValuePair>();
            for (String key : keys) {
                NameValuePair nameValuePair = new BasicNameValuePair(key, params.get(key));
                paramList.add(nameValuePair);
            }

            UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(paramList, Consts.UTF_8);
            String response = doPost(url, formEntity);
            LOG.record(String.format("Http Post finish, URL[%s], Params[%s], Response[%s].", url, params, response));
            return response;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return "";
    }

    public static String post(String url, Map<String, String> params, int timeout) {
        LOG.record(String.format("Http Post start, URL[%s], Params[%s].", url, params));
        try {
            if (!RegexUtil.isValidURL(url) || params == null || params.size() == 0) {
                return null;
            }

            Set<String> keys = params.keySet();
            List<NameValuePair> paramList = new ArrayList<NameValuePair>();
            for (String key : keys) {
                NameValuePair nameValuePair = new BasicNameValuePair(key, params.get(key));
                paramList.add(nameValuePair);
            }

            UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(paramList, Consts.UTF_8);
            String response = doPost(url, formEntity, timeout);
            LOG.record(String.format("Http Post finish, URL[%s], Params[%s], Response[%s].", url, params, response));
            return response;
        } catch (Exception ex) {
            ex.printStackTrace();
            LOG.record(String.format("Http Post error, URL[%s], Params[%s].", url, params),ex);
        }
        return "";
    }
    /**
     * 发送POST请求
     *
     * @param url     请求URL
     * @param content 请求实体内容
     * @return
     * @throws IOException
     * @throws ClientProtocolException
     * @author sqj
     * @created 2014-4-9 下午1:10:01
     */
    public static String post(String url, String content) {
        LOG.record(String.format("Http Post start, URL[%s], ParamContent[%s].", url, content));
        try {
            if (!RegexUtil.isValidURL(url) || !StringUtils.hasText(content)) {
                return null;
            }

            ByteArrayInputStream bais = new ByteArrayInputStream(content.getBytes("UTF-8"));
            InputStreamEntity inputStreamEntity = new InputStreamEntity(bais);
            String response = doPost(url, inputStreamEntity);
            LOG.record(String.format("Http Post finish, URL[%s], ParamContent[%s], Response[%s].", url, content, response));
            return response;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return "";
    }

    public static String postKV(String url, String content) throws Exception {
        LOG.record(String.format("Http Post start, URL[%s], ParamContent[%s].", url, content));
        if (!RegexUtil.isValidURL(url) || !StringUtils.hasText(content)) {
            return null;
        }

        List<NameValuePair> paramList = new ArrayList<NameValuePair>();
        String[] args = content.split("&");
        for (String arg : args) {
            if (!StringUtils.isEmpty(arg)) {
                String[] kv = arg.split("=");
                if (kv != null && kv.length == 2) {
                    NameValuePair nameValuePair = new BasicNameValuePair(kv[0], kv[1]);
                    paramList.add(nameValuePair);
                }
            }
        }
        UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(paramList, Consts.UTF_8);
        String response = doPost(url, formEntity);
        LOG.record(String.format("Http Post finish, URL[%s], ParamContent[%s], Response[%s].", url, content, response));
        return response;
    }

    @SuppressWarnings("unchecked")
    public static String postJSON(String url, String json) throws Exception {
        if (!RegexUtil.isValidURL(url) || !StringUtils.hasText(json)) {
            return null;
        }

        Map<String, String> params = JSONObject.parseObject(json, Map.class);
        return post(url, params);
    }

    public static String doPost(final String url, HttpEntity httpEntity) throws IOException {
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(httpEntity);

        ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
            public String handleResponse(HttpResponse response) throws IOException {
                int status = response.getStatusLine().getStatusCode();
                String content = "";
                if (status >= HttpStatus.SC_OK && status < HttpStatus.SC_MULTIPLE_CHOICES) {
                    HttpEntity entity = response.getEntity();
                    BufferedReader reader = new BufferedReader(new InputStreamReader(entity.getContent(), "UTF-8"));
                    String tmp;
                    while ((tmp = reader.readLine()) != null) {
                        content += tmp + "\r\n";
                    }
                }
                return content;
            }
        };

        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            return httpClient.execute(httpPost, responseHandler);
        } finally {
            try {
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    private static String doPost(final String url, HttpEntity httpEntity, int timeout) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(httpEntity);

        ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
            public String handleResponse(HttpResponse response) throws IOException {
            	String content = "";
            	HttpEntity entity = null;
            	try{
            		int status = response.getStatusLine().getStatusCode();
                    
                    if (status >= HttpStatus.SC_OK && status < HttpStatus.SC_MULTIPLE_CHOICES) {
                        entity = response.getEntity();
                        BufferedReader reader = new BufferedReader(new InputStreamReader(entity.getContent(), "UTF-8"));
                        String tmp;
                        while ((tmp = reader.readLine()) != null) {
                            content += tmp + "\r\n";
                        }
                    }
            	}finally{
            		entity.getContent().close();
            		EntityUtils.consume(entity);
            	}
                
                return content;
            }
        };
        RequestConfig defaultRequestConfig = RequestConfig.custom()
        	    .setSocketTimeout(timeout)
        	    .setConnectTimeout(timeout)
        	    .setConnectionRequestTimeout(timeout)
        	    .setStaleConnectionCheckEnabled(true)
        	    .build();
//        HttpClients.custom()
//        .setDefaultRequestConfig(defaultRequestConfig)
//        .build();
        httpPost.setConfig(defaultRequestConfig);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            return httpClient.execute(httpPost, responseHandler);
        } finally {
            try {
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static String postData(String urlStr, String data){
        return postData(urlStr, data, null);
    }

    public static String postData(String urlStr, String data, String contentType){
        BufferedReader reader = null;
        try {
            URL url = new URL(urlStr);
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            conn.setConnectTimeout(CONNECT_TIMEOUT);
            conn.setReadTimeout(CONNECT_TIMEOUT);
            if(contentType != null)
                conn.setRequestProperty("content-type", contentType);
            OutputStreamWriter writer = new OutputStreamWriter(conn.getOutputStream(), DEFAULT_ENCODING);
            if(data == null)
                data = "";
            writer.write(data);
            writer.flush();
            writer.close();

            reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), DEFAULT_ENCODING));
            StringBuilder sb = new StringBuilder();
            String line = null;
            while ((line = reader.readLine()) != null) {
                sb.append(line);
                sb.append("\r\n");
            }
            return sb.toString();
        } catch (IOException e) {
            //logger.error("Error connecting to " + urlStr + ": " + e.getMessage());
        } finally {
            try {
                if (reader != null)
                    reader.close();
            } catch (IOException e) {
            }
        }
        return null;
    }
}