/**
 * @FileName: JsonEntityTransform.java
 * @Package cn.haliaeetus.framework.util
 * 
 * @author warship
 * @created 2013年12月1日 下午3:03:24
 * 
 * Copyright 2013-2050 佰颂物联 版权所有
 */
package cn.haliaeetus.sendmessage.framework.util;

import cn.haliaeetus.sendmessage.framework.entity.baseEntity.BaseEntity;
import com.alibaba.fastjson.JSON;

import java.util.List;
import java.util.Map;

/**
 * <p>json与实体转化</p>
 * 
 * @author warship
 * @since 1.0
 * @version 1.0
 */
public final class JsonEntityTransform {
	
	/**
	 * 
	 * 将json转换成Entity
	 *
	 * @author warship
	 * @created 2013年12月1日 下午3:06:08
	 *
	 * @param json	要转换的json数据
	 * @param clazz	目标实体
	 * @return	Entity	转换后的实体对象
	 */
	public static <T extends BaseEntity> T jsonToEntity(String json, Class<T> clazz) {
		return JSON.parseObject(json, clazz);
	}
	
	/**
	 * 
	 * 将Json转换成实体集合
	 *
	 * @author warship
	 * @created 2013年12月1日 下午3:09:08
	 *
	 * @param json	要转换的json数据
	 * @param clazz	目标实体
	 * @return	List	转换后的实体对象集合
	 */
	public static <T extends BaseEntity> List<T> jsonToEntityList(String json, Class<T> clazz) {
		return JSON.parseArray(json, clazz);
	}
	
	/**
	 * 
	 * 将Entity对象转换成Json
	 *
	 * @author warship
	 * @created 2013年12月1日 下午3:25:26
	 *
	 * @param entity	要转换的实体
	 * @return	转换后的json
	 */
	public static <T extends BaseEntity> String entityToJson(T entity) {
		return JSON.toJSONString(entity);
	}
	
	/**
	 * 
	 * 将json转换成呢个map
	 *
	 * @author warship
	 * @created 2013年12月1日 下午3:29:10
	 *
	 * @param json	要转换的json
	 * @return	转换后的Map
	 */
	public static Map<?, ?> jsonToMap(String json) {
		return JSON.parseObject(json, Map.class);
	}
	
}
