package cn.haliaeetus.sendmessage.framework.util;

import cn.haliaeetus.sendmessage.constants.SMSConfig;
import cn.haliaeetus.sendmessage.framework.base.crawler.Crawler;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

/**
 * 短信服务工具类
 */
public class SMSUtil {
    private static final Logger LOG = LoggerFactory.getLogger(SMSUtil.class);

    private static final Crawler CRAWLER = Crawler.of("SMS");

    /**
     * 发送短信服务的URL
     */
    private static final String SMS_URL = SMSConfig.getBSSendSmsUrl();

    /**
     * 短信发送通道，2表示“百分通联”
     */
    public static final int CHANNEL_BF = 2;
    /**
     * 短信发送通道，3表示“国都”
     */
    public static final int CHANNEL_MD = 3;

    /**
     * 发送短信
     *
     * @param phones  手机号码，多个用英文逗号隔开
     * @param content 短信文本
     */
    public static String sendSMSText(String phones, String content, String sid, int channel) {
        return send(phones, content, sid, channel);
    }

    /**
     * 通过Apache HttpComponents发送请求到短信服务器，发送短信
     *
     * @param phones  手机号码，多个用英文逗号隔开
     * @param content 短信文本
     */
    private static String send(String phones, String content, String sid, int channel) {
        if (!StringUtils.hasText(phones) || !StringUtils.hasText(content)) {
            return null;
        }

        final List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("name", "haliaeetus_express"));
        params.add(new BasicNameValuePair("token", "token"));
        params.add(new BasicNameValuePair("phones", phones));
        params.add(new BasicNameValuePair("sid", sid));
        params.add(new BasicNameValuePair("content", content));
        params.add(new BasicNameValuePair("sendtype", channel + ""));
        UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, Consts.UTF_8);

        Pair<Integer, String> result = CRAWLER.post(SMS_URL, entity);
        LOG.info("SMS send: URL[{}],Entity[{}],ResultCode[{}],Result[{}]", SMS_URL, consume(entity), result.getLeft(), result.getRight());
        return result.getRight();
    }

    private static String consume(HttpEntity entity) {
        String result = "";
        try {
            InputStream inputStream = entity.getContent();
            result = StreamUtils.copyToString(inputStream, Charset.forName("UTF-8"));
            result = URLDecoder.decode(result, "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }
}