package cn.haliaeetus.sendmessage.message.consumer;

import cn.haliaeetus.sendmessage.bean.jiguang.JiguangBean;
import cn.haliaeetus.sendmessage.utils.jiguang.JpushClientUtil;
import com.google.gson.Gson;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Map;

/**
 * @ClassName PushExpressQueueListener
 * @Description TODO
 * @Author kael
 * @Date 2019/8/6 0006 下午 16:32
 * @Version 1.0
 **/
@Component
public class JPushQueueListener {
    Logger log = LoggerFactory.getLogger(JPushQueueListener.class);

    /**
     * 配置监听的哪一个队列，同时在没有queue和exchange的情况下会去创建并建立绑定关系
     * @param jsonObject
     * @param headers
     * @param channel
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.jPushQueue}",durable = "true"),
            exchange = @Exchange(name="${spring.rabbitmq.jPushExchange}",durable = "true",type = "direct"),
            key = "${spring.rabbitmq.jPushRoutingKey}"
    ))
    @RabbitHandler
    public void receive(@Payload String jsonObject, @Headers Map<String,Object> headers, Channel channel) {
        try {
            // 采用手动应答模式, 手动确认应答更为安全稳定
            channel.basicAck((Long) headers.get(AmqpHeaders.DELIVERY_TAG), true);
            if (jsonObject != null) {
                JiguangBean jiguangBean = new Gson().fromJson(jsonObject, JiguangBean.class);
                if (jiguangBean == null) {
                    channel.basicAck((Long) headers.get(AmqpHeaders.DELIVERY_TAG), false);
                    return;
                }
                JpushClientUtil.jiguangSend(jiguangBean);
            } else {
                channel.basicAck((Long) headers.get(AmqpHeaders.DELIVERY_TAG), false);
                return;
            }
        }catch (Exception e){
            log.error("错误", e);
        }
    }

}
