package cn.haliaeetus.sendmessage.message.consumer;

import cn.haliaeetus.sendmessage.bean.dto.PhoneLibDto;
import cn.haliaeetus.sendmessage.dao.PhoneLibDao;
import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PhoneLibListener
 * @Description
 * @Author SunJJ
 * @Date 2020/4/27 10:16
 * @Version V1.0
 **/
@Component
public class PhoneLibListener {
    Logger log = LoggerFactory.getLogger(JPushQueueListener.class);

    @Resource
    private PhoneLibDao phoneLibDao;

    /**
     * 配置监听队列，如果没有queue和exchange会去创建并建立绑定关系
     *
     * @param jsonObject
     * @param headers
     * @param channel
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.phoneLibQueue}", durable = "true"),
            exchange = @Exchange(name = "${spring.rabbitmq.phoneLibExchange}", durable = "true", type = "direct"),
            key = "${spring.rabbitmq.phoneLibRoutingKey}"
    ))
    @RabbitHandler
    public void receive(@Payload String jsonObject, @Headers Map<String, Object> headers, Channel channel) {
        try {
            PhoneLibDto phoneLibDto = JSONObject.parseObject(jsonObject, PhoneLibDto.class);
            // 插入之前先根据storeId和phone到数据库查询判断是否已经存在
            // 如果存在使用次数 +1，如果不存在插入一条新的数据
            int count = phoneLibDao.countByStoreIdAndPhone(phoneLibDto.getStoreId(), phoneLibDto.getPhones());
            if (count == 0) {
                List<PhoneLibDto> phoneLibDtos = new ArrayList<>();
                phoneLibDtos.add(phoneLibDto);
                phoneLibDao.insert(phoneLibDtos);
            }
            //收件人信息同步
            int counta = phoneLibDao.consigneeInfoCount(phoneLibDto.getStoreId(), phoneLibDto.getPhones());
            if (counta == 0) {
                phoneLibDto.setPhones(phoneLibDto.getPhones().substring(phoneLibDto.getPhones().length() - 4));
                phoneLibDto.setAreaId(phoneLibDao.getAreaId(phoneLibDto.getStoreId()));
                phoneLibDao.insertConsigneeInfo(phoneLibDto);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                // 忽略异常信息
                channel.basicAck((Long) headers.get(AmqpHeaders.DELIVERY_TAG), true);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }
}
