package cn.haliaeetus.sendmessage.message.consumer;

import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.service.ExpressHandleService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @ClassName PushExpressQueueListener
 * @Description TODO
 * @Author kael
 * @Date 2019/8/6 0006 下午 16:32
 * @Version 1.0
 **/
@Component
public class PushExpressQueueListener {
    Logger log = LoggerFactory.getLogger(PushExpressQueueListener.class);

    @Autowired
    private ExpressHandleService expressHandleService;

    @Autowired
    private SmsRecordService smsRecordService;

    @Value("${spring.rabbitmq.expressQueue}")
    private String expressQueue;

    /**
     * 配置监听的哪一个队列，同时在没有queue和exchange的情况下会去创建并建立绑定关系
     * @param jsonObject
     * @param headers
     * @param channel
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.expressQueue}",durable = "true"),
            exchange = @Exchange(name="${spring.rabbitmq.expressExchange}",durable = "true",type = "direct"),
            key = "${spring.rabbitmq.expressKey}"
    ))
    @RabbitHandler
    public void receive(@Payload String jsonObject, @Headers Map<String,Object> headers, Channel channel) {
        SignBackDto signBackDto=null;
        try {
            // 采用手动应答模式, 手动确认应答更为安全稳定
            channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG),true);
            // 添加mq日志
            smsRecordService.installApiLogMq(jsonObject,"expressQueue");
            log.info("进入快递消费者进行消费，传入参数："+jsonObject);
            if(jsonObject == null){
                channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
                return;
            }
            signBackDto = getSignBackDto(jsonObject, signBackDto);
            if(signBackDto == null){
                channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
                return;
            }

            /*
               重试时会使用这个signBackDto,为了防止在下面的操作中改变掉这个signBackDto内部的值，
               复制一遍
             */
            SignBackDto copy = new SignBackDto();
            BeanUtils.copyProperties(signBackDto, copy);
            expressHandleService.signBack(copy);
        }catch (Exception e){
            smsRecordService.msgErrorHandle(headers, channel, signBackDto,expressQueue);
            log.error("错误", e);
        }
    }

    /**
     * 双重转换防止意外报错
     * @param jsonObject
     * @param smsSendQueue
     * @return
     */
    private SignBackDto getSignBackDto(@Payload String jsonObject, SignBackDto signBackDto) {
        try {
            signBackDto=new Gson().fromJson(jsonObject, SignBackDto.class);
        }catch (Exception e) {
            signBackDto= JSON.parseObject(jsonObject, SignBackDto.class);
        }
        return signBackDto;
    }
}
