package cn.haliaeetus.sendmessage.message.consumer;

import cn.haliaeetus.sendmessage.bean.WhiteList;
import cn.haliaeetus.sendmessage.bean.basedata.ExpressType;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.WxSignDto;
import cn.haliaeetus.sendmessage.bean.wx.TemplateData;
import cn.haliaeetus.sendmessage.bean.wx.WxResBean;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplate;
import cn.haliaeetus.sendmessage.bean.wx.WxTemplateEntity;
import cn.haliaeetus.sendmessage.framework.util.JsonUtil;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.UserService;
import cn.haliaeetus.sendmessage.service.WhiteListService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.service.wx.WxTemplateService;
import cn.haliaeetus.sendmessage.utils._;
import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.jfinal.weixin.sdk.utils.JsonUtils;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @ClassName WXSignMsgRemindQueueListener
 * @Description TODO
 * @Author Administrator
 * @Date 2019/8/14 0014 下午 15:33
 * @Version 1.0
 **/
@Component
@Slf4j
public class WXSignMsgRemindQueueListener {

    @Autowired
    private BasicInfoService basicInfoService;

    @Autowired
    private WhiteListService whiteListService;

    @Autowired
    private WxTemplateService wxTemplateService;

    @Autowired
    private SmsRecordService smsRecordService;

    @Autowired
    private UserService userService;

    @Autowired
    private AmqpTemplate rabbitTemplate;

    @Value("${spring.rabbitmq.wechatQueue}")
    private String wechatQueue;

    /**
     * 配置监听的哪一个队列，同时在没有queue和exchange的情况下会去创建并建立绑定关系
     * @param jsonObject
     * @param headers
     * @param channel
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.wechatQueue}",durable = "true"),
            exchange = @Exchange(name="${spring.rabbitmq.wechatExahcange}",durable = "true",type = "direct"),
            key = "${spring.rabbitmq.wechatKey}"
    ))
    @RabbitHandler
    public void receive(@Payload String jsonObject, @Headers Map<String,Object> headers, Channel channel) {
        WxSignDto wxSignDto=null;
        try {
            // AMQP服务机制，消费后一分钟没有ACK默认失败，一分钟后重推MQ，最多16次
            // 采用手动应答模式, 手动确认应答更为安全稳定
            channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG),true);
            //添加mq日志
            smsRecordService.installApiLogMq(jsonObject,"wechatQueue");
            log.info("微信签收提醒进行消费，传入参数："+jsonObject);
            if(jsonObject == null){
                channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
                return;
            }
            try {
                wxSignDto = new Gson().fromJson(jsonObject, WxSignDto.class);
            }catch (Exception e){
                wxSignDto = JSON.parseObject(jsonObject,WxSignDto.class);
            }

            if(wxSignDto == null){
                channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
                return;
            }
            this.wxSignMsgRemind(wxSignDto.getStoreId(),wxSignDto.getDelivery());
        }catch (Exception e){
            smsRecordService.msgErrorHandle(headers, channel, wxSignDto,wechatQueue);
            log.error("错误", e);
        }
    }

    private void wxSignMsgRemind(Integer storeId,Delivery delivery) {
        List<WhiteList> list = whiteListService.getByStoreId(storeId);
        Map map = new HashMap();
        for (WhiteList white : list) {
            map.put(white.getUsermobile(), white);
        }
        //白名单用户不推送微信公总号信息+
        if (map.get(delivery.getUserPhone()) == null) {

            Map temMap = new HashMap();
            temMap.put("storeId", storeId);
            List<WxTemplateEntity> wxTemplateEntityList = new ArrayList<>();
            try {
                wxTemplateEntityList = wxTemplateService.selectTemplateForSignByStoreId(temMap);
            } catch (Exception e) {
                log.info(e.getMessage());
            }
            WxTemplate wxTemplate2 = getTemplate(storeId, delivery, wxTemplateEntityList, delivery.getUserPhone().trim());
            //发送微信消息
            WxResBean wxResBean = getWxResBean(delivery, wxTemplate2);
            String wxMessage2 = JSON.toJSONString(wxTemplate2);
            log.info("wxMessage2:::::::::::::::::" + wxMessage2);
        }
    }

    private WxTemplate getTemplate(int storeId, Delivery delivery, List<WxTemplateEntity> wxTemplateEntityList, String userPhone) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Store store = basicInfoService.getStoreById(storeId);
        WxTemplate wxTemplate = new WxTemplate();
        wxTemplate.setStore_id(String.valueOf(storeId));
        wxTemplate.setMobile_number(Long.valueOf(userPhone));
        wxTemplate.setTemplate_id(wxTemplateEntityList.get(0).getMsgtempid());
        Map<String, TemplateData> data = new HashMap<String, TemplateData>();

        /**
         *  first
         * 	expName
         * 	expCode
         * 	arriveTime
         * 	deliveryMan
         * 	deliveryManTel
         * 	remark
         */

        TemplateData templateData = new TemplateData();
        templateData.setValue("尊敬的客户，你的快递已被签收。");
        data.put("first", templateData);
        for (WxTemplateEntity wxTemplateEntity : wxTemplateEntityList) {
            try {
                log.info("wxTemplateEntity:::::::::::::::::::::" + JsonUtil.format(wxTemplateEntity));
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (wxTemplateEntity.getKey().equals("expName")) {
                TemplateData template = new TemplateData();
                Map<Integer, ExpressType> allExpress = basicInfoService.getAllExpress();
                for (Map.Entry<Integer, ExpressType> expressType: allExpress.entrySet()) {
                    if (expressType.getKey().equals(delivery.getExpId())){
                        template.setValue(expressType.getValue().getExpName());
                        data.put(wxTemplateEntity.getValue(), template);
                        break;
                    }
                }
            }
            if (wxTemplateEntity.getKey().equals("expCode")) {
                TemplateData template = new TemplateData();
                template.setValue(delivery.getExpcode());
                data.put(wxTemplateEntity.getValue(), template);
            }
            if (wxTemplateEntity.getKey().equals("arriveTime")) {
                TemplateData template = new TemplateData();
                template.setValue(sdf.format(new Date()));
                data.put(wxTemplateEntity.getValue(), template);
            }
            String deliveryManTel = delivery.getDeliveryUid();
            if (wxTemplateEntity.getKey().equals("deliveryMan")) {
                TemplateData template = new TemplateData();
                User user = userService.getUserByuserId(deliveryManTel);
                template.setValue(user.getUserName());
                data.put(wxTemplateEntity.getValue(), template);
            }
            if (wxTemplateEntity.getKey().equals("deliveryManTel")) {
                TemplateData template = new TemplateData();
                template.setValue(deliveryManTel);
                data.put(wxTemplateEntity.getValue(), template);
            }

        }
        TemplateData templateData6 = new TemplateData();
        templateData6.setValue("如有疑问请联系: " + delivery.getDeliveryUid());
        data.put("remark", templateData6);

        wxTemplate.setData(data);
        wxTemplate.setUrl(store.getAdvertisingUrl());
        return wxTemplate;
    }

    private WxResBean getWxResBean(Delivery delivery, WxTemplate wxTemplate2) {
        String url = basicInfoService.getConfigurationByCode("sendWxSignMsgUrl").getOValue1();
        WxResBean wxResBean = null;
        try {
            log.info("param................." + JsonUtils.toJson(wxTemplate2));
            Long beginTime = System.currentTimeMillis();
            String res = OkHttpUtil.postJson(url, wxTemplate2);
            Long endTime = System.currentTimeMillis();
            log.info("res..............................." + res);
            wxResBean = JSON.parseObject(res, WxResBean.class);
            System.out.println("微信快递签收消息返回结果：" + res);
        } catch (Exception e) {
            log.error(_.f("微信发送消息失败，订单信息：%s，异常信息：%s.", delivery, e.getMessage()));
        }
        return wxResBean;
    }
}
