package cn.haliaeetus.sendmessage.service;

import cn.haliaeetus.sendmessage.bean.basedata.*;
import cn.haliaeetus.sendmessage.bean.jspo.Configuration;
import com.fasterxml.jackson.databind.JsonNode;

import java.util.List;
import java.util.Map;

/**
 * 基础信息接口：包括门店信息，快递信息等
 * @author liutianyuan
 *
 */
public interface BasicInfoService {
	
	/**
	 * 从UCenter获取所有快递信息。供刷缓存使用。其他模块慎用.
	 * @return
	 */
	public Map<Integer, ExpressType> getAllExpress();

	/**
	 * 系统判断
	 * @return
	 */
	public boolean getSystem(String oCode);
	/**
	 * 从memcached获取所有快递信息。若无数据，则从数据库获取。
	 */
	public Map<Integer, ExpressType> getAllExpressFromMemcached();
	
	/**
	 * 获取快递信息。供内部接口使用。
	 * @return
	 */
	public ExpressType getExpressById(int expressId);

	ExpressType getExpressByShortName(String shortName);


    /**
	 * 获取所有门店信息。供刷新缓存使用。其他模块慎用。
	 * @return
	 */
	public Map<Integer, Store> getAllStore();

	/**
	 * 从memcached获取所有门店信息。
	 */
	public List<Store> getAllStoreFromMemcached();
	
	/**
	 * 获取门店信息。供内部接口使用。
	 * @param storeId
	 * @return
	 */
	public Store getStoreById(int storeId);

	/**
	 * 获取配置信息。供内部接口使用。
	 * @param oCode
	 * @return
	 */
	public Configuration getConfigurationByCode(String oCode);

	/**
	 * 获取配置信息。供内部接口使用。
	 * @param storeId
	 * @return
	 */
	public StoreReqLimit getStoreReqLimitByUserId(String storeId);
	
	/**
	 * 获取门店的快递列表
	 * @param storeId
	 * @return
	 * key: 0,全部快递  1,派件快递列表  2，揽件快递列表
	 */
	public Map<Integer, List<ExpressType>> getExpressByStoreId(int storeId);
	
	/**
	 * BS中更新门店信息，同步到usercenter，同时刷新缓存
	 * @param storeInfo
	 * @return
	 */
	public boolean update(Store storeInfo);

	List<Country> getAllCountries();
	
	List<Province> getAllProvince();
	
	List<City> getAllCitiesByProvinceId(int provinceId);
	
	List<Area> getAllAreasByCityId(long cityId);

	List<Country> getCountryRemoteService(int countryId);

    List<City> getCityRemoteService(int provinceId);

    List<Province> getProvinceRemoteService();

    List<Area> getAreaRemoteService(long cityId);

    int getUserIdByWxOpenId(String openId);

    Map<String,String> getUserInfoById(int userId);

    /**
     * 微信预绑定
     * @return
     */
    int userInit4wx(String openId, String nickName, String sex, String headimgurl);

    /**
     * 微信绑定
     * @return
     */
    int userBind4wx(String openId, String telphone, int collegeId, String smsCode, String icode);

    /**
     * 对手机发送验证码
     * @param telphone
     * @return
     */
    String getSmsCode(String telphone);

    /**
     * 校验手机验证码
     * @param telphone
     * @return
     */
    boolean checkSmsCode(String telphone, String smsCode);
    
    /**
     * 通过手机号查询openId
     * @param telphone
     * @return
     */
    JsonNode getUserByPhone(String telphone);


	JsonNode addCreditByMobilePhone(String telphone, long credit, int costType);
	//修改门店地址
	boolean updateShotrAddress(Store store);
	//获取所有快递
	List<ExpressType> getAllExpressMapMemcached();
	////修改合作快递同时刷缓存
	int saveDispathchIds(Store store);

	List<ExpressType> getAllExpressList();

	List<Configuration> getAllConfigurationList();

	Store getStoreFromDB(int storeId);

    String[] getStoreAddressByStoreIdAndExpId(int storeId, int expId);
}
