package cn.haliaeetus.sendmessage.service;

import cn.haliaeetus.sendmessage.bean.basedata.StoreShelf;
import cn.haliaeetus.sendmessage.bean.dto.StoreShelfDto;
import cn.haliaeetus.sendmessage.framework.toolkit.PagedList;

import java.util.List;

/**
 * 货架所有服务
 * 
 * @author zhang-shuai
 *
 */
public interface StoreShelfService {

	/**
	 * 首先从缓存获取货架信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * 
	 * @return 货架不存在时返回null
	 */
	public StoreShelf getById(int id);

	public List<StoreShelf> getAll();

	/**
	 * 首先从缓存获取门店货架信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * 
	 * @return 门店货架不存在时返回null
	 */
	public List<StoreShelf> getByStoreId(int storeId);

	public PagedList<StoreShelf> getByStoreId(int storeId, int p, int pageSize);

	/**
	 * storeShelf中有包裹数,flag标示按什么排序：1，标示按时间降序；0，按包裹数降序
	 * 
	 * @param storeId
	 * @return
	 */
	public List<StoreShelfDto> getDtoByStoreId(int storeId, int flag);

	/**
	 * storeShelf中有包裹数,flag标示按什么排序：1，标示按时间降序；0，按包裹数降序;分页
	 * 
	 * @param storeId
	 * @return
	 */
	public PagedList<StoreShelfDto> getDtoByStoreId(int storeId, int flag,
                                                    int p, int pageSize);

	public List<StoreShelf> getByStoreIdAndShelfName(int storeId,
                                                     String shelfName);

	/**
	 * 依据storeId、shelfName查找货架，且不等于该id
	 *
	 * @param id
	 * @param storeId
	 * @param shelfName
	 * @return
	 */
	public List<StoreShelf> getByStoreIdAndShelfName(int id, int storeId,
                                                     String shelfName);

	public int insert(int storeId, StoreShelf shelf);

	/**
	 * 更新货架，同时更新门店货架缓存和货架缓存 选择性更新，不需要更新字段置为null
	 * 
	 * @param shelf
	 * @return
	 */
	public int update(StoreShelf shelf);

	/**
	 * 删除货架，同时删除门店货架缓存和货架缓存 软删除
	 * 
	 * @param id
	 * @return
	 */
	public int delete(int id);

	/**
	 * 根据门店ID获取货架预编码,当已有编码最大值达到9999，替换数据库中isdel等于1的编码
	 * 
	 * @param storeId
	 * @return
	 */
	public String getBarCode(int storeId, boolean isdel);

	public void updateBarCodeBlank(int storeId);

	//设为默认货架
	public void setIsDefault(StoreShelfDto storeShelfDto);
	public void setIsDefaultAll(StoreShelfDto storeShelfDto);

}
