package cn.haliaeetus.sendmessage.service;

import cn.haliaeetus.sendmessage.bean.basedata.Menu;
import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.UserQueryCondition;
import cn.haliaeetus.sendmessage.utils.PageData;
import cn.haliaeetus.sendmessage.utils.Role;

import java.math.BigDecimal;
import java.util.List;

/**
 * 用户信息相关接口
 * @author liutianyuan
 *
 */
public interface UserService {
	
	/**
	 * 查询所有用户，供缓存刷数据使用。
	 * @return
	 */
	public List<User> getAllUser();
	
	
	/**
	 * 根据ID和密码查询用户。登录使用。不做缓存。
	 * @param userId 必填
	 * @param password 不需要时传null
	 * @return
	 */
	public User getUserByIdAndPwd(String userId, String password);
	
	
	/**
	 * 首先从缓存获取用户信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * @param userId
	 * @return  用户不存在时返回null
	 */
	public User getUserById(String userId);
	
	/**
	 * 更新用户信息，同时更新缓存
	 * @param qc
	 */
	public void updateUser(UserQueryCondition qc);
	
	/**
	 * 根据ID和密码查询用户。
	 * @param userId 必填
	 * @param storepwd
	 * @return
	 */
	public User getUserByIdAndStorePwd(String userId, String storepwd);
	
	public void saveuser(String userId);
	
	
	public void createDelivery();
	
	public void createTrigge(int storeId);
	
	
	public void updateMsgTemplate(String ids);
	
	/**
	 * 扣减余额
	 */
	public void deductBalance(User user, int smsCount, int operatorBusiness);
	
	/**
	 * 充值余额
	 */
	public void addBalance(int storeId, BigDecimal money);
	
	/**
	 * 更新短信价格配置
	 * @return
	 */
	public int updateSmsPriceConf(String userId, int isSmsFree, String smsPrice);

	public User getUserByuserId(String userId);

	public int updateOcrPriceConf(String userId, int isOcrFree, String ocrPrice);

	public int updateVigilance(String userId, String vigilance);

	User getPhenixUserInfo(PageData pd)throws Exception;

	List<Menu> gongPeilistAllMenu(String menu_name) throws Exception;


	Role getUserAndRoleByName(String USER_ID) throws Exception;

	public User selectByPrimaryKey(String userId);

	/**
	 * 获取所有用户信息，通过用户状态
	 * @return
	 */
	List<User> selectAllUserByStatus();

    List<Menu> getMenuList(String userId, User user);

    List<User> getUserByStoreId(String storeId) throws Exception;
}
