package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.basedata.*;
import cn.haliaeetus.sendmessage.bean.jspo.Configuration;
import cn.haliaeetus.sendmessage.controller.util.APIConfig;
import cn.haliaeetus.sendmessage.framework.base.crawler.CrawlResponse;
import cn.haliaeetus.sendmessage.framework.base.crawler.Crawler;
import cn.haliaeetus.sendmessage.framework.base.digt.EscapeUtil;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.framework.util.JsonUtil;
import cn.haliaeetus.sendmessage.jobs.DBUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.utils._;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.base.Strings;
import com.google.gson.JsonObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.Header;
import org.apache.http.message.BasicHeader;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * 待优化
 *
 * @author liutianyuan
 *
 */
@Service
public class BasicInfoServiceImpl implements BasicInfoService {
	private static final Logger LOG = Logger.getInstance(BasicInfoServiceImpl.class);

	private static final Crawler expressCrawler = Crawler.of("express.crawler", 1000, 2000);
	private static final Crawler storeCrawler = Crawler.of("store.crawler", 2000, 30000);
	private static final Crawler openIdCrawler = Crawler.of("openId.crawler", 2000, 30000);
	private static final Crawler proviceCrawler = Crawler.of("province.crawler", 1000, 2000);
	private static final Crawler cityCrawler = Crawler.of("city.crawler", 1000, 2000);
	private static final Crawler areaCrawler = Crawler.of("area.crawler", 1000, 2000);
	private static final Header[] headers = new Header[4];
	static {
		headers[0] = new BasicHeader("sign", "f5647e28807bb054c9c2bcc249e84cc6");
		headers[1] = new BasicHeader("appTime", "2014-09-01 11:17:23");
		headers[2] = new BasicHeader("userId", "17677");
		headers[3] = new BasicHeader("Content-Type", "application/json");
	}

	private static final String userCenterUrl = APIConfig.getUserCenterUrl();

	public boolean getSystem(String oCode) {
		Configuration systemC =getConfigurationByCode("system");
		if (oCode.equals(systemC.getOValue1())) {
			return true;
		}
		return false;
	}
	/**
	 * 获取门店的快递列表
	 *
	 * @param storeId
	 * @return key: 0,全部快递 1,派件快递列表 2，揽件快递列表
	 */
	@Override
	public Map<Integer, List<ExpressType>> getExpressByStoreId(int storeId) {
		/*
		 * if(true){ Map<Integer, List<ExpressType>> mapExpressType = new
		 * HashMap<Integer, List<ExpressType>>(); mapExpressType.put(0,
		 * MockData.getExpressTypeList()); return mapExpressType; }
		 */
		Store store = getStoreById(storeId);
		if (null == store)
			return null;

		List<Integer> receiveExpList = formatIds(store.getReceiveIds(), ",");
		List<Integer> deliveryExpList = formatIds(store.getDispatchIds(), ",");

		Set<Integer> allExpSet = new HashSet<Integer>();
		allExpSet.addAll(receiveExpList);
		allExpSet.addAll(deliveryExpList);
		List<Integer> allExpList = new ArrayList<Integer>();
		allExpList.addAll(allExpSet);
		Map<Integer, ExpressType> allExpMap = getAllExpressFromMemcached();
		List<ExpressType> receiveExpressList = getExpressList(receiveExpList, allExpMap);
		List<ExpressType> deliveryExpressList = getExpressList(deliveryExpList, allExpMap);
		List<ExpressType> allExpressList = getExpressList(allExpList, allExpMap);

		Map<Integer, List<ExpressType>> expressMap = new HashMap<Integer, List<ExpressType>>();
		expressMap.put(0, allExpressList);
		expressMap.put(1, deliveryExpressList);
		expressMap.put(2, receiveExpressList);
		return expressMap;
	}

	/**
	 * 首先从memcached中获取，如果存在直接返回 否则，从UCenter获取，同时刷新memcached
	 */
	@Override
	public ExpressType getExpressById(int expressId) {
		// if(true){
		// return MockData.getExpressType();
		//
		// }
		CacheTool cacheTool = CacheTool.initExpress();
		ExpressType express = (ExpressType) cacheTool.getBean(expressId + "");
		if (express != null) {
			Map<Integer, ExpressType> expMap = getExpressRemoteServeic(expressId);
			//LOG.record("从memcached获取快递信息:" + expressId);
			return express;
		} else {
			Map<Integer, ExpressType> expMap = getExpressRemoteServeic(expressId);
			LOG.record("从远程接口获取快递信息:" + expMap);
			if (expMap != null && expMap.containsKey(expressId)) {
				express = expMap.get(expressId);
				cacheTool.saveBean(expressId + "", express);
				LOG.record("刷新快递memcached:" + expressId);
				return express;
			}
			return null;
		}

	}

	@Override
	public ExpressType getExpressByShortName(String shortName) {
		/*
		 * if (true) { return MockData.getExpressType(); }
		 */

		if (_.isEmpty(shortName)) {
			return null;
		}

		Map<Integer, ExpressType> expressTypeMap = getAllExpressFromMemcached();
		if (_.isNotEmpty(expressTypeMap)) {
			Set<Integer> expIds = expressTypeMap.keySet();
			for (Integer expId : expIds) {
				ExpressType expType = expressTypeMap.get(expId);
				if (shortName.equals(expType.getExpShortName())) {
					return expType;
				}
			}
		}
		return null;
	}

	/**
	 * 从数据库获取全部快递信息
	 */
	public Map<Integer, ExpressType> getAllExpress() {
		/*
		 * if(true){ Map<Integer, ExpressType> mapExpressType = new
		 * HashMap<Integer, ExpressType>(); mapExpressType.put(0,
		 * MockData.getExpressType()); return mapExpressType; }
		 */

		Map<Integer, ExpressType> expressTypeMap = getExpressRemoteServeic(0);

		return expressTypeMap;
	}

	/**
	 * 从memcached获取全部快递信息
	 */
	public Map<Integer, ExpressType> getAllExpressFromMemcached() {
		/*
		 * if(true){ Map<Integer, ExpressType> mapExpressType = new
		 * HashMap<Integer, ExpressType>(); for(ExpressType express :
		 * MockData.getExpressTypeList()) mapExpressType.put(express.getId(),
		 * express); return mapExpressType; }
		 */

		CacheTool cacheTool = CacheTool.initExpress();
		List<ExpressType> list = (List<ExpressType>) cacheTool.getBean(0 + "", ExpressType.class);
		Map<Integer, ExpressType> expressTypeMap = new HashMap<Integer, ExpressType>();
		if (null != list) {
			for (ExpressType express : list) {
				if(express !=null){
					expressTypeMap.put(express.getExpId(), express);
				}
			}

			if (expressTypeMap != null && expressTypeMap.size() > 0) {
				//LOG.record("从memcached获取所有快递，数量:" + expressTypeMap.size());
				return expressTypeMap;
			}
		}

		//LOG.record("从memcached获取所有快递失败，从数据库获取。");
		return getExpressRemoteServeic(0);
	}

	public List<Country> getAllCountries() {
		/*
		 * if(true){ return MockData.getCountryList(); }
		 */
		List<Country> countries = getCountryRemoteService(0);
		return countries;
	}

	/**
	 * 从UCenter获取快递信息。供系统初始化时使用。0表示所有快递.
	 */
	private Map<Integer, ExpressType> getExpressRemoteServeic(int expressId) {

		List<ExpressType> expressTypeList=this.getRemoteExpressList(expressId);
		if (expressTypeList!=null&&expressTypeList.size()>0) {
			return fromExpressJson(expressId, expressTypeList);
		}
		return null;
	}

	/**
	 * 从UCenter获取所有门店信息。供系统初始化时使用。0表示所有门店
	 */
	private Map<Integer, Store> getStoreRemoteService(int storeId) {

		List<Store> storeList=this.getRemoteStoreList(storeId);
/*		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/storeExtend/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 2*/
		if (storeList!=null&&storeList.size()>0) {
			return fromStoreJson(storeId, storeList);
		}
		return null;
	}

	private Map<String, Configuration> getConfigurationRemoteService(String oCode) {

		List<Configuration> configurationList=this.getRemoteConfigurationList(oCode);
/*		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/storeExtend/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 2*/
		if (configurationList!=null&&configurationList.size()>0) {
			return fromConfigurationJson(oCode, configurationList);
		}
		return null;
	}

	public List<Country> getCountryRemoteService(int countryId) {

		List<Country> countryList=this.getRemoteCountryList(countryId);

/*		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/country/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 3*/
		if (countryList!=null&&countryList.size()>0) {
			return countryList;
		}
		return null;
	}

	public List<City> getCityRemoteService(int provinceId) {
		int id=0;
		List<City> cityList=this.getRemoteCityList(provinceId,id);

		/*CrawlResponse resp = cityCrawler.postResp(userCenterUrl + "/city/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 4*/
		if (cityList!=null&&cityList.size()>0) {
			return cityList;
		}
		return null;
	}

	public List<Province> getProvinceRemoteService() {

		int id=0;
		List<Province> provinceList=this.getRemoteProvinceList(id);

		/*CrawlResponse resp = proviceCrawler.postResp(userCenterUrl + "/province/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 5*/
		if (provinceList!=null&&provinceList.size()>0) {
			return provinceList;
		}
		return null;
	}

	public List<Area> getAreaRemoteService(long cityId) {


		int id=0;
		List<Area> cityList=this.getRemoteAreaList(cityId,id);

/*		CrawlResponse resp = areaCrawler.postResp(userCenterUrl + "/area/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 6*/
		if (cityList!=null&&cityList.size()>0) {
			return cityList;
		}
		return null;
	}

	@Override
	public int getUserIdByWxOpenId(String openId) {
		JsonObject entity = new JsonObject();
		entity.addProperty("openId", openId);

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/user/getUserIdByWxOpenId", entity.toString(),
				headers, EscapeUtil.CHARSET_UTF8);//DONE 未用 6
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter获取用户id:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				return jsonNode.findValue("userId").asInt();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return 0;
	}

	@Override
	public Map<String, String> getUserInfoById(int userId) {
		JsonObject entity = new JsonObject();
		entity.addProperty("userId", userId);

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/user/getById", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 未用 7
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter获取用户信息:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				HashMap<String, String> map = new HashMap<>();
				JsonNode dataNode = jsonNode.findValue("data");
				map.put("mobilePhone", dataNode.findValue("mobilePhone").asText());
				map.put("realName", dataNode.findValue("realName").asText());
				return map;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	@Override
	public int userInit4wx(String openId, String nickName, String sex, String headimgurl) {
		JsonObject entity = new JsonObject();
		entity.addProperty("openid", openId);
		entity.addProperty("nickname", nickName);
		entity.addProperty("sex", sex);
		entity.addProperty("headimgurl", headimgurl);

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/user/init4wx", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 未用 8
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter初始化微信绑定:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				int code = jsonNode.findValue("code").asInt();
				if (code != 0)
					return -1;

				JsonNode dataNode = jsonNode.findValue("data");
				return dataNode.findValue("userId").asInt();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return 0;
	}

	@Override
	public int userBind4wx(String openId, String telphone, int collegeId, String smsCode, String icode) {
		JsonObject entity = new JsonObject();
		entity.addProperty("telphone", telphone);
		entity.addProperty("smsCode", smsCode);
		entity.addProperty("openid", openId);
		entity.addProperty("collegeId", collegeId);
		entity.addProperty("icode", icode);

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/user/bind4wx", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 未用 9
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter绑定微信:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				int code = jsonNode.findValue("code").asInt();
				if (code != 0)
					return -1;

				JsonNode dataNode = jsonNode.findValue("data");
				return dataNode.findValue("userId").asInt();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return 0;
	}

	@Override
	public String getSmsCode(String telphone) {
		JsonObject entity = new JsonObject();
		entity.addProperty("telPhone", telphone);
		entity.addProperty("action", "H5_MALL");

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/smsrecords/getSmsCode", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 未用 10
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter获取短信验证码:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				JsonNode dataNode = jsonNode.findValue("data");
				return dataNode.findValue("smsCode").asText();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	@Override
	public boolean checkSmsCode(String telphone, String smsCode) {
		JsonObject entity = new JsonObject();
		entity.addProperty("telPhone", telphone);
		entity.addProperty("action", "H5_MALL");
		entity.addProperty("smsCode", smsCode);

		CrawlResponse resp = storeCrawler.postResp(userCenterUrl + "/smsrecords/check", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 未用 11
		if (resp.httpCode / 100 == 2) {
			LOG.record("从UCenter获取校验短信验证码:" + resp.content);
			try {
				JsonNode jsonNode = JsonUtil.parse(resp.content);
				return jsonNode.findValue("code").asInt() == 0;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return false;
	}

	/**
	 * 从数据库获取全部门店信息
	 */
	public Map<Integer, Store> getAllStore() {
		return getStoreRemoteService(0);
	}

	public List<Store> getAllStoreFromMemcached() {
		/*
		 * if(true){ return MockData.getStoreList(); }
		 */

		CacheTool cacheTool = CacheTool.initStore();
		List<Store> stores = cacheTool.getBean(0 + "", Store.class);
		if (_.isNotEmpty(stores)) {
			//LOG.record("从memcached获取所有门店，数量:" + stores.size());
			return stores;
		}
		return null;
	}
	@Override
	public StoreReqLimit getStoreReqLimitByUserId(String storeId) {
		/*
		 * if(true){ return MockData.getStore(); }
		 */

		CacheTool cacheTool = CacheTool.initLimit();
		StoreReqLimit storeReqLimit = (StoreReqLimit) cacheTool.getBean(storeId);
		if (storeReqLimit != null) {
			storeReqLimit.setSumCount(storeReqLimit.getSumCount()+1);
			System.out.println(System.currentTimeMillis()-storeReqLimit.getTime());
			if((System.currentTimeMillis()-storeReqLimit.getTime())<Integer.parseInt(getConfigurationByCode("limtStoreReqTime").getOValue1())){
                storeReqLimit.setCount(storeReqLimit.getCount()+1);
                if(storeReqLimit.getCount()>=Integer.parseInt(getConfigurationByCode("limtStoreReqCount").getOValue1())){
                    storeReqLimit.setIsLimitFlag(true);
                }
			}else{
                storeReqLimit.setIsLimitFlag(false);
                storeReqLimit.setTime(System.currentTimeMillis());
                storeReqLimit.setCount(1);
            }
            cacheTool.saveBean(storeId, storeReqLimit);
			return storeReqLimit;
		} else {
			StoreReqLimit newSrl = new StoreReqLimit();
			newSrl.setStoreId(storeId);
			newSrl.setCount(1);
            newSrl.setSumCount(1);
            newSrl.setIsLimitFlag(false);
			newSrl.setTime(System.currentTimeMillis());
			cacheTool.saveBean(storeId, newSrl);
			return newSrl;
		}
	}


	/**
	 * 首先从memcached中获取，如果存在直接返回 否则，从UCenter获取，同时刷新缓存
	 */
	@Override
	public Configuration getConfigurationByCode(String oCode) {
		/*
		 * if(true){ return MockData.getStore(); }
		 */

		CacheTool cacheTool = CacheTool.initConfiguration();
		Configuration configuration = (Configuration) cacheTool.getBean(oCode);
		if (configuration != null) {
			return configuration;
		} else {

			Map<String, Configuration> configurationMap = getConfigurationRemoteService(oCode);
			LOG.record("从远程系统配置信息:" + configurationMap);
			if (configurationMap != null && configurationMap.containsKey(oCode)) {
				configuration = configurationMap.get(oCode);
				cacheTool.saveBean(oCode, configuration);
				LOG.record("刷新从远程系统配置信息:" + oCode);
				return configuration;
			}
			return null;
		}

	}

	/**
	 * 首先从memcached中获取，如果存在直接返回 否则，从UCenter获取，同时刷新缓存
	 */
	@Override
	public Store getStoreById(int storeId) {
		/*
		 * if(true){ return MockData.getStore(); }
		 */

		CacheTool cacheTool = CacheTool.initStore();
		Store store = (Store) cacheTool.getBean(storeId + "");
		if (store != null) {
			//LOG.record("从memcached获取门店信息:" + storeId);
			return store;
		} else {

			Map<Integer, Store> storeMap = getStoreRemoteService(storeId);
			LOG.record("从远程接口获取门店信息:" + storeMap);
			if (storeMap != null && storeMap.containsKey(storeId)) {
				store = storeMap.get(storeId);
				cacheTool.saveBean(storeId + "", store);
				LOG.record("刷新门店memcached:" + storeId);
				return store;
			}
			return null;
		}

	}

	/**
	 * 获取全部省份
	 */
	@Override
	public List<Province> getAllProvince() {
		/*
		 * if(true){ return MockData.getProvinceList(); }
		 */

		CacheTool cacheTool = CacheTool.initProvince();
		List<Province> provinceList = (List<Province>) cacheTool.getBean("0", Province.class);
		if (provinceList != null) {
			//LOG.record("从memcached获取所有省份，数量:" + provinceList.size());
			return provinceList;
		}
		provinceList = getProvinceRemoteService();
		LOG.record("从远程接口获取所有省份，数量:" + (provinceList == null ? 0 : provinceList.size()));

		if (provinceList != null && provinceList.size() > 0) {
			cacheTool.saveBean("0", provinceList);
		}
		return provinceList;
	}

	/**
	 * 根据城市获取全部地区信息
	 */
	@Override
	public List<City> getAllCitiesByProvinceId(int provinceId) {
		/*
		 * if(true){ return MockData.getCityList(); }
		 */

		CacheTool cacheTool = CacheTool.initCity();
		List<City> cityList = (List<City>) cacheTool.getBean(provinceId + "", City.class);
		if (cityList != null) {
			//LOG.record("从memcached获取城市列表，省份Id:" + provinceId + " 数量:" + cityList.size());
			return cityList;
		}
		cityList = getCityRemoteService(provinceId);
		LOG.record("从远程接口获取城市列表，省份Id:" + provinceId + " 数量:" + cityList.size());

		if (cityList.size() > 0) {
			cacheTool.saveBean(provinceId + "", cityList);
		}
		return cityList;
	}

	/**
	 * 根据省份获取全部城市信息
	 */
	@Override
	public List<Area> getAllAreasByCityId(long cityId) {
		/*
		 * if(true){ return MockData.getAreaList(); }
		 */

		CacheTool cacheTool = CacheTool.initArea();
		List<Area> areaList = (List<Area>) cacheTool.getBean(cityId + "", Area.class);
		if (areaList != null) {
			//LOG.record("从memcached获取地区列表，城市 Id:" + cityId + " 数量:" + areaList.size());
			return areaList;
		}
		areaList = getAreaRemoteService(cityId);
		LOG.record("从远程接口获取地区列表，城市Id:" + cityId + " 数量:" + areaList.size());

		if (areaList.size() > 0) {
			cacheTool.saveBean(cityId + "", areaList);
		}

		return areaList;
	}

	/**
	 * 后期写成通用方法
	 *
	 * @return
	 */
	private Map<Integer, ExpressType> fromExpressJson(int expressId, List<ExpressType> expressTypeList) {
		Map<Integer, ExpressType> expressMap = new HashMap<Integer, ExpressType>();
		CacheTool cacheTool = CacheTool.initExpress();
		try {
			for(ExpressType express :expressTypeList){
				expressMap.put(express.getExpId(), express);
			}
			if (expressTypeList.size() > 0 && expressId == 0) {
				cacheTool.saveBean(0 + "", expressTypeList);
			}
		} catch (Exception e) {
			LOG.record("error:" + e.getMessage());
		}
		return expressMap;
	}

	private Map<Integer, Store> fromStoreJson(int storeId, List<Store> storeList) {
		Map<Integer, Store> storeMap = new HashMap<Integer, Store>();
		CacheTool cacheTool = CacheTool.initStore();
		List<Store> list = new ArrayList<>();
		try {

			for(Store store:storeList){
				storeMap.put(store.getId(), store);
				cacheTool.saveBean(store.getId() + "", store);
				list.add(store);
			}

			if (list.size() > 0 && storeId == 0) {
				cacheTool.saveBean(0 + "", list);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return storeMap;
	}

	private Map<String, Configuration> fromConfigurationJson(String oCode, List<Configuration> configurationList) {
		Map<String, Configuration> configurationMap = new HashMap<String, Configuration>();
		CacheTool cacheTool = CacheTool.initConfiguration();
		List<Configuration> list = new ArrayList<>();
		try {

			for(Configuration configuration:configurationList){
				configurationMap.put(configuration.getOCode(), configuration);
				cacheTool.saveBean(configuration.getOCode(), configuration);
				list.add(configuration);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return configurationMap;
	}



	private List<Province> fromProvinceJson(final String json) {
		List<Province> provinces = new ArrayList<>();
		try {
			JsonNode jsonNode = JsonUtil.parse(json);
			JsonNode dataNode = jsonNode.findValue("data").findValue("foo");

			Iterator<JsonNode> it = dataNode.iterator();
			while (it.hasNext()) {
				JsonNode thisNode = it.next();

				Province province = new Province();
				province.setId(thisNode.findValue("id").asInt());
				province.setName(_.trimToEmpty(thisNode.findValue("name").asText()));
				provinces.add(province);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return provinces;
	}



	private List<Area> fromAreaJson(final String json) {
		List<Area> areas = new ArrayList<>();
		try {
			JsonNode jsonNode = JsonUtil.parse(json);
			JsonNode dataNode = jsonNode.findValue("data").findValue("foo");

			Iterator<JsonNode> it = dataNode.iterator();
			while (it.hasNext()) {
				JsonNode thisNode = it.next();

				Area area = new Area();
				area.setId(thisNode.findValue("id").asLong());
				area.setName(_.trimToEmpty(thisNode.findValue("name").asText()));
				area.setCityId(thisNode.findValue("cityId").asLong());
				areas.add(area);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return areas;
	}

	/**
	 * 将这种字符串数据 1,2,3 转化为int数组
	 *
	 * @param ids
	 * @return
	 */
	private List<Integer> formatIds(String ids, String separator) {
		List<Integer> idList = new ArrayList<Integer>();

		String[] idStringArray = StringUtils.split(ids, separator);
		for (String idString : idStringArray) {
			idList.add(Integer.parseInt(idString));
		}

		return idList;
	}

	private List<ExpressType> getExpressList(List<Integer> expList, Map<Integer, ExpressType> expressMap) {

		List<ExpressType> expInfoList = new ArrayList<ExpressType>();
		for (int expId : expList) {
			if (expressMap != null && expressMap.containsKey(expId)) {
				expInfoList.add(expressMap.get(expId));
			} else {
				ExpressType express = getExpressById(expId);
				expInfoList.add(express);
			}
		}
		return expInfoList;
	}

	/**
	 * BS中更新门店信息，同步到usercenter，同时刷新缓存
	 *
	 * @param storeInfo
	 * @return
	 */
	public boolean update(Store storeInfo) {
		try {
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("id", storeInfo.getId());
			jsonObject.put("shortAddress", storeInfo.getShortAddress());
			jsonObject.put("takeExpressPhone", storeInfo.getTakeExpressPhone());
			jsonObject.put("startTime", storeInfo.getStartTime());
			jsonObject.put("endTime", storeInfo.getEndTime());

			int code=this.updateSelective(storeInfo);
			/*String response = HttpUtil.post(userCenterUrl + "/store/updateSelective", jsonObject.toJSONString());//DONE 12
			JSONObject jsonResponse = JSONObject.parseObject(response);*/

			/*if (jsonResponse == null) {
				return false;
			}*/

			/*if (jsonResponse.getIntValue("code") == 0) {
				CacheTool cacheTool = CacheTool.initStore();
				cacheTool.saveBean(storeInfo.getId() + "", storeInfo);
				return true;
			}*/
			if (code == 1) {
				CacheTool cacheTool = CacheTool.initStore();
				cacheTool.saveBean(storeInfo.getId() + "", storeInfo);
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}

	@Override
	public JsonNode getUserByPhone(String telphone) {
		// TODO Auto-generated method stub /user/getExtraUserByPhoneForInnerJoin
		JsonNode jsonNode = null;
		JsonObject entity = new JsonObject();
		entity.addProperty("mobilePhone", telphone);

		CrawlResponse resp = openIdCrawler.postResp(userCenterUrl + "/user/getExtraUserByPhoneForInnerJoin",
				entity.toString(), headers, EscapeUtil.CHARSET_UTF8);//DONE 未用 13
		if (resp.httpCode / 100 == 2) {
			LOG.record("getUserByPhone:" + telphone + "从UCenter获取用户信息:" + resp.content);
			try {
				jsonNode = JsonUtil.parse(resp.content);
				if (jsonNode.findValue("code").asInt() == -1) {
					jsonNode = null;
				}
			} catch (IOException e) {
				LOG.error("getUserByPhone.phone:" + telphone, e);
				// e.printStackTrace();
			}
		}

		return jsonNode;
	}

	@Override
	public JsonNode addCreditByMobilePhone(String telphone, long credit, int costType) {
		// TODO Auto-generated method stub /user/getExtraUserByPhoneForInnerJoin
		JsonNode jsonNode = null;
		try {
			if (_.isEmpty(telphone)) {
				return jsonNode;
			}
			String paramStr = "mobilePhone=" + telphone + "&credits=" + credit + "&costType=" + costType + "&from=4";
			Pair<Integer, String> response = storeCrawler
					.get(userCenterUrl + "/credit/addCreditByMobilePhone" + "?" + paramStr);//DONE 未用 14
			LOG.record(response.getRight());
			jsonNode = JsonUtil.parse(response.getRight());
			LOG.record(_.f("add credit by mobile [%s] result:[%s]", telphone, jsonNode));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			LOG.error(_.f("error add credit by mobile [%s] result:[%s]", telphone, jsonNode), e);
		}
		return jsonNode;
	}
	//修改长地址用
	@Override
	public boolean updateShotrAddress(Store storeInfo) {
		try {
			int code=this.updateStoreaddress1(storeInfo);
			if (code == 1) {
				CacheTool cacheTool = CacheTool.initStore();
				cacheTool.saveBean(storeInfo.getId() + "", storeInfo);
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public List<ExpressType> getAllExpressMapMemcached() {
		CacheTool cacheTool = CacheTool.initExpress();
		List<ExpressType> list = (List<ExpressType>) cacheTool.getBean(0 + "", ExpressType.class);
		/*Map<Integer, ExpressType> expressTypeMap = new HashMap<Integer, ExpressType>();
		if (null != list) {
			for (ExpressType express : list) {
				if(express !=null){
					expressTypeMap.put(express.getExpId(), express);
				}
			}

			if (expressTypeMap != null && expressTypeMap.size() > 0) {
				//LOG.record("从memcached获取所有快递，数量:" + expressTypeMap.size());
				return expressTypeMap;
			}
		}*/
		if(null != list && list.size() > 0){
			return list;
		}
		List<ExpressType> expressTypeList=this.getAllRemoteExpressList();
		/*CrawlResponse resp = expressCrawler.postResp(userCenterUrl + "/express/get", entity.toString(), headers,
				EscapeUtil.CHARSET_UTF8);//DONE 1*/
		if (expressTypeList!=null&&expressTypeList.size()>0) {
			 fromExpressJson(0, expressTypeList);
			 return expressTypeList;
		}
		return new ArrayList<ExpressType>();
	}
	//修改合作快递同时刷缓存
	@Override
	public int saveDispathchIds(Store storeInfo) {
			int result = updateDispathchIds(storeInfo.getDispatchIds(),storeInfo.getId());
			if(result > 0){
				CacheTool cacheTool = CacheTool.initStore();
				cacheTool.saveBean(storeInfo.getId() + "", storeInfo);
				return 1;
			}
			return 0;
	}

	@Override
	public List<ExpressType> getAllExpressList() {
		List<ExpressType> allExps = new ArrayList<ExpressType>();
		allExps = getAllExpressMapMemcached();
		return allExps;
	}



	private int updateDispathchIds(String dispathchIds,int storeId) {
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "update ex_store set dispatch_ids ='"+dispathchIds+"' where  id = '"+String.valueOf(storeId)+"'";
				pstm = connection.prepareStatement(sql);

				int i = pstm.executeUpdate();
				connection.commit();
				if(i>0){
					return 1;
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return 0;
	}

	public List<ExpressType> getRemoteExpressList(int expressId){
		List<ExpressType> expressTypeList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.id ,a.name,a.shortname ,a.type,a.isdel ,a.expImage , a.sort FROM ex_express_type a  where  a.id = '"+String.valueOf(expressId)+"' ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					ExpressType expressType=new ExpressType();
					expressType.setExpId(rs.getInt(1));
					expressType.setExpName(rs.getString(2));
					expressType.setExpShortName(rs.getString(3));
					expressType.setType(rs.getInt(4));
					expressType.setIsDel(rs.getInt(5));
					expressType.setExpImage(rs.getString(6));
					expressType.setSort(rs.getInt(7));
					expressTypeList.add(expressType);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return expressTypeList;
	}

	public List<Store> getRemoteStoreList(int storeId){
		List<Store> storeList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.id,a.name ,a.short_name,a.owner,a.phone,a.take_express_phone,a.province_id,a.province_name,a.city_id,a.city_name," +
						"a.area_id,a.area_name,a.college_id,a.college_name,a.address,a.short_address," +
						"a.start_time,a.end_time,a.receive_ids,a.dispatch_ids ,a.isp2p,a.operate_type,a.status," +
						"a.isprinter,a.longitude,a.latitude,a.appid,a.appsecret,a.msgtempid,a.signtempid,a.domain,a.opendoor,sendexpress,a.advertisingUrl,a.advertisingMsg,a.yjmsgtempid,a.auditStatus, a.isOpenSignBack FROM ex_store a where  a.id = '"+String.valueOf(storeId)+"' ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Store store=new Store();
					store.setId(rs.getInt(1));
					store.setName(rs.getString(2));
					store.setShortName(rs.getString(3));
					store.setOwner(rs.getString(4));
					store.setPhone(rs.getString(5));
					store.setTakeExpressPhone(rs.getString(6));
					store.setProvinceId(rs.getInt(7));
					store.setProvinceName(rs.getString(8));
					store.setCityId(rs.getLong(9));
					store.setCityName(rs.getString(10));
					store.setAreaId(rs.getLong(11));
					store.setAreaName(rs.getString(12));
					store.setCollegeId(rs.getInt(13));
					store.setCollegeName(rs.getString(14));
					store.setAddress(rs.getString(15));
					store.setShortAddress(rs.getString(16));
					store.setStartTime(rs.getString(17));
					store.setEndTime(rs.getString(18));
					store.setReceiveIds(rs.getString(19));
					store.setDispatchIds(rs.getString(20));
					store.setIsP2p(rs.getInt(21));
					store.setOperate_type(rs.getInt(22));
					store.setStatus(rs.getInt(23));
					store.setIsPrinter(rs.getString(24));
					store.setLongitude(rs.getString(25));
					store.setLatitude(rs.getString(26));
					store.setAppid(rs.getString(27));
					store.setAppsecret(rs.getString(28));
					store.setMsgtempid(rs.getString(29));
					store.setSigntempid(rs.getString(30));
					store.setDomain(rs.getString(31));
					store.setOpendoor(rs.getString(32));
					store.setSendexpress(rs.getString(33));
					store.setAdvertisingUrl(rs.getString(34));
					store.setAdvertisingMsg(rs.getString(35));
					store.setYjmsgtempid(rs.getString(36));
					store.setAuditStatus(rs.getInt(37));
					store.setIsOpenSignBack(rs.getBoolean(38));
					storeList.add(store);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return storeList;
	}

	public List<Configuration> getRemoteConfigurationList(String oCode){
		List<Configuration> configurationList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.oCode, a.oValue1, a.oValue2 FROM ex_configuration a where  a.oCode = '"+oCode+"' ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Configuration configuration=new Configuration();
					configuration.setOCode(rs.getString(1));
					configuration.setOValue1(rs.getString(2));
					configuration.setOValue2(rs.getString(3));
					configurationList.add(configuration);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return configurationList;
	}

	@Override
	public List<Configuration> getAllConfigurationList(){
		List<Configuration> configurationList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.oCode, a.oValue1, a.oValue2 FROM ex_configuration a ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Configuration configuration=new Configuration();
					configuration.setOCode(rs.getString(1));
					configuration.setOValue1(rs.getString(2));
					configuration.setOValue2(rs.getString(3));
					configurationList.add(configuration);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return configurationList;
	}


	public List<Country> getRemoteCountryList(int countryId){
		List<Country> countryList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.country_id,a.country_name,a.english_name,a.code,a.level FROM address_country a  where  a.country_id = '"+String.valueOf(countryId)+"' ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Country country=new Country();
					country.setId(rs.getInt(1));
					country.setName(rs.getString(2));
					country.setEnName(rs.getString(3));
					country.setCode(rs.getString(4));
					country.setLevel(rs.getInt(5));
					countryList.add(country);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return countryList;
	}

	public List<City> getRemoteCityList(int provinceId,int id){
		List<City> cityList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.city_id,a.city_id,a.province_id,a.city_name FROM address_city a  where  a.province_id = '"+String.valueOf(provinceId)+"' and city_id='"+String.valueOf(id)+"'";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					City city=new City();
					city.setId(rs.getInt(1));
					city.setCode(rs.getString(2));
					city.setProvinceId(rs.getLong(3));
					city.setName(rs.getString(4));
					cityList.add(city);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return cityList;
	}


	public List<Province> getRemoteProvinceList(int id){
		List<Province> provinceList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.province_id,a.province_name FROM address_province a  where  a.province_id = '"+String.valueOf(id)+"'";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Province province=new Province();
					province.setId(rs.getInt(1));
					province.setName(rs.getString(2));
					provinceList.add(province);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return provinceList;
	}

	public List<Area> getRemoteAreaList(long cityId,int id){
		List<Area> areaList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.district_id,a.city_id,a.district_name FROM address_district a  where  a.city_id = '"+String.valueOf(cityId)+"' and district_id='"+String.valueOf(id)+"'";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					Area area=new Area();
					area.setId(rs.getInt(1));
					area.setCityId(rs.getLong(2));
					area.setName(rs.getString(3));
					areaList.add(area);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return areaList;
	}



	public int updateSelective(Store storeInfo){
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "update ex_store set start_time ='"+storeInfo.getStartTime()+"' ,end_time = '"+storeInfo.getEndTime()+"',take_express_phone ='"+storeInfo.getTakeExpressPhone()+"',short_address ='"+storeInfo.getShortAddress()+"' where  id = '"+String.valueOf(storeInfo.getId())+"'";
				pstm = connection.prepareStatement(sql);

				int i = pstm.executeUpdate();
				connection.commit();
				if(i>0){
					return 1;
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return 0;
	}

	public int updateStoreaddress1(Store storeInfo){
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql =  "update ex_store set address ='"+storeInfo.getAddress()+"' where  id = '"+String.valueOf(storeInfo.getId())+"'";
				pstm = connection.prepareStatement(sql);

				int i = pstm.executeUpdate();
				connection.commit();
				if(i>0){
					return 1;
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return 0;
	}

	private List<ExpressType> getAllRemoteExpressList(){
		List<ExpressType> expressTypeList=new ArrayList<>();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.id ,a.name,a.shortname ,a.type,a.isdel ,a.expImage , a.sort FROM ex_express_type a where  a.isdel = '"+String.valueOf(0)+"' ORDER BY a.sort ASC";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					ExpressType expressType=new ExpressType();
					expressType.setExpId(rs.getInt(1));
					expressType.setExpName(rs.getString(2));
					expressType.setExpShortName(rs.getString(3));
					expressType.setType(rs.getInt(4));
					expressType.setIsDel(rs.getInt(5));
					expressType.setExpImage(rs.getString(6));
					expressType.setSort(rs.getInt(7));
					expressTypeList.add(expressType);
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return expressTypeList;
	}
	@Override
	public Store getStoreFromDB(int storeId){
		Store store=new Store();
		try{
			Connection connection = null;
			PreparedStatement pstm = null;
			try{
				Class.forName("com.mysql.jdbc.Driver");
				connection = DBUtil.getJeesiteConnection();
				connection.setAutoCommit(false);
				String sql = "select a.id,a.name ,a.short_name,a.owner,a.phone,a.take_express_phone,a.province_id,a.province_name,a.city_id,a.city_name," +
						"a.area_id,a.area_name,a.college_id,a.college_name,a.address,a.short_address," +
						"a.start_time,a.end_time,a.receive_ids,a.dispatch_ids ,a.isp2p,a.operate_type,a.status," +
						"a.isprinter,a.longitude,a.latitude,a.appid,a.appsecret,a.msgtempid,a.signtempid,a.domain,a.opendoor,sendexpress,a.advertisingUrl,a.advertisingMsg,a.yjmsgtempid,a.auditStatus,a.isOpenSignBack FROM ex_store a where  a.id = '"+String.valueOf(storeId)+"' ";
				pstm = connection.prepareStatement(sql);
				ResultSet rs=pstm.executeQuery();
				while(rs.next()){
					store.setId(rs.getInt(1));
					store.setName(rs.getString(2));
					store.setShortName(rs.getString(3));
					store.setOwner(rs.getString(4));
					store.setPhone(rs.getString(5));
					store.setTakeExpressPhone(rs.getString(6));
					store.setProvinceId(rs.getInt(7));
					store.setProvinceName(rs.getString(8));
					store.setCityId(rs.getLong(9));
					store.setCityName(rs.getString(10));
					store.setAreaId(rs.getLong(11));
					store.setAreaName(rs.getString(12));
					store.setCollegeId(rs.getInt(13));
					store.setCollegeName(rs.getString(14));
					store.setAddress(rs.getString(15));
					store.setShortAddress(rs.getString(16));
					store.setStartTime(rs.getString(17));
					store.setEndTime(rs.getString(18));
					store.setReceiveIds(rs.getString(19));
					store.setDispatchIds(rs.getString(20));
					store.setIsP2p(rs.getInt(21));
					store.setOperate_type(rs.getInt(22));
					store.setStatus(rs.getInt(23));
					store.setIsPrinter(rs.getString(24));
					store.setLongitude(rs.getString(25));
					store.setLatitude(rs.getString(26));
					store.setAppid(rs.getString(27));
					store.setAppsecret(rs.getString(28));
					store.setMsgtempid(rs.getString(29));
					store.setSigntempid(rs.getString(30));
					store.setDomain(rs.getString(31));
					store.setOpendoor(rs.getString(32));
					store.setSendexpress(rs.getString(33));
					store.setAdvertisingUrl(rs.getString(34));
					store.setAdvertisingMsg(rs.getString(35));
					store.setYjmsgtempid(rs.getString(36));
					store.setAuditStatus(rs.getInt(37));
					store.setIsOpenSignBack(rs.getBoolean(38));
				}
			}catch (Exception e){
				e.printStackTrace();
				throw new RuntimeException(e);
			}finally {
				if (pstm != null) {
					try {
						pstm.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						e.printStackTrace();
						throw new RuntimeException(e);
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return store;
	}

	@Override
	public String[] getStoreAddressByStoreIdAndExpId(int storeId, int expId){
		if(storeId == 0||expId == 0){
			return new String[3];
		}
		Store store = this.getStoreById(storeId);
		if(store == null|| Strings.isNullOrEmpty(store.getExpIdAddress())){
			return new String[3];
		}
		List<String> expIdAddress = Arrays.asList(store.getExpIdAddress().split(","));
		for (String str: expIdAddress) {
			if(!Strings.isNullOrEmpty(str)){
				String[] ea = str.split("-");
				if(ea.length > 0&&!Strings.isNullOrEmpty(ea[0])&&expId==Integer.parseInt(ea[0])){
					String[] arr = Strings.isNullOrEmpty(ea[1])?null:ea[1].split("_");
					return arr;
				}
			}
		}
		return new String[3];
	}

}
