package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.DeliveryDto;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.sms.SMSSendQueue;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.DeliveryQueryCondition;
import cn.haliaeetus.sendmessage.dao.DeliveryDao;
import cn.haliaeetus.sendmessage.exception.BusinessException;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.UserService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.config.RabbitConfig;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 待重构
 *
 * @author liutianyuan
 */
@Service
@Slf4j
public class DeliveryServiceImpl implements DeliveryService {

	@Resource
	private DeliveryDao deliveryDao;

	@Autowired
	private BasicInfoService basicInfoService;

	@Autowired
	private UserService userService;

	@Autowired
	private SmsRecordService smsRecordService;

	@Autowired
	private RabbitConfig rabbitConfig;

	@Override
	public boolean update(final DeliveryQueryCondition qc){
		if(qc.getBeginTime()!=null){
			SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd 00:00:00");
			String dateStr = sdf.format(qc.getBeginTime());
			qc.setBeginTime(Timestamp.valueOf(dateStr));
		}
		if(qc.getEndTime()!=null){
			SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd 23:59:59");
			String dateStr = sdf.format(qc.getEndTime());
			qc.setEndTime(Timestamp.valueOf(dateStr));
		}
		int result=deliveryDao.update(qc);
		log.info("入库签收回传修改业务表修改成功条数："+result);
		//修改数据
		if (result > 0) {
			return true;
		}
		return false;
	}

	@Override
	public Delivery getById(Integer storeId, int id) {
		DeliveryQueryCondition qc = new DeliveryQueryCondition(storeId);
		qc.setId(id);
		return deliveryDao.getById(qc);
	}

	@Override
	public void resendSms(Integer storeId, List<Integer> ids, String platform) throws Exception {
		List<Delivery> deliveriesList = new ArrayList<>();
		CacheTool cacheTool = CacheTool.initSmsFlagTool();
		User user = userService.getUserByStoreId(storeId.toString()).get(0);
		for (Integer id : ids) {
			Delivery delivery = getById(storeId, id);
			if (delivery != null) {
				delivery.setPlatform(platform);
				deliveriesList.add(delivery);
			}
		}
		deliveriesList.forEach(delivery -> {
			if (delivery.getStatus()==null|| delivery.getStatus().compareTo(Constants.ORDER_STATUS_STORED)!=0) {
				return;
			}
			cacheTool.setStringKey(String.format("%d_%s_%s_%d", delivery.getExpId(), delivery.getExpcode(), delivery.getUserPhone(), delivery.getStoreId()),"0");
			//修改业务表中的短信发送状态为发送中
			if (delivery.getId() != null) {
				this.updateSingleDeliverySmsStatus(delivery.getId().toString(), 0, storeId);
			}
			//Mq推送发短信消息
			SMSSendQueue smsSendQueue = new SMSSendQueue();
			smsSendQueue.setDeliveryDtos(getDeliveryDtoToSendSms(delivery));
			smsSendQueue.setUser(user);
			String toJson = new Gson().toJson(smsSendQueue);
			// 发送短信
			smsRecordService.send(toJson,0L,rabbitConfig.getSmsQueueName());
		});
	}

	private DeliveryDto getDeliveryDtoToSendSms(Delivery delivery) {
		DeliveryDto deliveryDto = new DeliveryDto();
		deliveryDto.setDelivery(delivery);
		deliveryDto.setSmsStatus(delivery.getSmsStatus() == null ? -1 : delivery.getSmsStatus());
		deliveryDto.setSendSuccessType(delivery.getSendSuccessType());
		deliveryDto.setResultInfo(delivery.getResultInfo());
		deliveryDto.setSortTime(delivery.getCreateTime());
		return deliveryDto;
	}

	@Override
	public boolean updateExpSignBackStatus(String expcode, String id,String reason) {
		deliveryDao.updateExpSignBackStatus(expcode,id,reason);
		log.info("快递状态已修改");
		return true;
	}

	@Override
	public void addExpressRecode(Integer storeId, String expCode, int status) {
		deliveryDao.addExpressRecode(storeId, expCode, status, new Date());
	}

	private Map<Integer, DeliveryDto> getDeliveryWithSmsStatus(
			List<DeliveryDto> deliveryDtoList, int storeId) {

		Map<Integer, DeliveryDto> totalDeliveryMap = new HashMap<Integer, DeliveryDto>();
		List<Integer> totalDeliveryIdList = new ArrayList<Integer>();
		if (deliveryDtoList.size() == 0) {
			return totalDeliveryMap;
		}
		for (DeliveryDto dto : deliveryDtoList) {
			//修改订单发送状态从业务表中获取
			dto.setSmsStatus(dto.getDelivery().getSmsStatus()==null?-1:dto.getDelivery().getSmsStatus());
			dto.setSendSuccessType(dto.getDelivery().getSendSuccessType());
			dto.setResultInfo(dto.getDelivery().getResultInfo());
			totalDeliveryMap.put(dto.getDelivery().getId(), dto);
//			totalDeliveryIdList.add(dto.getDelivery().getId());
			dto.setSortTime(dto.getDelivery().getCreateTime());
		}
		return totalDeliveryMap;
	}

	private int updateSingleDeliverySmsStatus(String id, Integer smsStatus,Integer storeId) throws BusinessException {
		List<String> ids = new ArrayList<>();
		ids.add(id);
		return this.updateDeliverySmsStatus(ids,smsStatus,storeId);
	}

	private int updateDeliverySmsStatus(List<String> ids, Integer smsStatus,Integer storeId) throws BusinessException {
		if(ids == null || ids.isEmpty() || smsStatus == null || storeId == null){
			return 0;
		}
		DeliveryQueryCondition deliveryQueryCondition = new DeliveryQueryCondition(storeId);
		deliveryQueryCondition.setList(ids);
		deliveryQueryCondition.setSmsStatus(smsStatus);
		int count = deliveryDao.updateDeliverySmsStatus(deliveryQueryCondition);
		return count;
	}
}
