package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.basedata.SignBackSwitchBean;
import cn.haliaeetus.sendmessage.bean.basedata.Store;
import cn.haliaeetus.sendmessage.bean.dto.SignBackDto;
import cn.haliaeetus.sendmessage.bean.express.common.ExpressRequestBean;
import cn.haliaeetus.sendmessage.bean.express.common.ExpressResponseBean;
import cn.haliaeetus.sendmessage.bean.express.station.ExpressAgentsBean;
import cn.haliaeetus.sendmessage.bean.express.trace.ExpressTracePushBean;
import cn.haliaeetus.sendmessage.framework.util.OkHttpUtil;
import cn.haliaeetus.sendmessage.jobs.DBUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.function.Function;

/**
 * @ClassName ExpressApiService
 * @Description
 * @Author SunJJ
 * @Date 2020/9/2 13:27
 * @Version V1.0
 **/
@Slf4j
public class ExpressApiService {
    private static final String LOGISTICS_URL = "http://logistics.fengzhan100.com";
    private SignBackSwitchBean signBackSwitchBean;
    private Store store;
    private int storeId;
    private int expId;
    private String shelfName;
    private String expCode;
    private String actionType;
    private String shelfCode;
    private String telPhone;

    private ExpressApiService(SignBackDto signBackDto) {
        this.store = signBackDto.getStore();
        this.expId = signBackDto.getExpId();
        this.storeId = signBackDto.getStore().getId();
        this.signBackSwitchBean = this.getSignBackSwitch(expId, storeId);
    }

    public static ExpressApiService of(SignBackDto signBackDto) {
        return new ExpressApiService(signBackDto);
    }

    /**
     * 处理站点信息
     *
     * @param function 对执行结果进行处理
     */
    public ExpressApiService processAgent(Function<SignBackSwitchBean, Boolean> function) throws Exception {
        String serviceType = getServiceTypeParam();
        if (Strings.isNullOrEmpty(serviceType)) {
            return this;
        }
        ExpressAgentsBean expressAgentsBean = ExpressAgentsBean.builder()
                .agentCode(String.valueOf(store.getId()))
                .ytStationCode(signBackSwitchBean.getYuantongStoreCode())
                .agentName(store.getShortName())
                .agentType("station")
                .agentOwner(store.getOwner())
                .agentOwnerPhone(store.getBackupPhone())
                .agentOfficePhone(store.getTakeExpressPhone())
                .agentBusinessTime(store.getStartTime() + "-" + store.getEndTime())
                .agentState("open")
                .provinceCode(String.valueOf(store.getProvinceId()))
                .provinceName(store.getProvinceName())
                .cityCode(String.valueOf(store.getCityId()))
                .cityName(store.getCityName())
                .areaCode(String.valueOf(store.getAreaId()))
                .areaName(store.getAreaName())
                .agentAddress(store.getShortAddress())
                .longitude(store.getLongitude())
                .latitude(store.getLatitude())
                .build();
        //请求参数封装
        ExpressRequestBean expressRequestBean = ExpressRequestBean.builder()
                .toCode(ExpressNameEnum.getExpressName(expId))
                .version("1.0")
                .serviceType(serviceType)
                .data_digest("")
                .data(JSONObject.toJSONString(expressAgentsBean))
                .build();
        //调用接口
        String resp = OkHttpUtil.postJson(LOGISTICS_URL, expressRequestBean);
        // 对返回结果进行处理
        ExpressResponseBean expressResponseBean = JSONObject.parseObject(resp, ExpressResponseBean.class);
        if ("STATION_REGISTER".equals(serviceType)) {
            signBackSwitchBean.setYuantongStoreCode(expressResponseBean.getStatusCode());
        }
        updateSignBackSwitch(signBackSwitchBean);
        return this;
    }

    /**
     * 推送物流轨迹
     */
    public ExpressApiService tracePush() throws Exception {
        //封装业务参数
        ExpressTracePushBean expressTracePushBean = ExpressTracePushBean.builder()
                //.toCode(ExpressNameEnum.getExpressName(signBackDto.getExpId()))
                .expressCode(expCode)
                .expressTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")))
                .operationType(actionType)
                .shelfCode(shelfName)
                .takeCode(shelfCode)
                .receiverPhone(telPhone)
                .deliveryFallReasonCode("2")
                .deliveryFallReason("客户拒收")
                .ytStationCode(signBackSwitchBean.getYuantongStoreCode())
                .agentCode(String.valueOf(store.getId()))
                .agentName(store.getShortName())
                .agentType("station")
                .agentOwnerPhone(store.getBackupPhone())
                .agentOfficePhone(store.getTakeExpressPhone())
                .provinceName(store.getProvinceName())
                .cityName(store.getCityName())
                .areaName(store.getAreaName())
                .agentAddress(store.getShortAddress())
                .build();
        // 请求参数封装
        ExpressRequestBean expressRequestBean = ExpressRequestBean.builder()
                .version("1.0")
                .serviceType("STATION_EXPRESS_TRACE")
                .data_digest("")
                .toCode(ExpressNameEnum.getExpressName(expId))
                .data(JSONObject.toJSONString(expressTracePushBean))
                .build();
        // 调用接口
        String resp = OkHttpUtil.postJson(LOGISTICS_URL, expressRequestBean);
        // 对返回结果进行处理
        return this;
    }

    public ExpressApiService pddPush() {
        return this;
    }

    public ExpressApiService caiNiaoPush() {
        return this;
    }

    private String getServiceTypeParam() {
        if (!signBackSwitchBean.getIs_syStore()) {
            return "STATION_REGISTER";
        }
        if (!signBackSwitchBean.getIs_syAcperson()) {
            return "ACCOUNT_PERSON_INFO_BY_YTO";
        }
        if (!signBackSwitchBean.getIs_syUpStore()) {
            return "STATION_UPDATE";
        }
        return null;
    }

    private SignBackSwitchBean getSignBackSwitch(int expId, int storeId) {
        try {
            Connection connection = null;
            PreparedStatement pstm = null;
            try {
                Class.forName("com.mysql.jdbc.Driver");
                connection = DBUtil.getJeesiteConnection();
                connection.setAutoCommit(false);
                String sql = "select a.storeId,a.expId,a.is_valid,a.is_syStore,a.is_syAcPerson,a.is_syUpStore,a.yuantongStoreCode FROM sign_back_switch a where a.storeId = " + storeId + " and a.expId = " + expId;
                pstm = connection.prepareStatement(sql);
                ResultSet rs = pstm.executeQuery();
                SignBackSwitchBean signBackSwitchBean = null;
                while (rs.next()) {
                    signBackSwitchBean = new SignBackSwitchBean();
                    signBackSwitchBean.setStoreId(rs.getString(1));
                    signBackSwitchBean.setExpId(rs.getString(2));
                    signBackSwitchBean.setIs_valid(rs.getBoolean(3));
                    signBackSwitchBean.setIs_syStore(rs.getBoolean(4));
                    signBackSwitchBean.setIs_syAcperson(rs.getBoolean(5));
                    signBackSwitchBean.setIs_syUpStore(rs.getBoolean(6));
                    signBackSwitchBean.setYuantongStoreCode(rs.getString(7));
                }
                return signBackSwitchBean;
            } catch (Exception e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            } finally {
                if (pstm != null) {
                    try {
                        pstm.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                        throw new RuntimeException(e);
                    }
                }
                if (connection != null) {
                    try {
                        connection.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                        throw new RuntimeException(e);
                    }
                }
            }
        } catch (Exception e) {
            e.getStackTrace();
        }
        return null;
    }

    public void updateSignBackSwitch(SignBackSwitchBean signBackSwitchBean) {
        try {
            Class.forName("com.mysql.jdbc.Driver");
            Connection con = DBUtil.getJeesiteConnection();
            StringBuilder sb = new StringBuilder();
            sb.append("update sign_back_switch set ");
            if (!signBackSwitchBean.getIs_syStore()) {
                sb.append(" is_syStore = 1,");
            }
            if (!signBackSwitchBean.getIs_syAcperson()) {
                sb.append(" is_syAcPerson = 1,");
            }
            if (!signBackSwitchBean.getIs_syUpStore()) {
                sb.append(" is_syUpStore = 1,");
            }
            if (signBackSwitchBean.getYuantongStoreCode() != null && !"".equals(signBackSwitchBean.getYuantongStoreCode())) {
                sb.append(" yuantongStoreCode = '" + signBackSwitchBean.getYuantongStoreCode() + "',");
            }
            sb.append(" is_valid  = 1");
            sb.append(" where storeId = " + signBackSwitchBean.getStoreId() + " AND expId = " + signBackSwitchBean.getExpId());
            PreparedStatement ps = con.prepareStatement(sb.toString());
            ps.executeUpdate();
            con.close();
        } catch (Exception e) {
            log.error(e.getMessage());
        }

    }

    public static enum ExpressNameEnum {
        YUN_DA(17, "YUNDA"),
        ZHONG_TONG(187, "ZTO"),
        YUAN_TONG(202, "YTO"),
        BAI_SHI(223, "BAISHI"),
        SHEN_TONG(272, "STO"),
        JI_TU(296, "JITU");

        private Integer expressId;
        private String expressName;

        public Integer getExpressId() {
            return expressId;
        }

        public void setExpressId(Integer expressId) {
            this.expressId = expressId;
        }

        public String getExpressName() {
            return expressName;
        }

        public void setExpressName(String expressName) {
            this.expressName = expressName;
        }

        ExpressNameEnum(Integer expressId, String expressName) {
            this.expressId = expressId;
            this.expressName = expressName;
        }

        public static String getExpressName(Integer expressId) {
            for (ExpressNameEnum expressNameEnum : ExpressNameEnum.values()) {
                if (expressNameEnum.getExpressId().equals(expressId)) {
                    return expressNameEnum.getExpressName();
                }
            }
            return "";
        }
    }

    public static void main(String[] args) throws Exception {
        ExpressApiService.of(new SignBackDto()).processAgent(item -> Boolean.TRUE).tracePush();
    }
}
