package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.MsgTemplate;
import cn.haliaeetus.sendmessage.bean.basedata.*;
import cn.haliaeetus.sendmessage.bean.delivery.*;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataRequest;
import cn.haliaeetus.sendmessage.bean.sms.SmsDataResponse;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.dao.SmsRecordDao;
import cn.haliaeetus.sendmessage.framework.factory.platform.PlatformFactory;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.service.*;
import cn.haliaeetus.sendmessage.service.delivery.DeliveryMsgTemplateConfigService;
import cn.haliaeetus.sendmessage.service.delivery.DeliveryMsgTemplateService;
import cn.haliaeetus.sendmessage.service.delivery.DeliverySpecialMsgTemplateService;
import cn.haliaeetus.sendmessage.service.delivery.DeliveryStatusService;
import cn.haliaeetus.sendmessage.service.rsync.ConsumeRecordService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ValidationUtils;
import cn.haliaeetus.sendmessage.utils._;
import cn.haliaeetus.sendmessage.validate.groups.SendSms;
import com.google.common.base.Strings;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

import static cn.haliaeetus.sendmessage.utils._.isEmpty;
import static cn.haliaeetus.sendmessage.utils._.trimToEmpty;

/**
 * @ClassName MessageSendServiceImpl
 * @Description TODO
 * @Author Administrator
 * @Date 2019/6/25 0025 下午 15:15
 * @Version 1.0
 **/
@Service
public class MessageSendServiceImpl implements MessageSendService {

    private static final Logger LOG = Logger.getInstance(MessageSendServiceImpl.class);

    @Autowired
    private DeliveryService deliveryService;

    @Autowired
    private DeliveryStatusService deliveryStatusService;

    @Autowired
    private BasicInfoService basicInfoService;

    @Autowired
    private StoreStaffService staffService;

    @Autowired
    private DeliveryMsgFilterChain msgFilterChain;

    @Autowired
    private DeliveryMsgTemplateConfigService msgTemplateConfigService;

    @Autowired
    private DeliverySpecialMsgTemplateService specialMsgTemplateService;

    @Autowired
    private DeliveryMsgTemplateService msgTemplateService;

    @Autowired
    private StoreShelfService shelfService;

    @Autowired
    private SmsRecordService smsRecordService;

    @Autowired
    private SmsRecordDao smsRecordDao;

    @Autowired
    private PlatformFactory platformFactory;

    @Autowired
    private ConsumeRecordService consumeRecordService;

    @Override
    public void syncResendInStoreMsg(final Delivery delivery, final int smsStatus, final String bussCode, final User user) throws Exception {
        try {
            DeliveryStatus deliveryStatus = deliveryStatusService.getEarliestStatusOfDelivery(delivery.getStoreId(), delivery.getId(), Constants.ORDER_STATUS_STORED);
            if (deliveryStatus == null) {
                return;
            }
            Store store = basicInfoService.getStoreById(delivery.getStoreId());
            StoreStaff storeStaff = null;
            if (StringUtils.isNotBlank(delivery.getDeliveryUid())) {
                storeStaff = staffService.getById(delivery.getDeliveryUid());
            }

            DeliveryInfo deliveryInfo = new DeliveryInfo(delivery, deliveryStatus, store, storeStaff);

            // 校验参数
            ValidationUtils.ValidationResult validationResult = ValidationUtils.validate(deliveryInfo, SendSms.class);
            if (validationResult.hasErrors) {
                return;
            }

            //根据派件信息过滤是否发短信
            if (!msgFilterChain.doFilters(deliveryInfo)) {
                return;
            }

            DeliveryMsgTemplateInfo templateInfo = getTemplateInfo(deliveryInfo, Constants.MSG_TYPE_SMS, Constants.BUSS_CODE_DELIVERY_IN_STORE);
            if (null == templateInfo) {
                return;
            }

            String msg = msgTemplateHandle(deliveryInfo, templateInfo);
            if (msg != null && msg.length() > 0) {
                int smsCount = this.send(user, store, deliveryInfo.getDelivery(), templateInfo.getBussCode(), msg, user.getSmsPrice());
                // 统计短信条数用于计费
                if (smsCount > 0) {
                    CacheTool cacheTool = CacheTool.initSmsCount();
                    cacheTool.incrBy(user.getUserId(), smsCount);
                }
            }
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
            throw new Exception(e);
        }
    }

    private DeliveryMsgTemplateInfo getTemplateInfo(DeliveryInfo deliveryInfo, int msgTypeSms, String bussCode) {
        // 获取派件业务码前缀，用于匹配相应信息模板
        String bussCodePrefix = null;
        if (ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_ZT)) {
            // 自提
            bussCodePrefix = "";
        } else if (ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_PS)) {
            // 配送
            bussCodePrefix = "OUT_";
        } else if (ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_P2P)) {
            // 送货上门
            bussCodePrefix = "P2P_";
        }
        if (bussCodePrefix == null) {
            return null;
        }

        Integer source = deliveryInfo.getDeliveryStatus().getSource();
        if (source == null || source == 0) {
            source = Constants.CLIENT_TYPE_BS_2;
        }
        return new DeliveryMsgTemplateInfo(deliveryInfo.getDelivery().getStoreId(), source, msgTypeSms, bussCodePrefix + bussCode);
    }


    private String msgTemplateHandle(DeliveryInfo deliveryInfo, DeliveryMsgTemplateInfo templateInfo) throws Exception {
        String msg = null;
        // 特殊模板
        DeliverySpecialMsgTemplate specialMsgTemplate = specialMsgTemplateService.getMsgTemplate(templateInfo.getStoreId(), templateInfo.getClientType(), templateInfo.getMsgType(), templateInfo.getBussCode());
        if (specialMsgTemplate == null) {
            // 获取默认的特殊模板
            specialMsgTemplate = specialMsgTemplateService.getDefaultMsgTemplateOfStore(templateInfo.getClientType(), templateInfo.getMsgType(), templateInfo.getBussCode());
        }
        // 获取模板对应的配置信息
        if (specialMsgTemplate != null) {
            DeliveryMsgTemplateConfig config = getStoreMsgTemplateConfig(templateInfo);
            specialMsgTemplate.setConfig(config);
            msg = handleMsgTemplate(specialMsgTemplate, deliveryInfo);
        } else {
            // TODO DeliveryMsgTemplate msgTemplate = this.getStoreMsgTemplate(deliveryInfo.getDelivery().getStoreId().toString());
            DeliveryMsgTemplate msgTemplate = msgTemplateService.getStoreMsgTemplate(deliveryInfo.getDelivery().getStoreId().toString());
            if (msgTemplate != null) {
                msg = handleMsgTemplate(msgTemplate, deliveryInfo);
            }
        }
        // TODO 什么场景下会满足以下条件？？？
        if (msg != null && msg.indexOf("[traceUrl]") > -1 || msg.indexOf("[traceUrlWithAd]") > -1 || msg.indexOf("[traceUrlWithRedPack]") > -1) {
            DeliveryMsgTemplateConfig config = getStoreMsgTemplateConfig(templateInfo);
            if (config != null) {
                // -1为没有短链接的默认模板
                config.setType(-1);
                LOG.info("模板信息为：++++++++" + msg);
                DeliveryMsgTemplate template = msgTemplateService.getMsgTemplate(config, templateInfo.getBussCode());
                if (template != null) {
                    template.setConfig(config);
                    msg = handleMsgTemplate(template, deliveryInfo);
                }
            }
        }
        return msg;
    }

    /**
     * 获取门店信息模板配置
     */
    private DeliveryMsgTemplateConfig getStoreMsgTemplateConfig(DeliveryMsgTemplateInfo templateInfo) {
        DeliveryMsgTemplateConfig msgTemplateConfig = msgTemplateConfigService.getMsgTemplateConfig(
                templateInfo.getStoreId(), templateInfo.getClientType(), templateInfo.getMsgType());
        if (msgTemplateConfig == null) {
            msgTemplateConfig = msgTemplateConfigService.getDefaultMsgTemplateConfig(templateInfo.getClientType(), templateInfo.getMsgType());
        }
        LOG.record(String.format("Get msg template config of store, %s, %s.", msgTemplateConfig, templateInfo));
        return msgTemplateConfig;
    }

    /**
     * 处理信息模板
     */
    private String handleMsgTemplate(MsgTemplate msgTemplate, DeliveryInfo deliveryInfo) {
        String content = msgTemplate.getContent();
        String bussCode = msgTemplate.getBussCode();
        int type = msgTemplate.getType();

        int expId = deliveryInfo.getDelivery().getExpId();
        ExpressType expType = basicInfoService.getExpressById(expId);
        content = content.replace("[e]", expType == null ? "" : trimToEmpty(expType.getExpName()));
        String expcode = deliveryInfo.getDelivery().getExpcode();
        expcode = expcode.substring(0, 1) + "**" + expcode.substring(expcode.length() - 4);
        if (StringUtils.contains(bussCode, "DELIVERY_DETAIN") && type == 1 &&
                (expId == Constants.EXPRESS_TYPE_YD)) {
            content = content.replace("[o]", "包裹");
        } else {
            content = content.replace("[o]", expcode);
        }
        content = content.replace("[w_o]", deliveryInfo.getDelivery().getExpcode());
        content = content.replace("[storeId]", deliveryInfo.getStore().getId() + "");
        content = content.replace("[s]", trimToEmpty(deliveryInfo.getStore().getShortName()));
        content = content.replace("[instoreDay]", handleInstoreDay(deliveryInfo));
        content = content.replace("[takeCode]", handleTakeCode(msgTemplate, deliveryInfo.getDelivery().getShelfCode()));
        content = content.replace("[shelf]", handleShelf(deliveryInfo));

        Store store = deliveryInfo.getStore();
        StoreShelf shelf = null;
        if (deliveryInfo.getDelivery().getShelfId() != null) {
            shelf = shelfService.getById(deliveryInfo.getDelivery().getShelfId());
        }
        if (shelf != null && shelf.getShelfAddress() != null && shelf.getShelfAddress().trim().length() > 0
                && shelf.getShelfTelephone() != null && shelf.getShelfTelephone().trim().length() > 0) {
            content = content.replace("[storeAddress]", shelf.getShelfAddress().trim());
            content = content.replace("[storePhone]", shelf.getShelfTelephone().trim());
        } else {
            content = content.replace("[storeAddress]", handleShortAddress(msgTemplate, store));
            content = content.replace("[storePhone]", trimToEmpty(store.getTakeExpressPhone()));
        }

        content = content.replace("[userPhone]", trimToEmpty(deliveryInfo.getDelivery().getUserPhone()));
        content = content.replace("[earlyBusinessTime]", handleTime(store.getStartTime(), Constants.STORE_DEFAULT_START_BUSINESS_TIME));
        content = content.replace("[latestBusinessTime]", handleTime(store.getEndTime(), Constants.STORE_DEFAULT_END_BUSINESS_TIME));
        content = content.replace("[enter]", System.getProperty("line.separator"));   // 微信换行

        StoreStaff storeStaff = deliveryInfo.getStoreStaff();
        if (storeStaff != null) {
            content = content.replace("[deliveryUname]", storeStaff.getName());
            content = content.replace("[deliveryUmobile]", storeStaff.getMobile());
        }

        return content;
    }

    private String handleShortAddress(MsgTemplate msgTemplate, Store store) {
        String storeAddress = null;
        if (msgTemplate.getConfig() != null && msgTemplate.getConfig() instanceof DeliveryMsgTemplateConfig) {
            DeliveryMsgTemplateConfig config = (DeliveryMsgTemplateConfig) msgTemplate.getConfig();
            storeAddress = config.getStoreAddress();
        }
        if (isEmpty(storeAddress)) {
            storeAddress = trimToEmpty(store.getShortAddress());
        }
        return trimToEmpty(storeAddress);
    }

    /**
     * 生成入库日期
     */
    private String handleInstoreDay(DeliveryInfo deliveryInfo) {
        String instoreDay = "";
        if (deliveryInfo.getInstoreDate() != null) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(deliveryInfo.getInstoreDate());
            instoreDay = calendar.get(Calendar.DAY_OF_MONTH) + "";
        }
        return instoreDay;
    }

    private String handleTime(String time, String defaultTime) {

        time = _.isNotEmpty(time) ? time : Constants.STORE_DEFAULT_START_BUSINESS_TIME;
        String[] array = time.split(":");
        int startHour = Integer.parseInt(array[0]);
        int startMinute = Integer.parseInt(array[1].trim());
        if (startMinute != 0 && startMinute != 30) {
            return time;
        }

        String result = String.valueOf(startHour) + "点";
        if (startMinute == 30) {
            result += "半";
        }
        return result;
    }

    /**
     * 生成取货码
     */
    private String handleTakeCode(MsgTemplate msgTemplate, String takeCode) {
        String handledTakeCode = "";
        if (msgTemplate instanceof DeliveryMsgTemplate) {
            DeliveryMsgTemplate template = (DeliveryMsgTemplate) msgTemplate;
            if (ObjectUtils.equals(template.getContainsShelf(), Constants.YES) &&
                    ObjectUtils.equals(template.getContainsTakeCode(), Constants.YES)) {
                handledTakeCode = takeCode;
            }
        }

        if (isEmpty(handledTakeCode)) {
            handledTakeCode = takeCode;
        }

        return trimToEmpty(handledTakeCode);
    }

    private String handleShelf(DeliveryInfo deliveryInfo) {
        String shelfName = null;
        Integer shelfId = deliveryInfo.getDelivery().getShelfId();
        if (shelfId != null) {
            StoreShelf shelf = shelfService.getById(shelfId);
            if (shelf != null) {
                shelfName = shelf.getShelfName();
            }
        }
        return trimToEmpty(shelfName);
    }


    /**
     * Copy DeliverySMSSender.send
     */
    private int send(User user,Store store, Delivery delivery, String bussCode, String msg, BigDecimal smsPrice) throws Exception {
        SmsRecord smsRecord = null;
        // 查询短信表记录
        List<SmsRecord> smsRecordList = smsRecordDao.selectSmsRecordsOfStore(delivery.getStoreId(), delivery.getId().toString());
        if (null == smsRecordList || smsRecordList.size() == 0) {
            return 0;
        }
        smsRecord = smsRecordList.get(0);
        smsRecord.setSmsCount(countFeeNum(user.getComSign().concat(msg)));
        int id = smsRecord.getId();
        //Reminder字段 获取快件的提醒方式0短信，1微信提醒 , 2两个都提醒 , 3,先发微信若失败才发短信
        List<String> platforms = getPlatform(delivery, user.getReminder());
        Iterator<String> iterator = platforms.iterator();
        int smsCount = 0;
        while (iterator.hasNext()) {
            String platform = iterator.next();
            // 发送短信
            SmsDataResponse smsDataResponse = platformFactory.getPlatform(platform).sendSms(SmsDataRequest.builder()
                    .user(user)
                    .store(store)
                    .delivery(delivery)
                    .smsRecord(smsRecord)
                    .id(id)
                    .msg(msg)
                    .smsCount(smsRecord.getSmsCount())
                    .build());
            String next = "";
            if (iterator.hasNext()) {
                next = iterator.next();
            }
            // 发送失败
            if (_.isEmpty(smsDataResponse.getSmsId())) {
                continue;
            }
            // 添加短信金钱记录，不是阿里巴巴或者短信的短信都计费
            if (!platform.equals(Constants.PLATFORM_ALBB) && !platform.equals(Constants.PLATFORM_WECHAT)) {
                // 发送的短信条数
                smsCount = smsDataResponse.getSmsCount();
                consumeRecordService.sendMsgUpConsume(smsRecord, user, !_.isEmpty(smsDataResponse.getSmsId()));
            }
            smsRecord.setContent(msg);
            smsRecord.setBussCode(bussCode);
            smsRecord.setSmsPrice(smsPrice);
            smsRecord.setStatus(Constants.SMS_SEND_STATUS_SUCCESS);
            smsRecord.setResultInfo("0");
            smsRecord.setSmsCount(smsDataResponse.getSmsCount());
            smsRecord.setSendSuccessType(platform.equals(Constants.PLATFORM_WECHAT) ? 2 : 1);
            smsRecord.setIsmgid(smsDataResponse.getSmsId());
            smsRecord.setId(id);
            smsRecord.setCreatetime(null);
            smsRecordService.update(smsRecord);
            // 发送成功，跳出循环
            if (next.equals("OR")) {
                break;
            }
        }
        deliveryService.addExpressRecode(delivery.getStoreId(), delivery.getExpcode(), 0);
        return smsCount;
    }

    public static int countFeeNum(String content) {
        int len = content.length();
        int feeCount = 1;
        if (len > 70) {

            feeCount = new Double(Math.ceil((float) len / 67.0f)).intValue();
        }
        return feeCount;
    }

    /**
     * 获取平台信息，如果平台为空则选择默认平台（数据库设置），如果数据库为空，那么就默认
     *
     * @param delivery
     * @param reminder
     */
    private List<String> getPlatform(Delivery delivery, int reminder) {
        List<String> platforms = new ArrayList<>();
        if (Strings.isNullOrEmpty(delivery.getPlatform())) {
            String platform = basicInfoService.getConfigurationByCode("defaultSmsPlatform").getOValue1();
            if (Strings.isNullOrEmpty(platform)) {
                platform = Constants.PLATFORM_DEFAULT;
            }
            switch (reminder) {
                case 0:
                    // 只发短信
                    platforms.add(platform);
                    break;
                case 1:
                    // 只发微信
                    platforms.add(Constants.PLATFORM_WECHAT);
                    break;
                case 2:
                    // 同时发送短信和微信
                    // platforms.add(Constants.PLATFORM_WECHAT);
                    // AND 代表无论发送成功失败，都使用下一个平台发送
                    // platforms.add("AND");
                    platforms.add(platform);
                    break;
                case 3:
                    // 先发微信，微信发送失败后发送短信
                    // platforms.add(Constants.PLATFORM_WECHAT);
                    // OR 代表上一个平台发送失败后再选择下一个平台发送
                    // platforms.add("OR");
                    platforms.add(platform);
            }
        } else {
            platforms.add(delivery.getPlatform());
        }
        return platforms;
    }
}
