package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.basedata.StoreShelf;
import cn.haliaeetus.sendmessage.bean.dto.StoreShelfDto;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.DeliveryQueryCondition;
import cn.haliaeetus.sendmessage.dao.StoreShelfDao;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.framework.toolkit.PagedList;
import cn.haliaeetus.sendmessage.service.StoreShelfService;
import cn.haliaeetus.sendmessage.utils._;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class StoreShelfServiceImpl implements StoreShelfService {

	private static final Logger LOG = Logger.getInstance(StoreShelfServiceImpl.class);
	@Autowired
	private StoreShelfDao storeShelfDao;
	
	@Override
	public List<StoreShelf> getAll() {
		return storeShelfDao.getAll();
	}
	
	@Override
	public List<StoreShelf> getByStoreIdAndShelfName(int storeId,
			String shelfName) {
		return storeShelfDao.getListByStoreIdAndShelfName(storeId, shelfName);
	}
	
	@Override
	public List<StoreShelf> getByStoreIdAndShelfName(int id,int storeId,
			String shelfName) {
		return storeShelfDao.getListByStoreIdAndShelfName(id, storeId, shelfName);
	}

	
	/**
	 * 首先从缓存获取货架信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * @return  货架不存在时返回null
	 */
	@Override
	public StoreShelf getById(int id) {
		CacheTool cacheTool = CacheTool.initShelf();
		StoreShelf shelf = (StoreShelf)cacheTool.getBean(id + "");
		if(shelf != null){
			//LOG.record("从memcached获取货架信息:" + id);
			return shelf;
		}else{
			LOG.record("从数据库获取货架信息:" + id);
			shelf = storeShelfDao.getById(id);
			
			if(shelf != null){
				
				cacheTool.saveBean(id + "", shelf);
				LOG.record("刷新货架memcached:" + id);
				return shelf;
			}
			return null;
		}
		
	}

	/**
	 * 首先从缓存获取门店货架信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * @return  门店货架不存在时返回null
	 */
	@Override
	public List<StoreShelf> getByStoreId(int storeId) {
		CacheTool cacheTool = CacheTool.initStoreShelf();
		List<StoreShelf> shelfList = (List<StoreShelf>)cacheTool.getBean(storeId + "", StoreShelf.class);
		if(_.isNotEmpty(shelfList)){
			//LOG.record("从memcached获取门店货架信息:" + storeId);
			return shelfList;
		}else{
			LOG.record("从数据库获取门店货架信息:" + storeId);
			shelfList = storeShelfDao.getListByStoreId(storeId,0,0);
			
			if(_.isNotEmpty(shelfList)){
				
				cacheTool.saveBean(storeId + "", shelfList);
				LOG.record("刷新门店货架memcached:" + storeId);
				return shelfList;
			}
			return null;
		}
		
	}

	@Override
	public PagedList<StoreShelf> getByStoreId(int storeId, int p, int pageSize) {
		List<StoreShelf> list = storeShelfDao.getListByStoreId(storeId,p,pageSize);
		long count = storeShelfDao.count(storeId);
		
		return PagedList.of(count,list,p,pageSize);
	}
	
	//flag标示按什么排序：1，标示按时间降序；0，按包裹数降序
	@Override
	public List<StoreShelfDto> getDtoByStoreId(int storeId, int flag) {
		List<Integer> statusList = new ArrayList<Integer>();
		statusList.add(Constants.ORDER_STATUS_STORED);
		statusList.add(Constants.ORDER_STATUS_REJECT_PICK_UP);
		if(flag == 1){
			return storeShelfDao.getDtoTimeListByStoreId(storeId,0,0,statusList);
		}else if(flag == 0){
			return storeShelfDao.getDtoNumListByStoreId(storeId,0,0,statusList);
		}
		return null;
	}

	@Override
	public PagedList<StoreShelfDto> getDtoByStoreId(int storeId, int flag, int p,
			int pageSize) {
		List<Integer> statusList = new ArrayList<Integer>();
		statusList.add(Constants.ORDER_STATUS_STORED);
		statusList.add(Constants.ORDER_STATUS_REJECT_PICK_UP);
		
		List<StoreShelfDto> list = new ArrayList<StoreShelfDto>();
		int count=0;
		if(flag == 1){
			list = storeShelfDao.getDtoTimeListByStoreId(storeId,p,pageSize,statusList);
			count =  storeShelfDao.getDtoTimeListByStoreId(storeId,p,0,statusList).size();
			
		}else if(flag == 0){
			list = storeShelfDao.getDtoNumListByStoreId(storeId,p,pageSize,statusList);
			count =  storeShelfDao.getDtoNumListByStoreId(storeId,p,0,statusList).size();
		}
		return PagedList.of(count,list,p,pageSize);
		
		
		
	
	}

	@Override
	public int insert(int storeId, StoreShelf shelf) {
		Date time = new Date(System.currentTimeMillis());
		String barCode = getBarCode(storeId,true);
//		StoreShelf shelf = new StoreShelf();
		shelf.setIsdel(0);
//		shelf.setShelfName(shelfName);
		shelf.setStoreId(storeId);
		shelf.setTime(time);
		shelf.setBarCode(barCode);
		int result=storeShelfDao.insert(shelf);
		if(result == 1){
    		int shelfId = shelf.getId();
    		CacheTool cacheTool = CacheTool.initStoreShelf();
    		List<StoreShelf> shelfList = storeShelfDao.getListByStoreId(storeId,0,0);
            cacheTool.saveBean(storeId + "", shelfList);
            cacheTool = CacheTool.initShelf();
            cacheTool.saveBean(shelfId + "", shelf);
		}
		return result;
	}

	/**
	 * 更新货架，同时更新门店货架缓存和货架缓存
	 */
    @Override
    public int update(StoreShelf shelf) {
    	
    	int result = storeShelfDao.updateSelective(shelf);
    	if(result == 1){
    		int storeId = shelf.getStoreId();
    		int shelfId = shelf.getId();
        	CacheTool cacheTool = CacheTool.initStoreShelf();
    		List<StoreShelf> shelfList = storeShelfDao.getListByStoreId(storeId,0,0);
            cacheTool.saveBean(storeId + "", shelfList);
            cacheTool = CacheTool.initShelf();
            cacheTool.saveBean(shelfId + "", shelf);
    	}
    	return result;
    	
    }

	/**
	 * 删除货架，同时删除门店货架缓存和货架缓存
	 */
    @Override
    public int delete(int id) {
    	//删除之前先获取货架，否则删除后就查不到了
    	StoreShelf shelf = storeShelfDao.getById(id);
    	
    	int result = storeShelfDao.delete(id);
    	if(result == 1){
    		
    		int storeId = shelf.getStoreId();
    		int shelfId = shelf.getId();
        	CacheTool cacheTool = CacheTool.initStoreShelf();
    		List<StoreShelf> shelfList = storeShelfDao.getListByStoreId(storeId,0,0);
            cacheTool.saveBean(storeId + "", shelfList);
            cacheTool = CacheTool.initShelf();
            cacheTool.saveBean(shelfId + "", null);
    	}
    	return result;
    	
    }

	@Override
	public String getBarCode(int storeId,boolean isdel) {
		
		StoreShelf maxShelf = storeShelfDao.getMaxBarCode(storeId);
		
		int maxIndex = 0;
		
		if(maxShelf != null &&maxShelf.getBarCode() != null&& maxShelf.getBarCode().length()>0){
			maxIndex = _.toInt(maxShelf.getBarCode().substring(8));;
		}
		
		String barCodeString = "";
		
		if (maxIndex < 999999999) {
			barCodeString = "HJ " + _.f("%05d", storeId) + _.f("%04d", maxIndex+1);
		}else {
            StoreShelf minShelfIsdel = storeShelfDao.getMinBarCodeIsdel(storeId);
			barCodeString = minShelfIsdel.getBarCode();
			if(isdel){
				storeShelfDao.deleteReal(minShelfIsdel.getId());
			}
		}
		return barCodeString;
	}

	@Override
	public void updateBarCodeBlank(int storeId) {
		List<StoreShelf> shelfList=storeShelfDao.getListByBarcode(storeId,"");
		LOG.info("list size:"+shelfList.size());
		int count=0;
		for(StoreShelf shelf:shelfList){
			LOG.info("shelf id:"+shelf.getId());
			shelf.setBarCode(getBarCode(shelf.getStoreId(), true));
			storeShelfDao.update(shelf);
			LOG.info(String.format("%s,%s,shelf update success!",shelf.getShelfName(),shelf.getBarCode()));
			LOG.info(++count+"");
		}
	}

	//设为默认货架
	@Override
	public void setIsDefault(StoreShelfDto storeShelfDto) {
		storeShelfDao.setIsDefault(storeShelfDto);
	}

	@Override
	public void setIsDefaultAll(StoreShelfDto storeShelfDto) {
		storeShelfDao.setIsDefaultAll(storeShelfDto);
	}

}
