package cn.haliaeetus.sendmessage.service.impl;

import cn.haliaeetus.sendmessage.bean.basedata.*;
import cn.haliaeetus.sendmessage.bean.sms.SmsSendStatistics;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.controller.web.QueryCondition.UserQueryCondition;
import cn.haliaeetus.sendmessage.dao.SmsSendStatisticsDao;
import cn.haliaeetus.sendmessage.dao.UserDao;
import cn.haliaeetus.sendmessage.framework.base.digt.Md5Util;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.FjUserService;
import cn.haliaeetus.sendmessage.service.UserService;
import cn.haliaeetus.sendmessage.utils.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

//import cn.haliaeetus.util.Menu;

@Service
public class UserServiceImpl implements UserService {
	private static final Logger LOG = Logger.getInstance(UserServiceImpl.class);
	@Autowired
	private UserDao userDAO;
	@Autowired
	private BasicInfoService basicInfoService;

	@Autowired
	private FjUserService fjUserService;
	
	@Autowired
	private SmsSendStatisticsDao smsSendStatisticsDao;

	/**
	 * 查询所有用户，供缓存刷数据使用。
	 * 
	 * @return
	 */
	public List<User> getAllUser() {
		return userDAO.getAllUser();
	}

	/**
	 * 根据ID和密码查询用户。登录使用。不做缓存。
	 * 
	 * @param userId
	 *            必填
	 * @param password
	 *            不需要时传null
	 * @return
	 */
	@Override
	public User getUserByIdAndPwd(String userId, String password) {
		return userDAO.getUserByIdAndPwd(userId, password);
	}

	/**
	 * 首先从缓存获取用户信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
	 * 
	 * @param userId
	 * @return 用户不存在时返回null
	 */
	public User getUserById(String userId) {

		CacheTool cacheTool = CacheTool.initUser();
		User user = (User) cacheTool.getBean(userId);
		if (user != null && user.getStatus()!=null && (user.getStatus()==1||user.getStatus()==3)) {
			//LOG.record("从memcached获取用户信息:" + userId);
			return user;
		} else {
			user = userDAO.getUserByIdAndPwd(userId, null);
			LOG.record("从远程接口获取用户信息:" + userId);
			if (user != null) {
				cacheTool.saveBean(userId, user);
				LOG.record("刷新用户memcached:" + userId);
				return user;
			}
			return null;
		}

	}

	/**
	 * 更新用户信息，同时更新缓存
	 * 
	 * @param qc
	 */
	public void updateUser(UserQueryCondition qc) {
		if (qc != null && qc.getEntity() != null) {
			int result = userDAO.update(qc);
			if (result > 0) {
				CacheTool cacheTool = CacheTool.initUser();
				User user = qc.getEntity();
				cacheTool.saveBean(user.getUserId(), null);
			}
		}

	}

	@Override
	public User getUserByIdAndStorePwd(String userId, String storepwd) {
		return userDAO.getUserByIdAndStorePwd(userId, storepwd);
	}

	@Override
	public void saveuser(String userId) {
		Integer storeId = Integer.valueOf(userId);
		String _userId = "pa" + String.format("%05d", storeId);
		User user = userDAO.getUserById(_userId);
		if (user == null) {
			user = new User();
			user.setUserId(_userId);

			user.setPassword(Md5Util.md5AsLowerHex("pa12345678"));
			user.setStoreId(storeId);
			Store store = basicInfoService.getStoreById(storeId);
			String phoneString = store.getTakeExpressPhone();
			if (phoneString == null || "".equals(phoneString)) {
				user.setTelPhone(store.getPhone());
			} else {
				user.setTelPhone(store.getTakeExpressPhone());
			}
			user.setShortAddress(store.getCollegeName());
			user.setUserName(store.getOwner());
			user.setContainsShelf(0);
			user.setWebVersion("2.0");
			/*
			 * 0 未开 1 已开 2 废弃 3 筹建中
			 */
			user.setStatus(1);
			userDAO.save(user);
		}

		// 创建分拣中心用户
		String _fjUserId = "fj" + String.format("%05d", storeId);
		FjUser fjUser = fjUserService.findFjUserById(_fjUserId);
		if (fjUser == null) {
			fjUser = new FjUser();
			fjUser.setPassword(Md5Util.md5AsLowerHex("123456"));
			fjUser.setStoreids(userId);
			fjUser.setUserid(_fjUserId);
			fjUserService.save(fjUser);
		}
		long tableCount = userDAO.findCountByTableName(userId);
		if (tableCount < 1) {
			// 建表
			userDAO.createTableDelivery(userId);
			userDAO.createTableDeliveryStatus(userId);
			// 触发器
			userDAO.createDeliveryInsertTrigger(userId);
			userDAO.createDeliveryUpdateTrigger(userId);
		}

	}

	public static void main(String[] ages) {

	}

	@Override
	public void createDelivery() {
		// Map<Integer,Store> storeMap=basicInfoService.getAllStore();
		// Map<Integer,Store> storeMap=new HashMap<>();
		String userId = "";
		Integer[] ids = new Integer[] {};

		for (Integer key : ids) {
			System.out.println(key);
			userId = String.valueOf(key);

			long tableCount = userDAO.findCountByTableName(userId);
			if (tableCount < 1) {
				// 建表
				userDAO.createTableDelivery(userId);
				userDAO.createTableDeliveryStatus(userId);
				// 触发器
				userDAO.createDeliveryInsertTrigger(userId);
				userDAO.createDeliveryUpdateTrigger(userId);
				// 移数据

			}

		}
	}

	@Override
	public void createTrigge(int storeId) {
		// 触发器
		String userId = String.valueOf(storeId);
		userDAO.createDeliveryInsertTrigger(userId);
		userDAO.createDeliveryUpdateTrigger(userId);
	}

	/**
	 * 修改短信模板
	 */
	@Override
	public void updateMsgTemplate(String ids) {

	}
	
	@Override
	public void deductBalance(User user, int smsCount, int operatorBusiness) {
		
		// 更新余额
		Date now = new Date();
		BigDecimal consume = calConsume(user.getSmsPrice(), smsCount);
		if (consume.compareTo(BigDecimal.ZERO) > 0) {
			
			//int result = userDAO.deductBalance(consume, user.getUserId());
			String userId=user.getUserId();
			String userName=user.getUserName();
			User temUser=new User();
			int result=0;
			if(isNumeric(userId)){
				result = userDAO.deductBalance(consume, user.getUserId());
			}else{
				result = userDAO.deductBalance(consume, user.getUserName());
			}

			//updateSmsBalance(String.valueOf(user.getSmsBalance().subtract(consume)),user.getUserName());

			if (result > 0) {

				// 更新缓存
				CacheTool cacheTool = CacheTool.initUser();
				cacheTool.saveBean(user.getUserId() + "", userDAO.getUserByIdAndPwd(user.getUserId(), null));
				
				// 插入消费记录
				SmsSendStatistics smsSendStatistics = new SmsSendStatistics();
				smsSendStatistics.setStoreId(user.getStoreId());
				smsSendStatistics.setSendCount(smsCount);
				smsSendStatistics.setSendDate(DateUtil.getDay());
				smsSendStatistics.setConsumeMoney(consume);
				smsSendStatistics.setOperatorBusiness(operatorBusiness);
				smsSendStatistics.setCreateTime(now);
				smsSendStatistics.setUpdateTime(now);
				smsSendStatisticsDao.insert(smsSendStatistics);
			}
		}
	}
	
	private BigDecimal calConsume(BigDecimal smsPrice, int smsCount) {
		if (smsPrice == null) {
			return BigDecimal.ZERO;
		}
		return smsPrice.multiply(new BigDecimal(smsCount));
	}
	
	@Override
	public void addBalance(int storeId, BigDecimal money) {
		String userId = "xm" + String.format("%05d", storeId);
		int result = userDAO.addBalance(money, userId);
		
		if (result > 0) {

			// 更新缓存
			CacheTool cacheTool = CacheTool.initUser();
			cacheTool.saveBean(userId + "", userDAO.getUserByIdAndPwd(userId, null));
		}
	}
	
	@Override
	public int updateSmsPriceConf(String userId, int isSmsFree, String smsPrice) {
		int result = userDAO.updateSmsPriceConf(userId, isSmsFree, smsPrice);
		if (result > 0) {

			// 更新缓存
			CacheTool cacheTool = CacheTool.initUser();
			cacheTool.saveBean(userId + "", userDAO.getUserByIdAndPwd(userId, null));
		}
		return result;
	}

	@Override
	public User getUserByuserId(String userId) {
		return userDAO.getUserByuserId(userId);
	}

	/*public static void updateSmsBalance(String src,String userName) {
		String updateSql = "update sys_user set smsBalance = '" + src + "' where USERNAME = '" + userName + "'";

		Connection conn = null;
		Statement stmt = null;
		try {
			conn = DBUtil.getFhmConnection();
			stmt = conn.createStatement();
			stmt.executeUpdate(updateSql);
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try {
				if (stmt != null) {
					stmt.close();
				}
				if (conn != null) {
					conn.close();
				}
			} catch (SQLException ex) {
				ex.printStackTrace();
			}

		}
	}*/


	@Override
	public int updateOcrPriceConf(String userId, int isOcrFree, String ocrPrice) {
		int result = userDAO.updateOcrPriceConf(userId, isOcrFree, ocrPrice);
		if (result > 0) {

			// 更新缓存
			CacheTool cacheTool = CacheTool.initUser();
			cacheTool.saveBean(userId + "", userDAO.getUserByIdAndPwd(userId, null));
		}
		return result;
	}

	@Override
	public int updateVigilance(String userId,String vigilance) {
		int result = userDAO.updateVigilance(userId, vigilance);
		if (result > 0) {

			// 更新缓存
			CacheTool cacheTool = CacheTool.initUser();
			cacheTool.saveBean(userId + "", userDAO.getUserByIdAndPwd(userId, null));
		}
		return result;
	}

	public static boolean isNumeric(String str) {
		String bigStr;
		try {
			bigStr = new BigDecimal(str).toString();
		} catch (Exception e) {
			return false;//异常 说明包含非数字。
		}
		return true;
	}


	@Override
	public Role getUserAndRoleByName(String userId) throws Exception {
		return (Role) userDAO.findOne("cn.haliaeetus.dao.mapper.UserMapper.getUserAndRoleByUserId", userId);
	}

	@Override
	public User getPhenixUserInfo(PageData pd)throws Exception{
		return (User) userDAO.findOne("cn.haliaeetus.dao.mapper.UserMapper.getPhenixUserInfo", pd);
	}

	/*共配*/
	@Override
	public List<Menu> gongPeilistAllMenu(String  menu_name) throws Exception {
		List<Menu> rl = this.getGongPeiParentMenu(menu_name);
		for(Menu menu : rl){
			List<Menu> subList = this.listSubMenuByParentId(menu.getMENU_ID());
			if(subList.size()>0){
				for(Menu sl : subList){
					List<Menu> sList = this.listSubMenuByParentId(sl.getMENU_ID());
					if(sList.size()>0){
						sl.setSubMenu(sList);
					}
				}
			}
			menu.setSubMenu(subList);
		}
		return rl;
	}



	public List<Menu> listSubMenuByParentId(String  menu_name) throws Exception {
		return  userDAO.findAll("cn.haliaeetus.dao.mapper.UserMapper.listSubMenuByParentId", menu_name);

	}

	public List<Menu> getGongPeiParentMenu(String  menu_name) throws Exception {
		return  userDAO.findAll("cn.haliaeetus.dao.mapper.UserMapper.getGongPeiParentMenu", menu_name);
	}
	@Override
	public User selectByPrimaryKey(String userId){
		return  userDAO.selectByPrimaryKey( userId);
	}

	@Override
	public List<User> selectAllUserByStatus(){
		return userDAO.selectAllUserByStatus();
	}

	@Override
	public List<Menu> getMenuList(String userId, User user) {
		List<Menu> allmenuList = new ArrayList<Menu>();
		try{
			Role role= this.getUserAndRoleByName(userId);
			//Role role = user.getRole();
			String roleRights = role!=null ? role.getRIGHTS() : "";

			allmenuList = this.gongPeilistAllMenu("供配系统");
			if(Tools.notEmpty(roleRights)){
				for(Menu menu : allmenuList){
					menu.setHasMenu(RightsHelper.testRights(roleRights, menu.getMENU_ID()));
					/*if(menu.isHasMenu()){*/
					if(menu.getHasMenu()){
						List<Menu> subMenuList = menu.getSubMenu();
						for(Menu sub : subMenuList){
							sub.setHasMenu(RightsHelper.testRights(roleRights, sub.getMENU_ID()));
							if("d7fe9d0a302f46ffa6210f3057e88e33".equals(role.getROLE_ID())){
								if("209".equals(sub.getMENU_ID())||"210".equals(sub.getMENU_ID())||"211".equals(sub.getMENU_ID())
										||"212".equals(sub.getMENU_ID())||"213".equals(sub.getMENU_ID())||"215".equals(sub.getMENU_ID())||"216".equals(sub.getMENU_ID())||"217".equals(sub.getMENU_ID())){
									sub.setHasMenu(true);
								}
							}
							if(user.getSystemType().equals("1")&&"214".equals(sub.getMENU_ID())){
								sub.setHasMenu(true);
							}
							System.out.println("ccccccccccccccccccc:"+sub.getSubMenu());
							if(sub.getSubMenu()!= null){
								List<Menu> subMenuListTwo = sub.getSubMenu();
								for(Menu subTwo : subMenuListTwo){
									subTwo.setHasMenu(RightsHelper.testRights(roleRights, subTwo.getMENU_ID()));
									if("d7fe9d0a302f46ffa6210f3057e88e33".equals(role.getROLE_ID())) {
										if ("209".equals(subTwo.getMENU_ID()) || "210".equals(subTwo.getMENU_ID()) || "211".equals(subTwo.getMENU_ID())
												|| "212".equals(subTwo.getMENU_ID()) || "213".equals(subTwo.getMENU_ID())|| "215".equals(subTwo.getMENU_ID())||"216".equals(subTwo.getMENU_ID())||"217".equals(subTwo.getMENU_ID())) {
											subTwo.setHasMenu(true);
										}
									}
								}
							}
						}
					}
				}
			}
		}catch (Exception e){
			e.getStackTrace();
		}
		return allmenuList;
	}

	@Override
	public List<User> getUserByStoreId(String storeId) throws Exception{
		Map<String,String> param = new HashMap<>(1);
		param.put("storeId",storeId);
		return userDAO.getUserByStoreId(param);
	}
}
