package cn.haliaeetus.sendmessage.service.sms.impl;

import cn.haliaeetus.sendmessage.bean.alibaba.AliBaBaSmsStatusReportBean;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.haidiao.HaiDiaoSmsResultBean;
import cn.haliaeetus.sendmessage.bean.mengwang.MengWangSmsStatusBean;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.bean.sms.UpSmsInfo;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.dao.SmsRecordDao;
import cn.haliaeetus.sendmessage.framework.util.StringUtil;
import cn.haliaeetus.sendmessage.service.DeliveryService;
import cn.haliaeetus.sendmessage.service.sms.AsyncSmsRecodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName AsyncSmsRecordServiceImpl
 * @Description
 * @Author SunJJ
 * @Date 2020/5/26 14:28
 * @Version V1.0
 **/
@Service
public class AsyncSmsRecordServiceImpl implements AsyncSmsRecodeService {
    @Autowired
    private SmsRecordDao smsRecordDao;

    @Autowired
    private DeliveryService deliveryService;

    @Override
    @Async("taskExecutor")
    public void smsReceiptProcessMengWang(List<MengWangSmsStatusBean> mengWangSmsStatusBeanList) throws Exception {
        if (mengWangSmsStatusBeanList.size() > 0) {
            String createTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
            for (MengWangSmsStatusBean smsStatus : mengWangSmsStatusBeanList) {
                smsStatus.setCreateTime(createTime);
                this.upSmsMsg(UpSmsInfo.builder()
                        .bizId(smsStatus.getMsgid())
                        .platform(Constants.PLATFORM_MENGWANG)
                        .mobile(smsStatus.getMobile())
                        .smsSize(smsStatus.getPktotal())
                        .sendTime(smsStatus.getStime())
                        .receiptTime(smsStatus.getRtime())
                        .success(0 == smsStatus.getStatus())
                        .errorCode(smsStatus.getErrcode())
                        .errorMsg(smsStatus.getErrdesc())
                        .build());
            }
            addMengWangSmsStatusRecord(mengWangSmsStatusBeanList);
        }
    }

    @Override
    @Async("taskExecutor")
    public void smsReceiptProcessHaiDiao(List<HaiDiaoSmsResultBean> haiDiaoSmsResultBeanList) throws Exception {
        for (HaiDiaoSmsResultBean haiDiaoSmsResultBean : haiDiaoSmsResultBeanList) {
            boolean flag = "DELIVRD".equals(haiDiaoSmsResultBean.getStatus());
            this.upSmsMsg(UpSmsInfo.builder()
                    .bizId(haiDiaoSmsResultBean.getMsgid())
                    .platform(Constants.PLATFORM_HAIDIAO)
                    .mobile(haiDiaoSmsResultBean.getPhone())
                    .receiptTime(dealWithTime(haiDiaoSmsResultBean.getDonetime()))
                    .success(flag)
                    .errorCode(haiDiaoSmsResultBean.getStatus())
                    .errorMsg(flag ? "success" : "fail")
                    .build());
        }
    }

    private String dealWithTime(String donetime) {
        return donetime.substring(0,4) + "-" + donetime.substring(4,6) + "-" + donetime.substring(6,8) + " " + donetime.substring(8,10) + ":" + donetime.substring(10,12) + ":" + donetime.substring(12,14);
    }

    @Override
    @Async("taskExecutor")
    public void smsReceiptProcessALBB(List<AliBaBaSmsStatusReportBean> request) throws Exception {
        for (AliBaBaSmsStatusReportBean smsStatus : request) {
            this.upSmsMsg(UpSmsInfo.builder()
                    .bizId(smsStatus.getBiz_id())
                    .platform(Constants.PLATFORM_ALBB)
                    .mobile(smsStatus.getPhone_number())
                    .smsSize(Integer.valueOf(smsStatus.getSms_size()))
                    .sendTime(smsStatus.getSend_time())
                    .receiptTime(smsStatus.getReport_time())
                    .success(smsStatus.getSuccess())
                    .errorCode(smsStatus.getErr_code())
                    .errorMsg(smsStatus.getErr_msg())
                    .build());
        }
    }

    private void upSmsMsg(UpSmsInfo request) throws Exception {
        int status = request.getSuccess() ? 1 : 2;
        String msg = request.getErrorCode();
        if (!StringUtil.isEmpty(request.getErrorMsg())) {
            msg = msg + ":" + request.getErrorMsg();
        }
        smsRecordDao.updateSmsRecordByBizId(msg, request.getPlatform() + request.getBizId(), status, request.getReceiptTime());
        SmsRecord smsRecord = smsRecordDao.getSmsRecordByMsgid(request.getPlatform() + request.getBizId());
        if (request.getPlatform().equals(Constants.PLATFORM_DEFAULT) && !request.getSuccess()) {
            // 发送失败
            List<Integer> ids = new ArrayList<>();
            ids.add(smsRecord.getDeliveryid());
            deliveryService.resendSms(smsRecord.getStoreid(), ids, Constants.PLATFORM_ALBB);
        } else {
            if (smsRecord != null) {
                Delivery delivery = new Delivery();
                delivery.setStoreId(smsRecord.getStoreid());
                delivery.setId(smsRecord.getDeliveryid());
                delivery.setResultInfo(msg);
                delivery.setSmsStatus(status);
                smsRecordDao.updateSmsStatusById(delivery);
            }
        }
    }

    private void addMengWangSmsStatusRecord(List<MengWangSmsStatusBean> mengWangSmsStatusBeanList) {
        smsRecordDao.addMengWangSmsStatusInfo(mengWangSmsStatusBeanList);
    }
}
