package cn.haliaeetus.sendmessage.service.sms.impl;

import cn.haliaeetus.sendmessage.bean.MqBean;
import cn.haliaeetus.sendmessage.bean.api.ApiLog;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.mengwang.MengWangSmsStatusBean;
import cn.haliaeetus.sendmessage.bean.sms.SmsPurchaseRecord;
import cn.haliaeetus.sendmessage.bean.sms.SmsRecord;
import cn.haliaeetus.sendmessage.bean.sms.UpSmsInfo;
import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.dao.SmsRecordDao;
import cn.haliaeetus.sendmessage.framework.util.DateUtil;
import cn.haliaeetus.sendmessage.framework.util.StringUtil;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.config.RabbitConfig;
import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 短信发送记录Service
 */
@Service
@Slf4j
public class SmsRecordServiceImpl implements SmsRecordService {
	@Autowired
	private SmsRecordDao smsRecordDao;

	@Autowired
	private BasicInfoService basicInfoService;

	@Autowired
	private RabbitConfig rabbitConfig;

	@Override
	public int save(SmsRecord smsRecord) {
		return smsRecordDao.insert(smsRecord);
	}

	@Override
	public List<SmsRecord> getFailSmsRecordsOfStoreToday(int storeId) {
		Date nowDate = new Date();
		String beginTime = DateUtil.dateFormat(nowDate) + " 00:00:00";
		String endTime = DateUtil.dateFormat(nowDate) + " 23:59:59";

		return smsRecordDao.getFailSmsRecordsOfStoreToday(storeId, Constants.SMS_SEND_STATUS_FAIL,
				Constants.SMS_SEND_STATUS_UNKNOWN, beginTime, endTime);
	}
	@Override
	public long countSmsRecordsOfStoreToday(int storeId, Integer status) {
		Date nowDate = new Date();
		String beginTime = DateUtil.dateFormat(nowDate) + " 00:00:00";
		String endTime = DateUtil.dateFormat(nowDate) + " 23:59:59";
		
		return smsRecordDao.countByStoreIdAndStatusAndTime(storeId, status, beginTime, endTime);
	}

	@Override
	public SmsRecord getSmsRecordByMsgid(String msgid) {
		// TODO Auto-generated method stub
		return smsRecordDao.getSmsRecordByMsgid(msgid);
	}

	@Override
	public void update(SmsRecord smsRecord) {
		// TODO Auto-generated method stub
		SmsRecord smsRecord1 = smsRecordDao.findById(smsRecord.getId());
		if(!smsRecord1.getStatus().equals(smsRecord.getStatus()) && smsRecord.getStatus() != null){
			Delivery delivery = new Delivery();
			delivery.setId(smsRecord1.getDeliveryid());
			delivery.setSmsStatus(smsRecord.getStatus());
			delivery.setSendSuccessType(smsRecord.getSendSuccessType());
			delivery.setStoreId(smsRecord1.getStoreid());
            delivery.setResultInfo(smsRecord.getResultInfo());
			smsRecordDao.updateSmsStatusById(delivery);
		}
		smsRecordDao.update(smsRecord);
	}

	@Override
	public SmsRecord getSmsRecord(SmsRecord smsRecord) {
		// TODO Auto-generated method stub
		return smsRecordDao.getSmsRecord(smsRecord);
	}

	@Override
	public List<SmsRecord> getSendSmsList(String keyword, String type) {
		Map<String, Object> params = new HashMap<>();
		params.put("keyword", keyword);
		params.put("type", type);
		return smsRecordDao.getSendSmsList(params);
	}

	@Override
	public List<SmsRecord> getSendSmsListHis(String keyword, String type) {
		Map<String, Object> params = new HashMap<>();
		params.put("keyword", keyword);
		params.put("type", type);
		//return historysmsRecordDao.getSendSmsList(params);
		return null;
	}
	@Override
	public int savePurchaseRecord(SmsPurchaseRecord smsPurchaseRecord) {
		return smsRecordDao.savePurchaserecord(smsPurchaseRecord);
	}
	@Override
	public void updatePurchaseRecord(SmsPurchaseRecord SmsPurchaseRecord){
		smsRecordDao.updatePurchaseRecord(SmsPurchaseRecord);
	}
	@Override
	public SmsPurchaseRecord getRecordByOutTradeNo(SmsPurchaseRecord SmsPurchaseRecord){
		return smsRecordDao.getRecordByOutTradeNo(SmsPurchaseRecord);
	}


	@Override
	public List<SmsRecord> getSmsCenterList(int storeId ,Timestamp beginTime, Timestamp endTime, int status ) {
		Map<String, Object> params = new HashMap<>();
		params.put("storeId", storeId);
		params.put("beginTime", beginTime);
		params.put("endTime", endTime);
		if(status!=-1){
			params.put("status", status);
		}
		return smsRecordDao.getSmsCenterList(params);
	}

	@Autowired
	private AmqpTemplate rabbitTemplate;
	/**
	 * 重新发送mq
	 * @param msg
	 */
	@Override
	public void send(String msg, Long mqRetryTime, String queue) {
		//获取延迟发送时间
		log.info("mq传入：{}",msg);
		rabbitTemplate.convertAndSend(queue, msg, message -> {
			message.getMessageProperties().setHeader("delay", mqRetryTime);
			return message;
		});
	}

	@Override
	public void installApiLog(ApiLog log){
		rabbitTemplate.convertAndSend(rabbitConfig.getApiLogQueueName(), JSON.toJSONString(log));
	}

	@Override
	public void installApiLogMq(String jsonObject, String queueName) {
		this.installApiLog(ApiLog.builder()
				.requestMsg(jsonObject)
				.responseMsg("")
				.url(queueName)
				.createTime(new Date())
				.requestType(2)
				.platform("RABBITMQ")
				.build());
	}
	/**
	 * 错误处理重试机制
	 * @param headers
	 * @param channel
	 * @param
	 */
	@Override
	public <T extends MqBean> void msgErrorHandle(@Headers Map<String, Object> headers, Channel channel, T t, String queueName) {
		try {
			Integer count = t.getCountMq()==null?0:t.getCountMq();
			Integer mqRetryCount = Integer.parseInt(basicInfoService.getConfigurationByCode("mqRetryCount").getOValue1());
			Long mqRetryTime = Long.parseLong(basicInfoService.getConfigurationByCode("mqRetryTime").getOValue1());
			if(mqRetryCount==null||mqRetryCount==0){
				//这段代码表示，这次消息，我已经接受并消费掉了，不会再重复发送消费
				channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
			}else{
				if(mqRetryCount.compareTo(count) > 0){
					//重新发送
					t.setCountMq(++count);
					String json = new Gson().toJson(t);
					log.info("重新发送于"+mqRetryTime+"秒后，次数为："+count+"------>:"+json);
					this.send(json,mqRetryTime,queueName);
				}
				//重试次数大于或等于mqRetryCount将失败，这段代码表示，这次消息，我已经接受并消费掉了，不会再重复发送消费
				channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
			}
		}catch (IOException e1){
			log.error("rabbitMq连接失败");
			e1.printStackTrace();
			try {
				//重试次数大于或等于mqRetryCount将失败，这段代码表示，这次消息，我已经接受并消费掉了，不会再重复发送消费
				channel.basicAck((Long)headers.get(AmqpHeaders.DELIVERY_TAG), false);
			}catch (Exception e2) {
			    e2.printStackTrace();
			}
		}

	}

	@Override
	public void upSmsMsg(UpSmsInfo request) throws Exception {
		int status = 2;
		if (request.getSuccess()) {
			status = 1;
		}
		String msg = request.getErrorCode();
		if(!StringUtil.isEmpty(request.getErrorMsg())){
			msg = msg + ":" + request.getErrorMsg();
		}
		smsRecordDao.updateSmsRecordByBizId(msg, request.getPlatform() + request.getBizId(), status, request.getReceiptTime());
		SmsRecord smsRecord = smsRecordDao.getSmsRecordByMsgid(request.getPlatform() + request.getBizId());
		if (smsRecord != null) {
			Delivery delivery = new Delivery();
			delivery.setStoreId(smsRecord.getStoreid());
			delivery.setId(smsRecord.getDeliveryid());
			delivery.setResultInfo(msg);
			delivery.setSmsStatus(status);
			smsRecordDao.updateSmsStatusById(delivery);
		}
	}

	@Override
	public void addMengWangSmsStatusInfo(List<MengWangSmsStatusBean> mengWangSmsStatusBeanList) {
		smsRecordDao.addMengWangSmsStatusInfo(mengWangSmsStatusBeanList);
	}
}