package cn.haliaeetus.sendmessage.utils;

import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>反射工具类</p>
 * 
 * @author sqj
 * @since 1.0
 * @version 1.0
 */
public class ReflectionUtil {
	/**
	 * 获取某个类的，字段名以某个前缀开头的，值的集合
	 * @param clazz 类型对象
	 * @param prefix 字段名前缀
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public static List<Object> getFieldValuesByPrefix(Class clazz, String prefix) {
		if (clazz == null || !StringUtils.hasText(prefix)) {
			return null;
		}
		
		List<Object> values = new ArrayList<Object>();
		Field[] fields = clazz.getDeclaredFields();
		if (fields != null && fields.length > 0) {
			for (Field field : fields) {
				if (field.getName().startsWith(prefix)) {
					try {
						values.add(field.get(field.getName()));
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}
		}
		return values;
	}
	
	/**
	 * 通过字段名获取字段值
	 * @param clazz 类型对象
	 * @param name 字段名
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public static Object getFieldValueByName(Class clazz, String name) {
		if (clazz == null || !StringUtils.hasText(name)) {
			return null;
		}
		
		Field[] fields = clazz.getDeclaredFields();
		if (fields == null || fields.length == 0) {
			return null;
		}
		
		for (Field field : fields) {
			if (field.getName().equals(name)) {
				try {
					return field.get(field.getName());
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}
		
		return null;
	}
}