package cn.haliaeetus.sendmessage.utils;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.groups.Default;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @ClassName ValidationUtils
 * @Description
 * @Author SunJJ
 * @Date 2020/5/21 15:30
 * @Version V1.0
 **/
public class ValidationUtils {
    private static Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    public static <T> ValidationResult validate(T var) {
        return validate(var, Default.class);
    }

    public static <T> ValidationResult validate(T var1, Class... var2) {
        ValidationResult result = new ValidationResult();
        Set<ConstraintViolation<T>> set = validator.validate(var1, var2);
        if (set != null && set.size() != 0) {
            result.hasErrors = true;
            Map<String, String> errorMsg = new HashMap<>();
            for (ConstraintViolation<T> cv : set) {
                errorMsg.put(cv.getPropertyPath().toString(), cv.getMessage());
            }
            result.errorMsg = errorMsg;
        }
        return result;
    }


    public static <T> ValidationResult validateProperty(T obj, String propertyName) {
        ValidationResult result = new ValidationResult();
        Set<ConstraintViolation<T>> set = validator.validateProperty(obj, propertyName, Default.class);
        if (set != null && set.size() != 0) {
            result.hasErrors = true;
            Map<String, String> errorMsg = new HashMap<String, String>();
            for (ConstraintViolation<T> cv : set) {
                errorMsg.put(propertyName, cv.getMessage());
            }
            result.errorMsg = errorMsg;
        }
        return result;
    }

    public static class ValidationResult {
        // 校验结果是否有错
        public boolean hasErrors;
        // 校验错误信息
        public Map<String, String> errorMsg;
    }

}
