package cn.haliaeetus.sendmessage.utils.rufengda;

import cn.haliaeetus.sendmessage.controller.util.APIConfig;
import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.security.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;

public class RSAUtils {

	/**
	 * 公钥Key获取方法
	 * 
	 * @return
	 */
	public static PublicKey getPublicKey() {
		byte[] modulusBytes;
		PublicKey pubKey = null;
		try {
			modulusBytes = Base64.decode(APIConfig.getRufengdaRsaSignModulePuk()); // 公钥module节点
			byte[] exponentBytes = Base64.decode(APIConfig.getRufengdaRsaSignExponentPublic()); // 公钥Exponent节点
			BigInteger modulus = new BigInteger(1, modulusBytes);
			BigInteger exponent = new BigInteger(1, exponentBytes);
			RSAPublicKeySpec rsaPubKey = new RSAPublicKeySpec(modulus, exponent);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			pubKey = fact.generatePublic(rsaPubKey);
		} catch (InvalidKeySpecException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return pubKey;
	}

	/**
	 * 私钥Key获取方法
	 * 
	 * @return
	 * @throws Base64DecodingException
	 */
	public static PrivateKey getPrivateKey() {
		byte[] modulusBytes;
		PrivateKey priKey = null;
		try {
			modulusBytes = Base64.decode(APIConfig.getRufengdaRsaSignModulePrk()); // 私钥module节点
			byte[] exponentBytes = Base64.decode(APIConfig.getRufengdaRsaSignExponetPrivate()); // 私钥 D节点
			BigInteger modulus = new BigInteger(1, modulusBytes);
			BigInteger exponent = new BigInteger(1, exponentBytes);
			RSAPrivateKeySpec rsaPriKey = new RSAPrivateKeySpec(modulus,
					exponent);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			priKey = fact.generatePrivate(rsaPriKey);
		} catch (InvalidKeySpecException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return priKey;
	}

	/**
	 * RSA 签名方法 私钥签名
	 * 
	 * @param plainText
	 *            要签名的数据
	 * @return 签名后的密文
	 * @throws NoSuchAlgorithmException
	 * @throws UnsupportedEncodingException
	 */
	public static String sign(String plainText)
			throws NoSuchAlgorithmException, UnsupportedEncodingException {
		// 明文MD5加密转Base64再签名
		MessageDigest md5 = MessageDigest.getInstance("MD5");
		BASE64Encoder base64en = new BASE64Encoder();
		plainText = base64en.encode(md5.digest(plainText.getBytes("GB2312")));
		PrivateKey key = getPrivateKey();
		java.security.Signature sign;
		try {
			sign = java.security.Signature.getInstance("SHA1withRSA");
			sign.initSign(key);
			sign.update(plainText.getBytes());
			return base64en.encode(sign.sign());
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (SignatureException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * RSA验证方法，公钥验证
	 * 
	 * @param plainText
	 *            明文
	 * @param signed
	 * @return
	 * @throws IOException
	 * @throws NoSuchAlgorithmException
	 */
	public static boolean verify(String plainText, String signed)
			throws IOException, NoSuchAlgorithmException {

		// 明文MD5加密转Base64再签名
		MessageDigest md5 = MessageDigest.getInstance("MD5");
		BASE64Encoder base64en = new BASE64Encoder();
		plainText = base64en.encode(md5.digest(plainText.getBytes("GB2312")));
		PublicKey key = getPublicKey();
		boolean ret = false;
		java.security.Signature signatureChecker;
		try {
			signatureChecker = java.security.Signature.getInstance("SHA1withRSA");
			signatureChecker.initVerify(key);
			BASE64Decoder base64Decoder = new BASE64Decoder();
			byte[] decodebase64Sign = base64Decoder.decodeBufferToByteBuffer(signed).array();
			signatureChecker.update(plainText.getBytes());
			if (signatureChecker.verify(decodebase64Sign)) {
				ret = true;
			}
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (SignatureException e) {
			e.printStackTrace();
		}

		return ret;
	}
}
